@@ -29,17 +29,20 @@ $set 2 #BaseDisplay
 $set 3 #Common
 
 1 Alfa
-2 Gem automatisk
+2 Skjul automatisk
 10 menu
 11 stil
+12 Standarder
 13 Slet fra
 14 Fejl
 15 Fluxbox version
-17 SVN Revision
+16 Maksimer over
+17 GIT Revision
 18 Synlig
 
 $set 4 #Configmenu
 
+1 Antialias
 2 Hv automatisk
 3 Klik hver
 4 Klik for fokus
@@ -65,22 +68,36 @@ $set 6 #FbTkError
 
 1 Advarsel! Hjde > 3200, stter hjde = 3200
 2 Advarsel! Bredde > 3200, stter bredde = 3200
+3 Advarsel: Kan ikke indlse reserveskrifttype
 4 Allokeringsfejl.
 5 Fejl i oprettelse af pixmap
+6 Kunne ikke oprette XImage (`Xbillede')
 7 Kunne ikke lse.
+8 Advarsel: Fandt ikke noget gyldigt format for alfa.
+9 Advarsel: Kunne ikke oprette alfa-billede.
+10 Advarsel: Kunne ikke oprette alfa-pixmap.
+11 Kunne ikke finde format for skrm(%d)
 12 Lbet tr for hukommelse
 13 Lbet tr for hukommelse under allokering af bl buffer.
 14 Lbet tr for hukommelse under allokering af grn buffer.
 15 Lbet tr for hukommelse under allokering af rd buffer.
+16 Kunne ikke lse tema-object
+17 Uunderstttet visual
+18 Stter standardvrdi
 
 $set 7 #Fluxbox
 
-1 rc filnavn ugyldigt.
-4 Kunne ikke indlse databasen
+1 rc-filnavn ugyldigt!
+2 Kunne ikke indlse gruppefilen
+3 Kunne ikke indlse databasen
+4 Prver igen med
 6 Kan ikke oprette mappen %s
-8 Fejl: kan ikke forst det regulre udtryk
+7 Kunne ikke finde skrme til hndtering.\nSrg for at ikke at ha' en anden window manager krende.
+8 Fejl: kan ikke parse regulrt udtryk
 9 Fatal fejl: der kan kun vre en instans af klassen fluxbox.
+10 Kunne ikke forbinde til X-serveren.\nSrg for at starte X fr du starter Fluxbox.
 11 Advarsel: X-serveren understtter ikke lokale indstillinger (locale)
+12 Advarsel: Kan ikke stte `locale'-modifikatorer
 
 $set 8 #Gnome
 
@@ -106,7 +123,7 @@ $set 10 #Menu
 $set 11 #Remember
 
 1 Dekorationer
-2 Dimensioner
+2 Strrelse
 3 G til skriverbord
 4 Lag
 5 Husk...
@@ -114,11 +131,15 @@ $set 11 #Remember
 7 Gem ved lukning
 8 Kun titelbjlke
 9 P alle skriveborde
+10 Ukendt apps ngle
 11 Skrivebord
 
 $set 12 #Screen
 
-2 W: %4d x H: %4d
+1 BScreen::BScreen: der opstod en fejl under foresprgsel til X server.\n	en anden window manager krer allerede p display
+2 B: %4d x H: %4d
+3 BScreen::BScreen: hndterer skrm %d med visual 0x%lx, dybde %d\n
+4 B: %04d x H: %04d
 
 $set 13 #Slit
 
@@ -132,11 +153,12 @@ $set 13 #Slit
 $set 14 #Toolbar
 
 1 Rediger skrivebordets navn
-3 Alle vinduer
-4 Ikoner
+2 Vinduer vist i vjktjsbjlken
+3 Alle
+4 Minimerede
 5 Ingen
-6 Skrivebord
-7 Skrivebordsikoner
+6 P skrivebord
+7 Minimerede p skrivebord
 8 Vrktjsbjlkelag
 10 Vrktjsbjlkens placering
 11 Vrktjsbjlke
@@ -144,6 +166,8 @@ $set 14 #Toolbar
 14 Ur: 12 timer
 15 Rediger tidsformat
 16 Vis billeder
+17 Ikke-minimerede
+18 Ikke-minimerede p skrivebord
 
 $set 15 #Window
 
@@ -198,7 +222,7 @@ $set 19 #main
 -log <filename>\t\t\tlog output til fil.\n\
 -help\t\t\t\tvis denne hjlp og afslut.\n\n
 
-$set 20
+$set 20 #layers
 
 1 Over dok
 2 Nederst
