@@ -1,5 +1,5 @@
 // ClockTool.cc
-// Copyright (c) 2003 - 2005 Henrik Kinnunen (fluxgen at fluxbox dot org)
+// Copyright (c) 2003 - 2006 Henrik Kinnunen (fluxgen at fluxbox dot org)
 //                and Simon Bowden    (rathnor at users.sourceforge.net)
 //
 // Permission is hereby granted, free of charge, to any person obtaining a
@@ -20,18 +20,17 @@
 // FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
 // DEALINGS IN THE SOFTWARE.
 
-// $Id: ClockTool.cc 4010 2005-05-12 19:58:47Z mathias $
-
 #include "ClockTool.hh"
 
 #include "ToolTheme.hh"
 #include "Screen.hh"
-#include "CommandParser.hh"
+#include "FbTk/CommandParser.hh"
 #include "CommandDialog.hh"
 #include "fluxbox.hh"
 
 #include "FbTk/SimpleCommand.hh"
 #include "FbTk/ImageControl.hh"
+#include "FbTk/TextUtils.hh"
 #include "FbTk/Menu.hh"
 #include "FbTk/MenuItem.hh"
 #include "FbTk/I18n.hh"
@@ -45,24 +44,25 @@
 #else
   #include <time.h>
 #endif
-#include <string>
+#include <sys/time.h>
 #include <typeinfo>
 
 class ClockMenuItem: public FbTk::MenuItem {
 public:
-    explicit ClockMenuItem::ClockMenuItem(ClockTool &tool):
-        FbTk::MenuItem(""), m_tool(tool) { 
+    explicit ClockMenuItem(ClockTool &tool):
+        FbTk::MenuItem(""), m_tool(tool) {
         // determine 12/24 hour format
         _FB_USES_NLS;
         if (m_tool.timeFormat().find("%k") != std::string::npos ||
             m_tool.timeFormat().find("%H") != std::string::npos ||
             m_tool.timeFormat().find("%T") != std::string::npos)
-            setLabel( _FBTEXT(Toolbar, Clock24,   "Clock: 24h",   "set Clockmode to 24h") );
+            setLabel( _FB_XTEXT(Toolbar, Clock24,   "Clock: 24h",   "set Clockmode to 24h") );
         else
-            setLabel( _FBTEXT(Toolbar, Clock12,   "Clock: 12h",   "set Clockmode to 12h") );
+            setLabel( _FB_XTEXT(Toolbar, Clock12,   "Clock: 12h",   "set Clockmode to 12h") );
+        setCloseOnClick(false);
     }
 
-    void click(int button, int time) {
+    void click(int button, int time, unsigned int mods) {
         std::string newformat = m_tool.timeFormat();
         size_t pos = newformat.find("%k");
         std::string newstr;
@@ -76,7 +76,7 @@ public:
             newstr = "%I";
         else if ((pos = newformat.find("%T")) != std::string::npos)
             newstr = "%r";
-        
+
         // 12 hour
         if (newstr.empty()) {
             clock24hour = false;
@@ -86,9 +86,9 @@ public:
                 newstr = "%H";
             else if ((pos = newformat.find("%r")) != std::string::npos)
                 newstr = "%T";
-            
+
         }
-        
+
         if (!newstr.empty()) {
 
             newformat.replace(pos, 2, newstr);
@@ -100,24 +100,24 @@ public:
                     newformat.erase(pos, 2);
             }
 
-        
+
             m_tool.setTimeFormat(newformat);
 
             if (m_tool.timeFormat().find("%k") != std::string::npos ||
                 m_tool.timeFormat().find("%H") != std::string::npos ||
                 m_tool.timeFormat().find("%T") != std::string::npos)
-                setLabel( _FBTEXT(Toolbar, Clock24,   "Clock: 24h",   "set Clockmode to 24h") );
+                setLabel( _FB_XTEXT(Toolbar, Clock24,   "Clock: 24h",   "set Clockmode to 24h") );
             else
-                setLabel( _FBTEXT(Toolbar, Clock12,   "Clock: 12h",   "set Clockmode to 12h") );
-        
+                setLabel( _FB_XTEXT(Toolbar, Clock12,   "Clock: 12h",   "set Clockmode to 12h") );
+
         } // else some other strange format...so we don't do anything
-        FbTk::MenuItem::click(button, time);
+        FbTk::MenuItem::click(button, time, mods);
     }
 private:
     ClockTool &m_tool;
 };
 
-class EditClockFormatCmd: public FbTk::Command {
+class EditClockFormatCmd: public FbTk::Command<void> {
 public:
     void execute() {
         BScreen *screen = Fluxbox::instance()->mouseScreen();
@@ -125,9 +125,9 @@ public:
             return;
         std::string resourcename = screen->name() + ".strftimeFormat";
 
-        CommandDialog *dialog = new CommandDialog(*screen, "Edit Clock Format", 
+        CommandDialog *dialog = new CommandDialog(*screen, "Edit Clock Format",
                                                   "SetResourceValue " + resourcename + " ");
-        FbTk::RefCount<FbTk::Command> cmd(CommandParser::instance().parseLine("reconfigure"));
+        FbTk::RefCount<FbTk::Command<void> > cmd(FbTk::CommandParser<void>::instance().parse("reconfigure"));
         dialog->setPostCommand(cmd);
         dialog->setText(screen->resourceManager().resourceValue(resourcename));
         dialog->show();
@@ -135,38 +135,46 @@ public:
 };
 
 ClockTool::ClockTool(const FbTk::FbWindow &parent,
-                     ToolTheme &theme, BScreen &screen, FbTk::Menu &menu):
+                     FbTk::ThemeProxy<ToolTheme> &theme, BScreen &screen,
+                     FbTk::Menu &menu):
     ToolbarItem(ToolbarItem::FIXED),
-    m_button(parent, theme.font(), ""),
+    m_button(parent, theme->font(), ""),
     m_theme(theme),
     m_screen(screen),
     m_pixmap(0),
-    m_timeformat(screen.resourceManager(), std::string("%k:%M"), 
-                 screen.name() + ".strftimeFormat", screen.altName() + ".StrftimeFormat") {
+    m_timeformat(screen.resourceManager(), std::string("%k:%M"),
+                 screen.name() + ".strftimeFormat", screen.altName() + ".StrftimeFormat"),
+    m_stringconvertor(FbTk::StringConvertor::ToFbString) {
     // attach signals
     theme.reconfigSig().attach(this);
 
-    _FB_USES_NLS;	
+    std::string time_locale = setlocale(LC_TIME, NULL);
+    size_t pos = time_locale.find('.');
+    if (pos != std::string::npos)
+        time_locale = time_locale.substr(pos+1);
+    if (!time_locale.empty())
+        m_stringconvertor.setSource(time_locale);
 
-    // setup timer to update the graphics each second
-    timeval delay;
-    delay.tv_sec = 1;
-    delay.tv_usec = 0;
-    m_timer.setTimeout(delay);
-    FbTk::RefCount<FbTk::Command> update_graphic(new FbTk::SimpleCommand<ClockTool>(*this, 
+    _FB_USES_NLS;
+
+    // setup timer to check the clock every 0.01 second
+    // if nothing has changed, it wont update the graphics
+    m_timer.setInterval(1);
+    // m_timer.setTimeout(delay); // don't need to set timeout on interval timer
+    FbTk::RefCount<FbTk::Command<void> > update_graphic(new FbTk::SimpleCommand<ClockTool>(*this,
                                                                                     &ClockTool::updateTime));
     m_timer.setCommand(update_graphic);
     m_timer.start();
 
-    m_button.setGC(m_theme.textGC());
+    m_button.setGC(m_theme->textGC());
 
     // setup menu
-    FbTk::RefCount<FbTk::Command> saverc(CommandParser::instance().parseLine("saverc"));
+    FbTk::RefCount<FbTk::Command<void> > saverc(FbTk::CommandParser<void>::instance().parse("saverc"));
     FbTk::MenuItem *item = new ClockMenuItem(*this);
     item->setCommand(saverc);
     menu.insert(item);
-    FbTk::RefCount<FbTk::Command> editformat_cmd(new EditClockFormatCmd());
-    menu.insert(_FBTEXT(Toolbar, ClockEditFormat,   "Edit Clock Format",   "edit Clock Format") , editformat_cmd);
+    FbTk::RefCount<FbTk::Command<void> > editformat_cmd(new EditClockFormatCmd());
+    menu.insert(_FB_XTEXT(Toolbar, ClockEditFormat,   "Edit Clock Format",   "edit Clock Format") , editformat_cmd);
 
 
     update(0);
@@ -212,20 +220,31 @@ void ClockTool::update(FbTk::Subject *subj) {
     updateTime();
 
     // + 2 to make the entire text fit inside
-    std::string text(m_button.text().size() + 2, '0');
+    // we only replace numbers with zeros because everything else should be
+    // relatively static. If we replace all text with zeros then widths of
+    // proportional fonts with some strftime formats will be considerably off.
+    std::string text(m_button.text());
 
-    int new_width = m_theme.font().textWidth(text.c_str(), text.size());
-    if (new_width != m_button.width()) {
-        resize(new_width, m_button.height());
+    int textlen = text.size();
+    for (int i=0; i < textlen; ++i) {
+        if (isdigit(text[i])) // don't bother replacing zeros
+            text[i] = '0';
+    }
+    text.append("00"); // pad
+
+    unsigned int new_width = m_button.width();
+    unsigned int new_height = m_button.height();
+    translateSize(orientation(), new_width, new_height);
+    new_width = m_theme->font().textWidth(text.c_str(), text.size());
+    translateSize(orientation(), new_width, new_height);
+    if (new_width != m_button.width() || new_height != m_button.height()) {
+        resize(new_width, new_height);
         resizeSig().notify();
     }
 
-    if (subj != 0 && typeid(*subj) == typeid(ToolTheme))
-        renderTheme(m_button.alpha());
-
 }
 
-unsigned int ClockTool::borderWidth() const { 
+unsigned int ClockTool::borderWidth() const {
     return m_button.borderWidth();
 }
 
@@ -238,20 +257,33 @@ unsigned int ClockTool::height() const {
 }
 
 void ClockTool::updateTime() {
-
     // update clock
-    time_t the_time = time(0);
+    timeval now;
+    gettimeofday(&now, 0);
+    time_t the_time = now.tv_sec;
 
     if (the_time != -1) {
         char time_string[255];
+        int time_string_len;
         struct tm *time_type = localtime(&the_time);
         if (time_type == 0)
             return;
 
 #ifdef HAVE_STRFTIME
-        if (!strftime(time_string, 255, m_timeformat->c_str(), time_type) || m_button.text() == time_string)
+        time_string_len = strftime(time_string, 255, m_timeformat->c_str(), time_type);
+        if( time_string_len == 0)
             return;
-        m_button.setText(time_string);
+        std::string text = m_stringconvertor.recode(time_string);
+        if (m_button.text() == text)
+            return;
+
+        m_button.setText(text);
+
+        unsigned int new_width = m_theme->font().textWidth(time_string, time_string_len) + 2;
+        if (new_width > m_button.width()) {
+            resize(new_width, m_button.height());
+            resizeSig().notify();
+        }
 #else // dont have strftime so we have to set it to hour:minut
         //        sprintf(time_string, "%d:%d", );
 #endif // HAVE_STRFTIME
@@ -260,33 +292,38 @@ void ClockTool::updateTime() {
 
 // Just change things that affect the size
 void ClockTool::updateSizing() {
-    m_button.setBorderWidth(m_theme.border().width());
-    // resizes if new timeformat 
+    m_button.setBorderWidth(m_theme->border().width());
+    // resizes if new timeformat
     update(0);
 }
 
 void ClockTool::reRender() {
-    if (m_pixmap) 
+    if (m_pixmap)
         m_screen.imageControl().removeImage(m_pixmap);
 
-    if (m_theme.texture().usePixmap()) {
+    if (m_theme->texture().usePixmap()) {
         m_pixmap = m_screen.imageControl().renderImage(width(), height(),
-                                                       m_theme.texture());
+                                                       m_theme->texture(), orientation());
         m_button.setBackgroundPixmap(m_pixmap);
     } else {
         m_pixmap = 0;
-        m_button.setBackgroundColor(m_theme.texture().color());
+        m_button.setBackgroundColor(m_theme->texture().color());
     }
 }
 
 
 void ClockTool::renderTheme(unsigned char alpha) {
     m_button.setAlpha(alpha);
-    m_button.setJustify(m_theme.justify());
+    m_button.setJustify(m_theme->justify());
 
     reRender();
 
-    m_button.setBorderWidth(m_theme.border().width());
-    m_button.setBorderColor(m_theme.border().color());
+    m_button.setBorderWidth(m_theme->border().width());
+    m_button.setBorderColor(m_theme->border().color());
     m_button.clear();
 }
+
+void ClockTool::setOrientation(FbTk::Orientation orient) {
+    m_button.setOrientation(orient);
+    ToolbarItem::setOrientation(orient);
+}
