@@ -1,5 +1,5 @@
 // FbRootWindow.cc
-// Copyright (c) 2003 Henrik Kinnunen (fluxgen<at>users.sourceforge.net)
+// Copyright (c) 2003 - 2006 Henrik Kinnunen (fluxgen at fluxbox dot org)
 //
 // Permission is hereby granted, free of charge, to any person obtaining a
 // copy of this software and associated documentation files (the "Software"),
@@ -19,11 +19,9 @@
 // FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
 // DEALINGS IN THE SOFTWARE.
 
-// $Id: FbRootWindow.cc,v 1.2 2003/05/12 04:29:50 fluxgen Exp $
-
 #include "FbRootWindow.hh"
-#include "App.hh"
 
+#include "FbTk/App.hh"
 #include <X11/Xutil.h>
 
 FbRootWindow::FbRootWindow(int screen_num):
@@ -46,7 +44,8 @@ FbRootWindow::FbRootWindow(int screen_num):
         vinfo_nitems > 0) {
 
         for (int i = 0; i < vinfo_nitems; i++) {
-            if (DefaultDepth(disp, screen_num) < vinfo_return[i].depth)
+            // We can't handle 32-bit visuals just yet (Composite ARGB)
+            if (vinfo_return[i].depth != 32 && DefaultDepth(disp, screen_num) < vinfo_return[i].depth)
                 m_visual = vinfo_return[i].visual;
         }
 
@@ -63,5 +62,5 @@ FbRootWindow::FbRootWindow(int screen_num):
 }
 
 void FbRootWindow::updateGeometry() {
-    FbWindow::updateGeometry();
+    FbTk::FbWindow::updateGeometry();
 }
