@@ -19,8 +19,6 @@
 // FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
 // DEALINGS IN THE SOFTWARE.
 
-// $Id: Color.cc 4199 2006-02-16 06:53:05Z mathias $
-
 #include "Color.hh"
 
 #include "App.hh"
@@ -28,7 +26,10 @@
 #include "I18n.hh"
 
 #include <iostream>
-using namespace std;
+
+using std::cerr;
+using std::endl;
+using std::string;
 
 namespace {
 
@@ -56,9 +57,9 @@ Color::Color(const Color &col_copy):
 }
 
 Color::Color(unsigned short red, unsigned short green, unsigned short blue, int screen):
-    m_red(red),	m_green(green), m_blue(blue), 
+    m_red(red),	m_green(green), m_blue(blue),
     m_pixel(0), m_allocated(false),
-    m_screen(screen) { 
+    m_screen(screen) {
     allocate(red, green, blue, screen);
 }
 
@@ -95,7 +96,7 @@ bool Color::setFromString(const char *color_string, int screen) {
         return false;
 
     setPixel(color.pixel);
-    setRGB(maxValue(color.red), 
+    setRGB(maxValue(color.red),
            maxValue(color.green),
            maxValue(color.blue));
     setAllocated(true);
@@ -107,7 +108,7 @@ bool Color::setFromString(const char *color_string, int screen) {
 bool Color::validColorString(const char *color_string, int screen) {
     XColor color;
     Display *disp = App::instance()->display();
-    Colormap colm = DefaultColormap(disp, screen);   
+    Colormap colm = DefaultColormap(disp, screen);
     // trim white space
     string color_string_tmp = color_string;
     StringUtil::removeFirstWhitespace(color_string_tmp);
@@ -116,11 +117,11 @@ bool Color::validColorString(const char *color_string, int screen) {
     return XParseColor(disp, colm, color_string_tmp.c_str(), &color) != 0;
 }
 
-Color &Color::operator  = (const Color &col_copy) {
+Color &Color::operator = (const Color &col_copy) {
     // check for aliasing
     if (this == &col_copy)
         return *this;
-    
+
     copy(col_copy);
     return *this;
 }
@@ -134,7 +135,7 @@ void Color::free() {
         setPixel(0);
         setRGB(0, 0, 0);
         setAllocated(false);
-    }	
+    }
 }
 
 void Color::copy(const Color &col_copy) {
@@ -145,13 +146,11 @@ void Color::copy(const Color &col_copy) {
         return;
     }
 
-    free();
-		
-    allocate(col_copy.red()*0xFF, 
-             col_copy.green()*0xFF,
-             col_copy.blue()*0xFF,
+    allocate(col_copy.red()*0x101,
+             col_copy.green()*0x101,
+             col_copy.blue()*0x101,
              col_copy.m_screen);
-	
+
 }
 
 void Color::allocate(unsigned short red, unsigned short green, unsigned short blue, int screen) {
@@ -160,21 +159,22 @@ void Color::allocate(unsigned short red, unsigned short green, unsigned short bl
     XColor color;
     // fill xcolor structure
     color.red = red;
-    color.green = green;	
+    color.green = green;
     color.blue = blue;
-	
-	
+
+
     if (!XAllocColor(disp, DefaultColormap(disp, screen), &color)) {
         _FB_USES_NLS;
-        cerr<<"FbTk::Color: "<<_FBTKTEXT(Error, ColorAllocation, "Allocation error.", "XAllocColor failed...")<<endl;
+        cerr<<"FbTk::Color: "<<_FBTK_CONSOLETEXT(Error, ColorAllocation, "Allocation error.", "XAllocColor failed...")<<endl;
     } else {
+        free();
         setRGB(maxValue(color.red),
                maxValue(color.green),
                maxValue(color.blue));
         setPixel(color.pixel);
         setAllocated(true);
     }
-	
+
     m_screen = screen;
 }
 
