@@ -1,5 +1,5 @@
 // Image.cc for FbTk - Fluxbox ToolKit
-// Copyright (c) 2003 Henrik Kinnunen (fluxgen at users.sourceforge.net)
+// Copyright (c) 2003 - 2006 Henrik Kinnunen (fluxgen at fluxbox dot org)
 //
 // Permission is hereby granted, free of charge, to any person obtaining a
 // copy of this software and associated documentation files (the "Software"),
@@ -19,8 +19,6 @@
 // FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
 // DEALINGS IN THE SOFTWARE.
 
-// $Id: Image.cc,v 1.4 2004/01/02 22:01:08 fluxgen Exp $
-
 #include "Image.hh"
 #include "StringUtil.hh"
 
@@ -30,42 +28,79 @@
 
 #ifdef HAVE_XPM
 #include "ImageXPM.hh"
-#endif /// HAVE_XPM
+#endif // HAVE_XPM
+
+#ifdef HAVE_IMLIB2
+#include "ImageImlib2.hh"
+#endif // HAVE_IMLIB2
 
 #include <list>
-#include <iostream>
-using namespace std;
+#include <set>
+
+using std::string;
+using std::list;
+using std::set;
 
 namespace FbTk {
 
 Image::ImageMap Image::s_image_map;
 Image::StringList Image::s_search_paths;
 
-PixmapWithMask *Image::load(const std::string &filename, int screen_num) {
 
+void Image::init() {
+
+// create imagehandlers for their extensions
 #ifdef HAVE_XPM
-    // we must do this because static linkage with libFbTk will not init 
-    // a static autoreg variable for it
-    static ImageXPM xpm;
+    new ImageXPM();
 #endif // HAVE_XPM
+#ifdef HAVE_IMLIB2
+    new ImageImlib2();
+#endif // HAVE_IMLIB2
+}
+
+void Image::shutdown() {
+
+    set<ImageBase*> handlers;
+
+    // one imagehandler could be registered
+    // for more than one type
+    ImageMap::iterator it = s_image_map.begin();
+    ImageMap::iterator it_end = s_image_map.end();
+    for (; it != it_end; it++) {
+        if (it->second)
+            handlers.insert(it->second);
+    }
+
+    // free the unique handlers
+    set<ImageBase*>::iterator handler_it = handlers.begin();
+    set<ImageBase*>::iterator handler_it_end = handlers.end();
+    for(; handler_it != handler_it_end; handler_it++) {
+        delete (*handler_it);
+    }
+
+    s_image_map.clear();
+}
+
+PixmapWithMask *Image::load(const string &filename, int screen_num) {
+
 
     if (filename == "")
         return false;
 
     // determine file ending
-    std::string extension(StringUtil::toUpper(StringUtil::findExtension(filename)));
+    string extension(StringUtil::toUpper(StringUtil::findExtension(filename)));
 
     // valid handle?
-    if (s_image_map[extension] == 0)
+    if (s_image_map.find(extension) == s_image_map.end())
         return false;
-    
+
     // load file
     PixmapWithMask *pm = s_image_map[extension]->load(filename, screen_num);
     // failed?, try different search paths
     if (pm == 0 && s_search_paths.size()) {
         // first we need to get basename of current filename
-        std::string base_filename = StringUtil::basename(filename);
-        std::string path = "";
+        string base_filename = StringUtil::basename(filename);
+        string path = "";
         // append each search path and try to load
         StringList::iterator it = s_search_paths.begin();
         StringList::iterator it_end = s_search_paths.end();
@@ -80,13 +115,13 @@ PixmapWithMask *Image::load(const std::string &filename, int screen_num) {
     return pm;
 }
 
-bool Image::registerType(const std::string &type, ImageBase &base) {
+bool Image::registerType(const string &type, ImageBase &base) {
 
     string ucase_type = StringUtil::toUpper(type);
 
     // not empty and not this base?
     if (s_image_map[ucase_type] != 0 &&
-        s_image_map[ucase_type] != &base) 
+        s_image_map[ucase_type] != &base)
         return false;
     // already registered?
     if (s_image_map[ucase_type] == &base)
@@ -101,7 +136,7 @@ void Image::remove(ImageBase &base) {
     // find and remove all referenses to base
     ImageMap::iterator it = s_image_map.begin();
     ImageMap::iterator it_end = s_image_map.end();
-    std::list<std::string> remove_list;
+    list<string> remove_list;
     for (; it != it_end; ++it) {
         if (it->second == &base)
             remove_list.push_back(it->first);
@@ -113,11 +148,11 @@ void Image::remove(ImageBase &base) {
     }
 }
 
-void Image::addSearchPath(const std::string &search_path) {
+void Image::addSearchPath(const string &search_path) {
     s_search_paths.push_back(search_path);
 }
 
-void Image::removeSearchPath(const std::string &search_path) {
+void Image::removeSearchPath(const string &search_path) {
     s_search_paths.remove(search_path);
 }
 
@@ -125,4 +160,4 @@ void Image::removeAllSearchPaths() {
     s_search_paths.clear();
 }
 
-}; // end namespace FbTk
+} // end namespace FbTk
