@@ -1,5 +1,5 @@
 // KeyUtil.cc for FbTk
-// Copyright (c) 2003 - 2005 Henrik Kinnunen (fluxgen at fluxbox dot org)
+// Copyright (c) 2003 - 2006 Henrik Kinnunen (fluxgen at fluxbox dot org)
 //
 // Permission is hereby granted, free of charge, to any person obtaining a
 // copy of this software and associated documentation files (the "Software"),
@@ -19,17 +19,22 @@
 // FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
 // DEALINGS IN THE SOFTWARE.
 
-// $Id: KeyUtil.cc 3991 2005-05-06 09:22:53Z mathias $
-
 #include "KeyUtil.hh"
 #include "App.hh"
 
+#include <X11/keysym.h>
+
 #include <string>
+#ifdef HAVE_CSTRING
+  #include <cstring>
+#else
+  #include <string.h>
+#endif
 
 namespace {
 
 struct t_modlist{
-    char *str;
+    const char *str;
     unsigned int mask;
     bool operator == (const char *modstr) const {
         return  (strcasecmp(str, modstr) == 0 && mask !=0);
@@ -62,7 +67,7 @@ KeyUtil &KeyUtil::instance() {
 
 
 KeyUtil::KeyUtil()
-    : m_modmap(0)
+    : m_modmap(0), m_numlock(0), m_scrolllock(0)
 {
     init();
 }
@@ -93,9 +98,9 @@ void KeyUtil::loadModmap() {
                     m_modmap->modifiermap[realkey], 0);
 
             switch (ks) {
-            case XK_Caps_Lock:
-                m_capslock = modlist[i].mask;
-                break;
+            // we just want to clean the Lock modifier, not specifically the
+            // XK_Caps_Lock key
+            // the others tend to vary from distro to distro, though
             case XK_Scroll_Lock:
                 m_scrolllock = modlist[i].mask;
                 break;
@@ -112,57 +117,34 @@ void KeyUtil::loadModmap() {
  Grabs a key with the modifier
  and with numlock,capslock and scrollock
 */
-void KeyUtil::grabKey(unsigned int key, unsigned int mod) {
+void KeyUtil::grabKey(unsigned int key, unsigned int mod, Window win) {
     Display *display = App::instance()->display();
-    const unsigned int capsmod = instance().capslock();
     const unsigned int nummod = instance().numlock();
     const unsigned int scrollmod = instance().scrolllock();
-    
-    for (int screen=0; screen<ScreenCount(display); screen++) {
-		
-        Window root = RootWindow(display, screen);
-		
-        XGrabKey(display, key, mod,
-                 root, True,
-                 GrabModeAsync, GrabModeAsync);
-						
-        // Grab with numlock, capslock and scrlock	
 
-        //numlock	
-        XGrabKey(display, key, mod|nummod,
-                 root, True,
-                 GrabModeAsync, GrabModeAsync);		
-        //scrolllock
-        XGrabKey(display, key, mod|scrollmod,
-                 root, True,
-                 GrabModeAsync, GrabModeAsync);	
-        //capslock
-        XGrabKey(display, key, mod|capsmod,
-                 root, True,
-                 GrabModeAsync, GrabModeAsync);
-	
-        //capslock+numlock
-        XGrabKey(display, key, mod|capsmod|nummod,
-                 root, True,
-                 GrabModeAsync, GrabModeAsync);
-
-        //capslock+scrolllock
-        XGrabKey(display, key, mod|capsmod|scrollmod,
-                 root, True,
-                 GrabModeAsync, GrabModeAsync);						
-	
-        //capslock+numlock+scrolllock
-        XGrabKey(display, key, mod|capsmod|scrollmod|nummod,
-                 root, True,
-                 GrabModeAsync, GrabModeAsync);						
-
-        //numlock+scrollLock
-        XGrabKey(display, key, mod|nummod|scrollmod,
-                 root, True,
-                 GrabModeAsync, GrabModeAsync);
-	
+    // Grab with numlock, capslock and scrlock
+    for (int i = 0; i < 8; i++) {
+        XGrabKey(display, key, mod | (i & 1 ? LockMask : 0) |
+                 (i & 2 ? nummod : 0) | (i & 4 ? scrollmod : 0),
+                 win, True, GrabModeAsync, GrabModeAsync);
     }
-			
+
+}
+
+void KeyUtil::grabButton(unsigned int button, unsigned int mod, Window win,
+                         unsigned int event_mask, Cursor cursor) {
+    Display *display = App::instance()->display();
+    const unsigned int nummod = instance().numlock();
+    const unsigned int scrollmod = instance().scrolllock();
+
+    // Grab with numlock, capslock and scrlock
+    for (int i = 0; i < 8; i++) {
+        XGrabButton(display, button, mod | (i & 1 ? LockMask : 0) |
+                    (i & 2 ? nummod : 0) | (i & 4 ? scrollmod : 0),
+                    win, False, event_mask, GrabModeAsync, GrabModeAsync,
+                    None, cursor);
+    }
+
 }
 
 /**
@@ -172,8 +154,10 @@ void KeyUtil::grabKey(unsigned int key, unsigned int mod) {
 unsigned int KeyUtil::getKey(const char *keystr) {
     if (!keystr)
         return 0;
-    return XKeysymToKeycode(App::instance()->display(),
-                            XStringToKeysym(keystr));
+    KeySym sym = XStringToKeysym(keystr);
+    if (sym==NoSymbol)
+        return 0;
+    return XKeysymToKeycode(App::instance()->display(), sym);
 }
 
 
@@ -194,12 +178,14 @@ unsigned int KeyUtil::getModifier(const char *modstr) {
 }
 
 /// Ungrabs the keys
-void KeyUtil::ungrabKeys() {
+void KeyUtil::ungrabKeys(Window win) {
     Display * display = App::instance()->display();
-    for (int screen=0; screen<ScreenCount(display); screen++) {
-        XUngrabKey(display, AnyKey, AnyModifier,
-                   RootWindow(display, screen));		
-    }
+    XUngrabKey(display, AnyKey, AnyModifier, win);
+}
+
+void KeyUtil::ungrabButtons(Window win) {
+    Display * display = App::instance()->display();
+    XUngrabButton(display, AnyButton, AnyModifier, win);
 }
 
 unsigned int KeyUtil::keycodeToModmask(unsigned int keycode) {
