@@ -1,5 +1,5 @@
 // MenuTheme.cc for FbTk
-// Copyright (c) 2002 - 2005 Henrik Kinnunen (fluxgen at fluxbox dot org)
+// Copyright (c) 2002 - 2006 Henrik Kinnunen (fluxgen at fluxbox dot org)
 // 
 // Permission is hereby granted, free of charge, to any person obtaining a
 // copy of this software and associated documentation files (the "Software"),
@@ -19,8 +19,6 @@
 // FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
 // DEALINGS IN THE SOFTWARE.
 
-// $Id: MenuTheme.cc 3954 2005-04-26 01:41:55Z simonb $
-
 #include "MenuTheme.hh"
 
 #include "Color.hh"
@@ -44,6 +42,7 @@ MenuTheme::MenuTheme(int screen_num):
     f_text(*this, "menu.frame.textColor", "Menu.Frame.TextColor"),
     h_text(*this, "menu.hilite.textColor", "Menu.Hilite.TextColor"),
     d_text(*this, "menu.frame.disableColor", "Menu.Frame.DisableColor"),
+    u_text(*this, "menu.frame.underlineColor", "Menu.Frame.UnderlineColor"),
     title(*this, "menu.title", "Menu.Title"),
     frame(*this, "menu.frame", "Menu.Frame"),
     hilite(*this, "menu.hilite", "Menu.Hilite"),
@@ -53,6 +52,7 @@ MenuTheme::MenuTheme(int screen_num):
     titlefont_justify(*this, "menu.title.justify", "Menu.Title.Justify"),
     bullet_pos(*this, "menu.bullet.position", "Menu.Bullet.Position"),
     m_bullet(*this, "menu.bullet", "Menu.Bullet"),
+    m_shapeplace(*this, "menu.roundCorners", "Menu.RoundCorners"),
     m_title_height(*this, "menu.titleHeight", "Menu.TitleHeight"),
     m_item_height(*this, "menu.itemHeight", "Menu.ItemHeight"),
     m_border_width(*this, "menu.borderWidth", "Menu.BorderWidth"),
@@ -67,6 +67,7 @@ MenuTheme::MenuTheme(int screen_num):
     m_display(FbTk::App::instance()->display()),
     t_text_gc(RootWindow(m_display, screen_num)),
     f_text_gc(RootWindow(m_display, screen_num)),
+    u_text_gc(RootWindow(m_display, screen_num)),
     h_text_gc(RootWindow(m_display, screen_num)),
     d_text_gc(RootWindow(m_display, screen_num)),
     hilite_gc(RootWindow(m_display, screen_num)),
@@ -76,7 +77,15 @@ MenuTheme::MenuTheme(int screen_num):
     m_delayclose(0), // no delay as default
     m_real_title_height(*m_title_height),
     m_real_item_height(*m_item_height)
-{ 
+{
+    // set default values
+    *m_border_width = 0;
+    *m_bevel_width = 0;
+    *m_border_width = 0;
+    *m_shapeplace = FbTk::Shape::NONE;
+
+    ThemeManager::instance().loadTheme(*this);
+
     m_real_item_height = std::max(*m_item_height, frameFont().height() + 2*bevelWidth());
     m_real_title_height = std::max(*m_title_height,
                                    titleFont().height() + 2*bevelWidth());
@@ -84,13 +93,9 @@ MenuTheme::MenuTheme(int screen_num):
     m_real_item_height = m_real_item_height == 0 ? 1 : m_real_item_height;
     m_real_title_height = m_real_title_height == 0 ? 1 : m_real_title_height;
 
-    // set default values
-    *m_border_width = 0;
-    *m_bevel_width = 0;
-    *m_border_width = 0;
-
     t_text_gc.setForeground(*t_text);
     f_text_gc.setForeground(*f_text);
+    u_text_gc.setForeground(*u_text);
     h_text_gc.setForeground(*h_text);
     d_text_gc.setForeground(*d_text);
     hilite_gc.setForeground(hilite->color());
@@ -127,6 +132,7 @@ void MenuTheme::reconfigTheme() {
 
     t_text_gc.setForeground(*t_text);
     f_text_gc.setForeground(*f_text);
+    u_text_gc.setForeground(*u_text);
     h_text_gc.setForeground(*h_text);
     d_text_gc.setForeground(*d_text);
     hilite_gc.setForeground(hilite->color());
@@ -171,19 +177,4 @@ void ThemeItem<MenuTheme::BulletType>::load(const std::string *name, const std::
     // do nothing, we don't have anything extra to load
 }
 
-template <>
-void ThemeItem<unsigned int>::setDefaultValue() {
-    m_value = 0;
-}
-
-template <>
-void ThemeItem<unsigned int>::setFromString(const char *str) {
-    sscanf(str, "%d", &m_value);
-}
-
-template <>
-void ThemeItem<unsigned int>::load(const std::string *name, const std::string *altname) {
-}
-
-
-}; // end namespace  FbTk
+} // end namespace  FbTk
