@@ -1,5 +1,5 @@
 // MultiButtonMenuItem.cc for FbTk
-// Copyright (c) 2003 Henrik Kinnunen (fluxgen at users.sourceforge.net)
+// Copyright (c) 2003 - 2006 Henrik Kinnunen (fluxgen at fluxbox dot org)
 //                and Simon Bowden    (rathnor at users.sourceforge.net)
 // 
 // Permission is hereby granted, free of charge, to any person obtaining a
@@ -20,20 +20,20 @@
 // FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
 // DEALINGS IN THE SOFTWARE.
 
-// $Id: MultiButtonMenuItem.cc,v 1.2 2004/01/08 22:07:28 fluxgen Exp $
-
 #include "MultiButtonMenuItem.hh"
+#include "PixmapWithMask.hh"
+#include "Command.hh"
 
 namespace FbTk {
 
-MultiButtonMenuItem::MultiButtonMenuItem(int buttons, const char *label):
+MultiButtonMenuItem::MultiButtonMenuItem(int buttons, const FbString &label):
     MenuItem(label),
     m_button_exe(0),
     m_buttons(buttons) {
     init(buttons);
 }
 
-MultiButtonMenuItem::MultiButtonMenuItem(int buttons, const char *label, Menu *submenu):
+MultiButtonMenuItem::MultiButtonMenuItem(int buttons, const FbString &label, Menu *submenu):
     MenuItem(label, submenu),
     m_button_exe(0),
     m_buttons(buttons) {
@@ -45,13 +45,13 @@ MultiButtonMenuItem::~MultiButtonMenuItem() {
         delete [] m_button_exe;
 }
 
-void MultiButtonMenuItem::setCommand(int button, FbTk::RefCount<FbTk::Command> &cmd) {
+void MultiButtonMenuItem::setCommand(int button, FbTk::RefCount<FbTk::Command<void> > &cmd) {
     if (button <= 0 || button > static_cast<signed>(buttons()) || buttons() == 0)
         return;
     m_button_exe[button - 1] = cmd;
 }
 
-void MultiButtonMenuItem::click(int button, int time) {
+void MultiButtonMenuItem::click(int button, int time, unsigned int mods) {
     if (button <= 0 || button > static_cast<signed>(buttons()) || buttons() == 0)
         return;
 
@@ -66,7 +66,7 @@ void MultiButtonMenuItem::init(int buttons) {
         m_buttons = buttons;
 
     if (m_buttons != 0)
-        m_button_exe = new FbTk::RefCount<FbTk::Command>[m_buttons];
+        m_button_exe = new FbTk::RefCount<FbTk::Command<void> >[m_buttons];
     else
         m_button_exe = 0;
 }
