@@ -19,8 +19,6 @@
 // FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
 // DEALINGS IN THE SOFTWARE.
 
-// $Id: Resource.cc 4833 2007-04-03 17:14:00Z markt $
-
 #include "XrmDatabaseHelper.hh"
 #include "Resource.hh"
 #include "I18n.hh"
@@ -232,4 +230,4 @@ void ResourceManager::unlock() {
     }
 }
 
-}; // end namespace FbTk
+} // end namespace FbTk
