@@ -19,8 +19,6 @@
 // FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
 // DEALINGS IN THE SOFTWARE.
 
-// $Id: SignalHandler.cc 4481 2006-07-01 02:56:46Z markt $
-
 #include "SignalHandler.hh"
 
 namespace FbTk {
@@ -62,13 +60,6 @@ bool SignalHandler::registerHandler(int signum, SignalEventHandler *eh,
     return true;
 }
 
-#ifdef NOT_USED
-void SignalHandler::removeHandler(int signum) {
-    if (signum < NSIG)
-        s_signal_handler[signum] = 0; // clear handler pointer
-}
-#endif
-
 void SignalHandler::handleSignal(int signum) {
     if (signum >= NSIG)
         return;
