@@ -1,5 +1,5 @@
 // TextButton.cc for Fluxbox Window Manager
-// Copyright (c) 2003 - 2005 Henrik Kinnunen (fluxgen at fluxbox dot org)
+// Copyright (c) 2003 - 2006 Henrik Kinnunen (fluxgen at fluxbox dot org)
 //
 // Permission is hereby granted, free of charge, to any person obtaining a
 // copy of this software and associated documentation files (the "Software"),
@@ -19,25 +19,26 @@
 // FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
 // DEALINGS IN THE SOFTWARE.
 
-// $Id: TextButton.cc 3961 2005-04-27 15:15:36Z simonb $
-
 #include "TextButton.hh"
+#include "TextUtils.hh"
 #include "Font.hh"
 #include "GContext.hh"
 
 namespace FbTk {
 
 TextButton::TextButton(const FbTk::FbWindow &parent,
-                       const FbTk::Font &font,
+                       FbTk::Font &font,
                        const std::string &text):
     FbTk::Button(parent, 0, 0, 10, 10),
     m_font(&font),
     m_text(text),
-    m_justify(FbTk::LEFT), m_bevel(1),
+    m_justify(FbTk::LEFT),
+    m_orientation(FbTk::ROT0),
+    m_bevel(1),
     m_left_padding(0),
     m_right_padding(0) {
-    setRenderer(*this);
 
+    setRenderer(*this);
 }
 
 void TextButton::resize(unsigned int width, unsigned int height) {
@@ -60,6 +61,26 @@ void TextButton::setJustify(FbTk::Justify just) {
     m_justify = just;
 }
 
+bool TextButton::setOrientation(FbTk::Orientation orient) {
+    if (orient == m_orientation
+        || !m_font->validOrientation(orient))
+        return false;
+    invalidateBackground();
+
+    if (((m_orientation == FbTk::ROT0 || m_orientation == FbTk::ROT180) &&
+         (orient == FbTk::ROT90 || orient == FbTk::ROT270)) ||
+        ((m_orientation == FbTk::ROT90 || m_orientation == FbTk::ROT270) &&
+         (orient == FbTk::ROT0 || orient == FbTk::ROT180))) {
+        // flip width and height
+        m_orientation = orient;
+        resize(height(), width());
+    } else {
+        m_orientation = orient;
+    }
+
+    return true;
+}
+
 void TextButton::setText(const std::string &text) {
     if (m_text != text) {
         m_text = text;
@@ -68,18 +89,12 @@ void TextButton::setText(const std::string &text) {
     }
 }
 
-void TextButton::setFont(const FbTk::Font &font) {
+void TextButton::setFont(FbTk::Font &font) {
     // no need to set new font if it's the same
     if (&font == m_font)
         return;
     m_font = &font;
-}
-
-/// set bevel and redraw text
-void TextButton::setBevel(int bevel) {
-    if (m_bevel == bevel)
-        return;
-    m_bevel = bevel;
+    font.validOrientation(m_orientation); // load the orientation!
 }
 
 void TextButton::setTextPaddingLeft(unsigned int leftpadding) {
@@ -109,8 +124,9 @@ void TextButton::clearArea(int x, int y,
         drawText(0, 0, this);
 }
 
+
 unsigned int TextButton::textWidth() const {
-    return font().textWidth(text().c_str(), text().size());
+    return font().textWidth(text().data(), text().size());
 }
 
 void TextButton::renderForeground(FbWindow &win, FbDrawable &drawable) {
@@ -122,24 +138,47 @@ void TextButton::drawText(int x_offset, int y_offset, FbDrawable *drawable) {
     unsigned int textlen = text().size();
     // do text alignment
 
-    int align_x = FbTk::doAlignment(width() - x_offset - m_left_padding - m_right_padding,
-                                    bevel(),
-                                    justify(),
-                                    font(),
-                                    text().c_str(), text().size(),
-                                    textlen); // return new text lne
+    unsigned int textw = width(), texth = height();
+    translateSize(m_orientation, textw, texth);
+
+    int align_x = FbTk::doAlignment(textw - x_offset - m_left_padding - m_right_padding,
+                                    bevel(), justify(), font(),
+                                    text().data(), text().size(),
+                                    textlen); // return new text len
 
     // center text by default
-    int center_pos = height()/2 + font().ascent()/2 - 1;
+    int center_pos = texth/2 + font().ascent()/2 - 1;
+
+    int textx = align_x + x_offset + m_left_padding;
+    int texty = center_pos + y_offset;
 
     if (drawable == 0)
         drawable = this;
 
+    // give it ROT0 style coords
+    translateCoords(m_orientation, textx, texty, textw, texth);
+
     font().drawText(*drawable,
                     screenNumber(),
                     gc(), // graphic context
-                    text().c_str(), textlen, // string and string size
-                    align_x + x_offset + m_left_padding, center_pos + y_offset); // position
+                    text().data(), textlen, // string and string size
+                    textx, texty, m_orientation); // position
+}
+
+
+bool TextButton::textExceeds(int x_offset) {
+    
+    unsigned int textlen = text().size();
+    // do text alignment
+
+    unsigned int textw = width(), texth = height();
+    translateSize(m_orientation, textw, texth);
+
+    FbTk::doAlignment(textw - x_offset - m_left_padding - m_right_padding,
+                      bevel(), justify(), font(), text().data(), text().size(),
+                      textlen); // return new text len
+
+    return text().size()>textlen;
     
 }
 
@@ -147,4 +186,4 @@ void TextButton::exposeEvent(XExposeEvent &event) {
     clearArea(event.x, event.y, event.width, event.height, false);
 }
 
-}; // end namespace FbTk
+} // end namespace FbTk
