@@ -1,5 +1,5 @@
 // Texture.hh for Fluxbox Window Manager 
-// Copyright (c) 2002 - 2005 Henrik Kinnunen (fluxgen at fluxbox dot org)
+// Copyright (c) 2002 - 2006 Henrik Kinnunen (fluxgen at fluxbox dot org)
 //
 // from Image.cc for Blackbox - an X11 Window manager
 // Copyright (c) 1997 - 2000 Brad Hughes (bhughes@tcac.net)
@@ -22,8 +22,6 @@
 // FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
 // DEALINGS IN THE SOFTWARE.
 
-// $Id: Texture.cc 3864 2005-01-24 18:02:34Z mathias $
-
 #include "App.hh"
 #include "Texture.hh"
 
@@ -57,9 +55,7 @@ void Texture::setFromString(const char * const texture_str) {
     } else {
         setType(Texture::NONE);
 
-        if (strstr(ts, "solid"))
-            addType(Texture::SOLID);
-        else if (strstr(ts, "gradient")) {
+        if (strstr(ts, "gradient")) {
             addType(Texture::GRADIENT);
             if (strstr(ts, "crossdiagonal"))
                 addType(Texture::CROSSDIAGONAL);
@@ -79,23 +75,26 @@ void Texture::setFromString(const char * const texture_str) {
                 addType(Texture::VERTICAL);
             else
                 addType(Texture::DIAGONAL);
-        } else
+        } else if (strstr(ts, "solid"))
             addType(Texture::SOLID);
+        else
+            addType(Texture::DEFAULT_TEXTURE);
 
         if (strstr(ts, "raised"))
             addType(Texture::RAISED);
         else if (strstr(ts, "sunken"))
             addType(Texture::SUNKEN);
         else if (strstr(ts, "flat"))
-            addType(Texture::FLAT);        
+            addType(Texture::FLAT);
         else
-            addType(Texture::RAISED);
+            addType(Texture::DEFAULT_BEVEL);
 
-        if (! (type() & Texture::FLAT))
+        if (! (type() & Texture::FLAT)) {
             if (strstr(ts, "bevel2"))
                 addType(Texture::BEVEL2);
             else
                 addType(Texture::BEVEL1);
+        }
 
         if (strstr(ts, "invert"))
             addType(Texture::INVERT);
@@ -118,15 +117,15 @@ void Texture::calcHiLoColors(int screen_num) {
     xcol.red = (unsigned int) (m_color.red() +
                                (m_color.red() >> 1));
     if (xcol.red >= 0xff) xcol.red = 0xffff;
-    else xcol.red *= 0xff;
+    else xcol.red *= 0x101;
     xcol.green = (unsigned int) (m_color.green() +
                                  (m_color.green() >> 1));
     if (xcol.green >= 0xff) xcol.green = 0xffff;
-    else xcol.green *= 0xff;
+    else xcol.green *= 0x101;
     xcol.blue = (unsigned int) (m_color.blue() +
                                 (m_color.blue() >> 1));
     if (xcol.blue >= 0xff) xcol.blue = 0xffff;
-    else xcol.blue *= 0xff;
+    else xcol.blue *= 0x101;
 
     if (! XAllocColor(disp, colm, &xcol))
         xcol.pixel = 0;
@@ -135,13 +134,13 @@ void Texture::calcHiLoColors(int screen_num) {
 
     xcol.red =
         (unsigned int) ((m_color.red() >> 2) +
-			(m_color.red() >> 1)) * 0xff;
+			(m_color.red() >> 1)) * 0x101;
     xcol.green =
         (unsigned int) ((m_color.green() >> 2) +
-			(m_color.green() >> 1)) * 0xff;
+			(m_color.green() >> 1)) * 0x101;
     xcol.blue =
         (unsigned int) ((m_color.blue() >> 2) +
-			(m_color.blue() >> 1)) * 0xff;
+			(m_color.blue() >> 1)) * 0x101;
 
     if (! XAllocColor(disp, colm, &xcol))
         xcol.pixel = 0;
@@ -149,4 +148,4 @@ void Texture::calcHiLoColors(int screen_num) {
     m_locolor.setPixel(xcol.pixel);
 }
 
-}; // end namespace FbTk
+} // end namespace FbTk
