@@ -1,5 +1,5 @@
 // ThemeItems.cc for FbTk - Fluxbox ToolKit
-// Copyright (c) 2002 - 2005 Henrik Kinnunen (fluxgen at fluxbox dot org)
+// Copyright (c) 2002 - 2006 Henrik Kinnunen (fluxgen at fluxbox dot org)
 //
 // Permission is hereby granted, free of charge, to any person obtaining a
 // copy of this software and associated documentation files (the "Software"),
@@ -19,13 +19,15 @@
 // FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
 // DEALINGS IN THE SOFTWARE.
 
-// $Id: ThemeItems.cc 3864 2005-01-24 18:02:34Z mathias $
-
-/// @file implements common theme items
+/// @file ThemeItems.cc implements common theme items
 
 #ifndef THEMEITEMS_HH
 #define THEMEITEMS_HH
 
+#ifdef    HAVE_CONFIG_H
+#include "config.h"
+#endif // HAVE_CONFIG_H
+
 #include "Theme.hh"
 #include "Color.hh"
 #include "Texture.hh"
@@ -33,6 +35,7 @@
 #include "GContext.hh"
 #include "PixmapWithMask.hh"
 #include "Image.hh"
+#include "Shape.hh"
 #include "StringUtil.hh"
 
 #include <string>
@@ -41,7 +44,20 @@
 #else
   #include <stdio.h>
 #endif
+#ifdef HAVE_CSTRING
+  #include <cstring>
+#else
+  #include <string.h>
+#endif
+#ifdef HAVE_CSTDLIB
+  #include <cstdlib>
+#else
+  #include <stdlib.h>
+#endif
+
 #include <iostream>
+#include <memory>
+
 namespace FbTk {
 
 using std::string;
@@ -64,7 +80,7 @@ void ThemeItem<string>::setFromString(const char *str) {
 }
 
 template <>
-void ThemeItem<int>::load(const string *name, const string *altname) { }
+void ThemeItem<int>::load(const std::string *name, const std::string *altname) { }
 
 template<>
 void ThemeItem<int>::setDefaultValue() {
@@ -82,25 +98,56 @@ void ThemeItem<int>::setFromString(const char *str) {
         setDefaultValue();
 }
 template<>
-void FbTk::ThemeItem<bool>::load(const std::string *name, const std::string *altname) { }
+void ThemeItem<bool>::load(const std::string *name, const std::string *altname) { }
 
 template<>
-void FbTk::ThemeItem<bool>::setDefaultValue() {
+void ThemeItem<bool>::setDefaultValue() {
     *(*this) = false;
 }
 
 template<>
-void FbTk::ThemeItem<bool>::setFromString(char const *strval) {
+void ThemeItem<bool>::setFromString(char const *strval) {
     if (strcasecmp(strval, "true")==0)
         *(*this) = true;
     else
         *(*this) = false;
 }
 
+template <>
+void ThemeItem<unsigned int>::setDefaultValue() {
+    m_value = 0;
+}
+
+template <>
+void ThemeItem<unsigned int>::setFromString(const char *str) {
+    sscanf(str, "%d", &m_value);
+}
+
+template <>
+void ThemeItem<unsigned int>::load(const std::string *name, const std::string *altname) {
+}
+
 template <>
 void ThemeItem<Font>::setDefaultValue() {
-    if (!m_value.load("fixed")) {
+    if (!m_value.load("__DEFAULT__")) {
         cerr<<"ThemeItem<Font>: Warning! Failed to load default value 'fixed'"<<endl;
+    } else {
+        string effect(ThemeManager::instance().resourceValue(name()+".effect", altName()+".Effect"));
+        if (effect == "halo") {
+            m_value.setHalo(true);
+            Color halo_color(ThemeManager::instance().resourceValue(name()+".halo.color", altName()+".Halo.Color").c_str(), 
+                    theme().screenNum());
+            m_value.setHaloColor(halo_color);
+
+        } else if (effect == "shadow" ) {
+            Color shadow_color(ThemeManager::instance().resourceValue(name()+".shadow.color", altName()+".Shadow.Color").c_str(), 
+                    theme().screenNum());
+            
+            m_value.setShadow(true);
+            m_value.setShadowColor(shadow_color);
+            m_value.setShadowOffX(atoi(ThemeManager::instance().resourceValue(name()+".shadow.x", altName()+".Shadow.X").c_str()));
+            m_value.setShadowOffY(atoi(ThemeManager::instance().resourceValue(name()+".shadow.y", altName()+".Shadow.Y").c_str()));
+        }
     }
 }
 
@@ -110,19 +157,35 @@ void ThemeItem<Font>::setFromString(const char *str) {
     if (str == 0 || m_value.load(str) == false) {
         if (ThemeManager::instance().verbose()) {
             cerr<<"Theme: Error loading font "<<
-                ((m_value.isAntialias() || m_value.utf8()) ? "(" : "")<<
-
-                (m_value.isAntialias() ? "antialias" : "")<<
-                (m_value.utf8() ? " utf8" : "")<<
-
-                ((m_value.isAntialias() || m_value.utf8()) ? ") " : "")<<
+                ((m_value.utf8()) ? "(utf8)" : "")<<
                 "for \""<<name()<<"\" or \""<<altName()<<"\": "<<str<<endl;
 
             cerr<<"Theme: Setting default value"<<endl;
         }
         setDefaultValue();
-    }
+    } else {
+        string effect(ThemeManager::instance().resourceValue(name()+".effect", altName()+".Effect"));
+        if (effect == "halo") {
+            m_value.setHalo(true);
+            Color halo_color(ThemeManager::instance().resourceValue(name()+".halo.color", altName()+".Halo.Color").c_str(), 
+                    theme().screenNum());
+            m_value.setHaloColor(halo_color);
 
+        } else if (effect == "shadow" ) {
+            Color shadow_color(ThemeManager::instance().resourceValue(name()+".shadow.color", altName()+".Shadow.Color").c_str(), 
+                    theme().screenNum());
+            
+            m_value.setShadow(true);
+            m_value.setShadowColor(shadow_color);
+
+            int offset_x = atoi(ThemeManager::instance().resourceValue(name()+".shadow.x", altName()+".Shadow.X").c_str());
+            int offset_y = atoi(ThemeManager::instance().resourceValue(name()+".shadow.y", altName()+".Shadow.Y").c_str());
+            if (offset_x != 0)
+                m_value.setShadowOffX(offset_x);
+            if (offset_y != 0)
+                m_value.setShadowOffY(offset_y);
+        }
+    }
 }
 
 // do nothing
@@ -164,8 +227,8 @@ void ThemeItem<Texture>::load(const string *o_name, const string *o_altname) {
         return;
     }
 
-    std::auto_ptr<PixmapWithMask> pm(Image::load(pixmap_name,
-                                                             m_tm.screenNum()));
+    std::auto_ptr<PixmapWithMask> pm(Image::load(pixmap_name, m_tm.screenNum()));
+
     if (pm.get() == 0) {
         if (ThemeManager::instance().verbose()) {
             cerr<<"Resource("<<m_name+".pixmap"
@@ -179,7 +242,7 @@ void ThemeItem<Texture>::load(const string *o_name, const string *o_altname) {
 
 template <>
 void ThemeItem<Texture>::setDefaultValue() {
-    m_value.setType(Texture::FLAT | Texture::SOLID);
+    m_value.setType(Texture::DEFAULT_BEVEL | Texture::DEFAULT_TEXTURE);
     load(); // one might forget to add line something:  so we try to load something.*:  too
 }
 
@@ -309,7 +372,29 @@ void ThemeItem<GContext::CapStyle>::setFromString(char const *strval) {
 template<>
 void ThemeItem<GContext::CapStyle>::load(const string *name, const string *altname) { }
 
+template <>
+void ThemeItem<Shape::ShapePlace>::load(const string *name, const string *altname) { }
 
+template <>
+void ThemeItem<Shape::ShapePlace>::setDefaultValue() {
+    *(*this) = Shape::NONE;
+}
+
+template <>
+void ThemeItem<Shape::ShapePlace>::setFromString(const char *str) {
+    int places = 0;
+
+    if (StringUtil::strcasestr(str, "topleft") != 0)
+        places |= Shape::TOPLEFT;
+    if (StringUtil::strcasestr(str, "topright") != 0)
+        places |= Shape::TOPRIGHT;
+    if (StringUtil::strcasestr(str, "bottomleft") != 0)
+        places |= Shape::BOTTOMLEFT;
+    if (StringUtil::strcasestr(str, "bottomright") != 0)
+        places |= Shape::BOTTOMRIGHT;
+
+    *(*this) = static_cast<Shape::ShapePlace>(places);
+}
 
 } // end namespace FbTk
 
