@@ -19,8 +19,6 @@
 // FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
 // DEALINGS IN THE SOFTWARE.
 
-// $Id: Transparent.cc 4199 2006-02-16 06:53:05Z mathias $
-
 #include "Transparent.hh"
 #include "App.hh"
 #include "I18n.hh"
@@ -35,7 +33,13 @@
 
 #include <iostream>
 #include <stdio.h>
-using namespace std;
+
+
+#ifdef HAVE_XRENDER
+using std::cerr;
+using std::endl;
+#endif // HAVE_XRENDER
+
 
 namespace {
 #ifdef HAVE_XRENDER
@@ -52,7 +56,9 @@ Picture createAlphaPic(Window drawable, unsigned char alpha) {
                                                   PictFormatDepth | PictFormatAlphaMask,
                                                   &pic_format, 0);
     if (format == 0) {
-        cerr<<"FbTk::Transparent: "<<_FBTKTEXT(Error, NoRenderFormat, "Warning: Failed to find valid format for alpha.", "transparency requires a pict format, can't get one...")<<endl;
+        cerr<<"FbTk::Transparent: "<<_FBTK_CONSOLETEXT(Error, NoRenderFormat,
+                                                       "Warning: Failed to find valid format for alpha.",
+                                                       "transparency requires a pict format, can't get one...")<<endl;
         return 0;
     }
 
@@ -60,7 +66,9 @@ Picture createAlphaPic(Window drawable, unsigned char alpha) {
     Pixmap alpha_pm = XCreatePixmap(disp, drawable,
                                     1, 1, 8);
     if (alpha_pm == 0) {
-        cerr<<"FbTk::Transparent: "<<_FBTKTEXT(Error, NoRenderPixmap, "Warning: Failed to create alpha pixmap.", "XCreatePixmap failed for our transparency pixmap")<<endl;
+        cerr<<"FbTk::Transparent: "<<_FBTK_CONSOLETEXT(Error, NoRenderPixmap,
+                                                       "Warning: Failed to create alpha pixmap.",
+                                                       "XCreatePixmap failed for our transparency pixmap")<<endl;
         return 0;
     }
 
@@ -71,7 +79,9 @@ Picture createAlphaPic(Window drawable, unsigned char alpha) {
                                              format, CPRepeat, &attr);
     if (alpha_pic == 0) {
         XFreePixmap(disp, alpha_pm);
-        cerr<<"FbTk::Transparent: "<<_FBTKTEXT(Error, NoRenderPicture, "Warning: Failed to create alpha picture.", "XRenderCreatePicture failed")<<endl;
+        cerr<<"FbTk::Transparent: "<<_FBTK_CONSOLETEXT(Error, NoRenderPicture,
+                                                       "Warning: Failed to create alpha picture.",
+                                                       "XRenderCreatePicture failed")<<endl;
         return 0;
     }
 
@@ -100,6 +110,7 @@ namespace FbTk {
 bool Transparent::s_init = false;
 bool Transparent::s_render = false;
 bool Transparent::s_composite = false;
+bool Transparent::s_use_composite = false;
 
 void Transparent::init() {
     Display *disp = FbTk::App::instance()->display();
@@ -107,48 +118,35 @@ void Transparent::init() {
     int major_opcode, first_event, first_error;
     if (XQueryExtension(disp, "RENDER",
                         &major_opcode,
-                        &first_event, &first_error) == False) {
-        s_render = false;
-        s_composite = false;
-    } else { // we have RENDER support
+                        &first_event, &first_error)) {
+        // we have XRENDER support
         s_render = true;
 
         if (XQueryExtension(disp, "Composite",
                             &major_opcode,
-                            &first_event, &first_error) == False) {
-            s_composite = false;
-        } else { // we have Composite support
+                            &first_event, &first_error)) {
+            // we have Composite support
             s_composite = true;
+            s_use_composite = true;
         }
     }
     s_init = true;
 }
 
-void Transparent::usePseudoTransparent(bool no_composite) {
-    if (s_composite != no_composite)
-        return;
-
-    s_init = false;
-    init(); // only use render if we have it
-
-    if (no_composite)
-        s_composite = false;
+void Transparent::usePseudoTransparent(bool force) {
+    if (!s_init)
+        init();
+    s_use_composite = (!force && s_composite);
 }
 
 bool Transparent::haveComposite(bool for_real) {
-    if (for_real) {
-        Display *disp = FbTk::App::instance()->display();
-        int major_opcode, first_event, first_error;
-
-        return (XQueryExtension(disp, "Composite",
-                               &major_opcode,
-                               &first_event, &first_error) == True);
-    } else {
-        if (!s_init)
-            init();
+    if (!s_init)
+        init();
 
+    if (for_real)
         return s_composite;
-    }
+    else
+        return s_use_composite;
 }
 
 Transparent::Transparent(Drawable src, Drawable dest, unsigned char alpha, int screen_num):
@@ -234,11 +232,16 @@ void Transparent::setDest(Drawable dest, int screen_num) {
         if (format == 0) {
             _FB_USES_NLS;
             cerr<<"FbTk::Transparent: ";
-            fprintf(stderr, _FBTKTEXT(Error, NoRenderVisualFormat, "Failed to find format for screen(%d)", "XRenderFindVisualFormat failed... include %d for screen number"), screen_num);
-            cerr<<endl;
-        }
-        m_dest_pic = XRenderCreatePicture(disp, dest, format, 0, 0);
+            fprintf(stderr,
+                    _FBTK_CONSOLETEXT(Error, NoRenderVisualFormat,
+                                      "Failed to find format for screen(%d)",
+                                      "XRenderFindVisualFormat failed... include %d for screen number").
+                    c_str(), screen_num);
 
+            cerr<<endl;
+        } else {
+            m_dest_pic = XRenderCreatePicture(disp, dest, format, 0, 0);
+        }
 
     }
     m_dest = dest;
@@ -273,11 +276,15 @@ void Transparent::setSource(Drawable source, int screen_num) {
         if (format == 0) {
             _FB_USES_NLS;
             cerr<<"FbTk::Transparent: ";
-            fprintf(stderr, _FBTKTEXT(Error, NoRenderVisualFormat, "Failed to find format for screen(%d)", "XRenderFindVisualFormat failed... include %d for screen number"), screen_num);
+            fprintf(stderr, _FBTK_CONSOLETEXT(Error, NoRenderVisualFormat,
+                                              "Failed to find format for screen(%d)",
+                                              "XRenderFindVisualFormat failed... include %d for screen number").
+                    c_str(), screen_num);
             cerr<<endl;
+        } else {
+            m_src_pic = XRenderCreatePicture(disp, m_source, format,
+                                             0, 0);
         }
-        m_src_pic = XRenderCreatePicture(disp, m_source, format,
-                                         0, 0);
     }
 
     // recreate new alpha
@@ -328,7 +335,7 @@ void Transparent::freeAlpha() {
     m_alpha = 255;
 }
 
-}; // end namespace FbTk
+} // end namespace FbTk
 
 
 
