@@ -1,5 +1,5 @@
 // XLayer.cc for FbTk - fluxbox toolkit
-// Copyright (c) 2003 Henrik Kinnunen (fluxgen at users.sourceforge.net)
+// Copyright (c) 2003 - 2006 Henrik Kinnunen (fluxgen at fluxbox dot org)
 //                and Simon Bowden    (rathnor at users.sourceforge.net)
 //
 // Permission is hereby granted, free of charge, to any person obtaining a
@@ -20,16 +20,22 @@
 // FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
 // DEALINGS IN THE SOFTWARE.
 
-// $Id: XLayer.cc 3700 2004-11-19 11:37:27Z mathias $
-
 #include "XLayer.hh"
 #include "XLayerItem.hh"
 #include "App.hh"
+#include "FbWindow.hh"
+#include "MultLayers.hh"
 
 #include <iostream>
-using namespace std;
+
+using std::find;
 using namespace FbTk;
 
+#ifdef DEBUG
+using std::cerr;
+using std::endl;
+#endif // DEBUG
+
 XLayer::XLayer(MultLayers &manager, int layernum):
     m_manager(manager), m_layernum(layernum) {
 }
@@ -47,11 +53,9 @@ void XLayer::restack() {
     // each LayerItem can contain several windows
     iterator it = itemList().begin();
     iterator it_end = itemList().end();
-    it = itemList().begin();
-    it_end = itemList().end();
     Window *winlist = new Window[num_windows];
     size_t j=0;
-    
+
     // add all the windows from each item
     for (; it != it_end; ++it) {
         XLayerItem::Windows::const_iterator wit = (*it)->getWindows().begin();
@@ -84,43 +88,29 @@ void XLayer::stackBelowItem(XLayerItem *item, XLayerItem *above) {
     if (!m_manager.isUpdatable())
         return;
 
-    Window *winlist;
-    size_t winnum, size, num = item->numWindows();
-
-    // if there are no windows provided for above us, 
-    // then we must have to go right to the top of the stack
+    // if there are no windows provided for above us,
+    // then we must restack the entire layer
+    // we can't do XRaiseWindow because a restack then causes OverrideRedirect
+    // windows to get pushed to the bottom
     if (!above) { // must need to go right to top
-        if (item->getWindows().front()->window())
-            XRaiseWindow(FbTk::App::instance()->display(), item->getWindows().front()->window());
-
-        // if this XLayerItem has more than one window, 
-        // then we'll stack the rest in under the front one too
-        // our size needs to be the number of windows in the group, since there isn't one above.
-        if (num > 1) {
-            winnum = 0;
-            // stack relative to top one (just raised)
-            size = num;
-            winlist = new Window[size];
-        } else {
-            // we've raised the window, nothing else to do
-            return;
-        }
-    } else {
-        // We do have a window to stack below
-
-        // so we put it on top, and fill the rest of the array with the ones to go below it.
-        winnum = 1;
-        size = num+1;
-        winlist = new Window[size];
-        // assume that above's window exists
-        winlist[0] = above->getWindows().back()->window();
+        restack();
+        return;
     }
 
+    Window *winlist;
+    size_t winnum = 1, size = item->numWindows()+1;
+
+    // We do have a window to stack below
+    // so we put it on top, and fill the rest of the array with the ones to go below it.
+    winlist = new Window[size];
+    // assume that above's window exists
+    winlist[0] = above->getWindows().back()->window();
+
     // fill the rest of the array
     XLayerItem::Windows::iterator it = item->getWindows().begin();
     XLayerItem::Windows::iterator it_end = item->getWindows().end();
     for (; it != it_end; ++it) {
-        if ((*it)->window()) 
+        if ((*it)->window())
             winlist[winnum++] = (*it)->window();
     }
 
@@ -128,7 +118,36 @@ void XLayer::stackBelowItem(XLayerItem *item, XLayerItem *above) {
     XRestackWindows(FbTk::App::instance()->display(), winlist, winnum);
 
     delete [] winlist;
-    
+
+}
+
+// We can't just use Restack here, because it won't do anything if they're
+// already in the same relative order excluding other windows
+void XLayer::alignItem(XLayerItem &item) {
+    if (itemList().front() == &item) {
+        stackBelowItem(&item, m_manager.getLowestItemAboveLayer(m_layernum));
+        return;
+    }
+
+    // Note: some other things effectively assume that the window list is
+    // sorted from highest to lowest
+    // get our item
+    iterator myit = find(itemList().begin(), itemList().end(), &item);
+    iterator it = myit;
+
+    // go to the one above it in our layer (top is front, so we decrement)
+    --it;
+
+    // keep going until we find one that is currently visible to the user
+    while (it != itemList().begin() && !(*it)->visible())
+        --it;
+
+    if (it == itemList().begin() && !(*it)->visible())
+        // reached front item, but it wasn't visible, therefore it was already raised
+        stackBelowItem(&item, m_manager.getLowestItemAboveLayer(m_layernum));
+    else
+        stackBelowItem(&item, *it);
+
 }
 
 XLayer::iterator XLayer::insert(XLayerItem &item, unsigned int pos) {
@@ -137,7 +156,7 @@ XLayer::iterator XLayer::insert(XLayerItem &item, unsigned int pos) {
     if (pos != 0)
         cerr<<__FILE__<<"("<<__LINE__<<"): Insert using non-zero position not valid in XLayer"<<endl;
 #endif // DEBUG
-    
+
     itemList().push_front(&item);
     // restack below next window up
     stackBelowItem(&item, m_manager.getLowestItemAboveLayer(m_layernum));
@@ -155,106 +174,13 @@ void XLayer::remove(XLayerItem &item) {
     }
 }
 
-void XLayer::cycleUp() {
-    // need to find highest visible window, and move it to bottom
-    iterator it = itemList().begin();
-    iterator it_end = itemList().end();
-    while (it != it_end && !(*it)->visible()) 
-        ++it;
-    
-    // if there is something to do
-    if (it != it_end)
-        lower(**it);
-
-}
-
-void XLayer::cycleDown() {
-    // need to find lowest visible window, and move it to top
-    // so use a reverse iterator, and the same logic as cycleUp()
-    reverse_iterator it = itemList().rbegin();
-    reverse_iterator it_end = itemList().rend();
-    while (it != it_end && !(*it)->visible()) 
-        ++it;
-    
-    // if there is something to do
-    if (it != it_end)
-        raise(**it);
-    
-}
-
-void XLayer::stepUp(XLayerItem &item) {
-    // need to find next visible window upwards, and put it above that
-
-    if (&item == itemList().front()) 
-        return; // nothing to do
-
-    // TODO: is there a better way of doing this?
-
-    // get our item
-    iterator myit = std::find(itemList().begin(), itemList().end(), &item);
-    iterator it = myit;
-    
-    // go to the one above it in our layer (top is front, so we decrement)
-    --it;
-
-    // keep going until we find one that is currently visible to the user
-    while (it != itemList().begin() && !(*it)->visible()) 
-        --it;
-    
-    if (it == itemList().begin() && !(*it)->visible()) {
-        // reached front item, but it wasn't visible, therefore it was already raised
-    } else {
-        // it is the next visible item down, we need to be above it.
-        // remove that item from the list and add it back to before it (the one we want to go above)
-
-        itemList().erase(myit);
-        itemList().insert(it, &item);
-
-        // if we've reached the top of the layer, we need to stack below the next one up
-        if (it == itemList().begin()) {
-            stackBelowItem(&item, m_manager.getLowestItemAboveLayer(m_layernum));
-        } else {
-            // otherwise go up one in this layer (i.e. above the one we want to go above) 
-            --it;
-            // and stack below that.
-            stackBelowItem(&item, *it);
-        }
-    }
-}
-
-void XLayer::stepDown(XLayerItem &item) {
-    // need to find next visible window down, and put it below that
-
-    // if we're already the bottom of the layer
-    if (&item == itemList().back()) 
-        return; // nothing to do
-
-    // get our position
-    iterator myit = std::find(itemList().begin(), itemList().end(), &item);
-    iterator it = myit;
-
-    // go one below it (top is front, so we must increment)
-    it++;
-    iterator it_end = itemList().end();
-
-    // keep going down until we find a visible one
-    while (it != it_end && !(*it)->visible()) 
-        it++;
-
-    // if we didn't reach the end, then stack below the
-    // item that we found.
-    if (it != it_end)
-        stackBelowItem(&item, *it);
-    // if we did reach the end, then there are no visible windows, so we don't do anything
-}
-
 void XLayer::raise(XLayerItem &item) {
     // assume it is already in this layer
 
     if (&item == itemList().front())
         return; // nothing to do
 
-    iterator it = std::find(itemList().begin(), itemList().end(), &item);
+    iterator it = find(itemList().begin(), itemList().end(), &item);
     if (it != itemList().end())
         itemList().erase(it);
     else {
@@ -266,7 +192,7 @@ void XLayer::raise(XLayerItem &item) {
 
     itemList().push_front(&item);
     stackBelowItem(&item, m_manager.getLowestItemAboveLayer(m_layernum));
-    
+
 }
 
 void XLayer::tempRaise(XLayerItem &item) {
@@ -275,7 +201,7 @@ void XLayer::tempRaise(XLayerItem &item) {
     if (&item == itemList().front())
         return; // nothing to do
 
-    iterator it = std::find(itemList().begin(), itemList().end(), &item);
+    iterator it = find(itemList().begin(), itemList().end(), &item);
     if (it == itemList().end()) {
 #ifdef DEBUG
         cerr<<__FILE__<<"("<<__LINE__<<"): WARNING: raise on item not in layer["<<m_layernum<<"]"<<endl;
@@ -285,17 +211,17 @@ void XLayer::tempRaise(XLayerItem &item) {
 
     // don't add it back to the top
     stackBelowItem(&item, m_manager.getLowestItemAboveLayer(m_layernum));
-    
+
 }
 
 void XLayer::lower(XLayerItem &item) {
     // assume already in this layer
 
     // is it already the lowest?
-    if (&item == itemList().back()) 
+    if (&item == itemList().back())
         return; // nothing to do
 
-    iterator it = std::find(itemList().begin(), itemList().end(), &item);
+    iterator it = find(itemList().begin(), itemList().end(), &item);
     if (it != itemList().end())
         // remove this item
         itemList().erase(it);
@@ -315,13 +241,13 @@ void XLayer::lower(XLayerItem &item) {
 
     // go up one so we have an object (which must exist, since at least this item is in the layer)
     it--;
-    
+
     // go down another one
     // must exist, otherwise our item must == itemList().back()
-    it--; 
+    it--;
 
     // and restack our window below that one.
-    stackBelowItem(&item, *it); 
+    stackBelowItem(&item, *it);
 }
 
 void XLayer::raiseLayer(XLayerItem &item) {
@@ -338,15 +264,15 @@ void XLayer::moveToLayer(XLayerItem &item, int layernum) {
 
 
 XLayerItem *XLayer::getLowestItem() {
-    if (itemList().empty()) 
+    if (itemList().empty())
         return 0;
-    else 
+    else
         return itemList().back();
 }
 
 XLayerItem *XLayer::getItemBelow(XLayerItem &item) {
     // get our iterator
-    iterator it = std::find(itemList().begin(), itemList().end(), &item);
+    iterator it = find(itemList().begin(), itemList().end(), &item);
 
     // go one lower
     it++;
@@ -360,12 +286,12 @@ XLayerItem *XLayer::getItemBelow(XLayerItem &item) {
 
 XLayerItem *XLayer::getItemAbove(XLayerItem &item) {
     // get our iterator
-    iterator it = std::find(itemList().begin(), itemList().end(), &item);
+    iterator it = find(itemList().begin(), itemList().end(), &item);
 
-    // if this is the beginning (top-most item), do nothing, otherwise give the next one up 
+    // if this is the beginning (top-most item), do nothing, otherwise give the next one up
     // the list (which must be there since we aren't the beginning)
     if (it == itemList().begin())
         return 0;
-    else        
+    else
         return *(--it);
 }
