@@ -1,5 +1,5 @@
 // XLayerItem.cc for FbTk - fluxbox toolkit
-// Copyright (c) 2003 - 2005 Henrik Kinnunen (fluxgen at fluxbox dot org)
+// Copyright (c) 2003 - 2006 Henrik Kinnunen (fluxgen at fluxbox dot org)
 //                and Simon Bowden    (rathnor at users.sourceforge.net)
 //
 // Permission is hereby granted, free of charge, to any person obtaining a
@@ -20,15 +20,13 @@
 // FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
 // DEALINGS IN THE SOFTWARE.
 
-// $Id: XLayerItem.cc 3864 2005-01-24 18:02:34Z mathias $
-
 #include "XLayerItem.hh"
 #include "XLayer.hh"
 
 using namespace FbTk;
 
 XLayerItem::XLayerItem(FbWindow &win, XLayer &layer) :
-    m_layer(&layer), m_layeriterator(0) {
+    m_layer(&layer) {
     m_windows.push_front(&win);
     m_layer->insert(*this);
 }
@@ -60,14 +58,6 @@ void XLayerItem::tempRaise() {
     m_layer->tempRaise(*this); 
 }
 
-void XLayerItem::stepUp() {
-    m_layer->stepUp(*this);
-}
-
-void XLayerItem::stepDown() {
-    m_layer->stepDown(*this);
-}
-
 void XLayerItem::raiseLayer() {
     m_layer->raiseLayer(*this); 
 }
@@ -81,9 +71,10 @@ void XLayerItem::moveToLayer(int layernum) {
 }
 
 void XLayerItem::addWindow(FbWindow &win) {
-  // I'd like to think we can trust ourselves that it won't be added twice...
-  // Otherwise we're always scanning through the list.
-  m_windows.push_back(&win);
+    // I'd like to think we can trust ourselves that it won't be added twice...
+    // Otherwise we're always scanning through the list.
+    m_windows.push_back(&win);
+    m_layer->alignItem(*this);
 }
 
 void XLayerItem::removeWindow(FbWindow &win) {
@@ -91,7 +82,8 @@ void XLayerItem::removeWindow(FbWindow &win) {
   // Otherwise we're always scanning through the list.
 
   XLayerItem::Windows::iterator it = std::find(m_windows.begin(), m_windows.end(), &win);
-  m_windows.erase(it);
+  if (it != m_windows.end())
+      m_windows.erase(it);
 }
 
 void XLayerItem::bringToTop(FbWindow &win) {
