@@ -1,5 +1,5 @@
 // XftFontImp.cc  Xft font implementation for FbTk
-// Copyright (c) 2002 - 2005 Henrik Kinnunen (fluxgen at fluxbox dot org)
+// Copyright (c) 2002 - 2006 Henrik Kinnunen (fluxgen at fluxbox dot org)
 //
 // Permission is hereby granted, free of charge, to any person obtaining a
 // copy of this software and associated documentation files (the "Software"),
@@ -19,53 +19,87 @@
 // FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
 // DEALINGS IN THE SOFTWARE.
 
-//$Id: XftFontImp.cc 3864 2005-01-24 18:02:34Z mathias $
-
 #include "XftFontImp.hh"
 #include "App.hh"
 #include "FbDrawable.hh"
 
+#include <math.h>
+
 #ifdef HAVE_CONFIG_H
 #include "config.h"
 #endif //HAVE_CONFIG_H
 
 namespace FbTk {
 
-XftFontImp::XftFontImp(const char *name, bool utf8):m_xftfont(0),
-                                                    m_utf8mode(utf8) {
+XftFontImp::XftFontImp(const char *name, bool utf8):
+    m_utf8mode(utf8), m_name("") {
+
+    for (int r = ROT0; r <= ROT270; r++) {
+        m_xftfonts[r] = 0;
+        m_xftfonts_loaded[r] = false;
+    }
 
     if (name != 0)
         load(name);
 }
 
 XftFontImp::~XftFontImp() {
-    if (m_xftfont != 0)
-        XftFontClose(App::instance()->display(), m_xftfont);
+    for (int r = ROT0; r <= ROT270; r++) 
+        if (m_xftfonts[r] != 0)
+            XftFontClose(App::instance()->display(), m_xftfonts[r]);
 }
 
 bool XftFontImp::load(const std::string &name) {
     //Note: assumes screen 0 for now, changes on draw if needed
 
     Display *disp = App::instance()->display();
-    XftFont *newxftfont = XftFontOpenName(disp, 0, name.c_str());
 
-    if (newxftfont == 0) { // failed to open font, lets test with XLFD
-        newxftfont = XftFontOpenXlfd(disp, 0, name.c_str());
+    XftFont *newxftfont = XftFontOpenXlfd(disp, 0, name.c_str());
+    if (newxftfont == 0) {
+        newxftfont = XftFontOpenName(disp, 0, name.c_str());
         if (newxftfont == 0)
             return false;
     }
-    // destroy old font and set new
-    if (m_xftfont != 0)
-        XftFontClose(disp, m_xftfont);
+    
+    // destroy all old fonts and set new
+    for (int r = ROT0; r <= ROT270; r++) {
+        m_xftfonts_loaded[r] = false;
+        if (m_xftfonts[r] != 0) {
+            XftFontClose(App::instance()->display(), m_xftfonts[r]);
+            m_xftfonts[r] = 0;
+        }
+    }
 
-    m_xftfont = newxftfont;
+    m_xftfonts[ROT0] = newxftfont;
+    m_xftfonts_loaded[ROT0] = true;
+    m_name = name;
 
     return true;
 }
 
-void XftFontImp::drawText(const FbDrawable &w, int screen, GC gc, const char *text, size_t len, int x, int y) const {
-    if (m_xftfont == 0)
+void XftFontImp::drawText(const FbDrawable &w, int screen, GC gc, const FbString &text, size_t len, int x, int y, FbTk::Orientation orient) {
+
+    if (!validOrientation(orient))
         return;
+
+    // we adjust y slightly so that the baseline is in the right spot
+    // (it is offset one by rotation >=180 degrees)
+    switch (orient) {
+    case ROT0:
+        break;
+    case ROT90:
+        break;
+    case ROT180:
+        x+=1;
+        y+=1;
+        break;
+    case ROT270:
+        y+=1;
+        break;
+    }
+
+    XftFont *font = m_xftfonts[orient];
+
     XftDraw *draw = XftDrawCreate(w.display(),
                                   w.drawable(),
                                   DefaultVisual(w.display(), screen),
@@ -101,15 +135,15 @@ void XftFontImp::drawText(const FbDrawable &w, int screen, GC gc, const char *te
         // if the size is zero we use the XftDrawString8 function instead.
         XGlyphInfo ginfo;
         XftTextExtentsUtf8(w.display(),
-                           m_xftfont,
-                           (XftChar8 *)text, len,
+                           m_xftfonts[ROT0],
+                           (XftChar8 *)text.data(), len,
                            &ginfo);
         if (ginfo.xOff != 0) {
             XftDrawStringUtf8(draw,
                               &xftcolor,
-                              m_xftfont,
+                              font,
                               x, y,
-                              (XftChar8 *)(text), len);
+                              (XftChar8 *)(text.data()), len);
             XftColorFree(w.display(), 
                          DefaultVisual(w.display(), screen),
                          DefaultColormap(w.display(), screen), &xftcolor);
@@ -121,9 +155,9 @@ void XftFontImp::drawText(const FbDrawable &w, int screen, GC gc, const char *te
 
     XftDrawString8(draw,
                    &xftcolor,
-                   m_xftfont,
+                   font,
                    x, y,
-                   (XftChar8 *)(text), len);
+                   (XftChar8 *)(text.data()), len);
 
 
     XftColorFree(w.display(), 
@@ -132,18 +166,21 @@ void XftFontImp::drawText(const FbDrawable &w, int screen, GC gc, const char *te
     XftDrawDestroy(draw);
 }
 
-unsigned int XftFontImp::textWidth(const char * const text, unsigned int len) const {
-    if (m_xftfont == 0)
+unsigned int XftFontImp::textWidth(const FbString &text, unsigned int len) const {
+    if (m_xftfonts[ROT0] == 0)
         return 0;
 
     XGlyphInfo ginfo;
     Display* disp = App::instance()->display();
 
+    XftFont *font = m_xftfonts[ROT0];
+
+
 #ifdef HAVE_XFT_UTF8_STRING
     if (m_utf8mode) {
         XftTextExtentsUtf8(disp,
-                           m_xftfont,
-                           (XftChar8 *)text, len,
+                           font,
+                           (XftChar8 *)text.data(), len,
                            &ginfo);
         if (ginfo.xOff != 0)
             return ginfo.xOff;
@@ -152,21 +189,64 @@ unsigned int XftFontImp::textWidth(const char * const text, unsigned int len) co
     }
 #endif  //HAVE_XFT_UTF8_STRING
 
+    std::string localestr = text;
+    localestr.erase(len, std::string::npos);
+    localestr = FbStringUtil::FbStrToLocale(localestr);
+
     XftTextExtents8(disp,
-                    m_xftfont,
-                    (XftChar8 *)text, len,
+                    font,
+                    (XftChar8 *)localestr.data(), localestr.size(),
                     &ginfo);
 
     return ginfo.xOff;
 }
 
 unsigned int XftFontImp::height() const {
-    if (m_xftfont == 0)
+    if (m_xftfonts[ROT0] == 0)
         return 0;
-    return m_xftfont->height;
+    else
+        return m_xftfonts[ROT0]->height;
     //m_xftfont->ascent + m_xftfont->descent;
     // curiously, fonts seem to have a smaller height, but the "height"
     // is specified within the actual font, so it must be right, right?
 }
 
-}; // end namespace FbTk
+bool XftFontImp::validOrientation(FbTk::Orientation orient) {
+    if (orient == ROT0 || m_xftfonts[orient])
+        return true;
+
+    if (m_xftfonts_loaded[orient])
+        return false; // m_xftfonts is zero here
+
+    if (m_xftfonts[ROT0] == 0)
+        return false;
+
+    m_xftfonts_loaded[orient] = true;
+
+    // otherwise, try to load that orientation
+    // radians is actually anti-clockwise, so we reverse it
+    double radians = -(orient) * 90 * M_PI / 180;
+
+    XftMatrix matrix;
+    XftMatrixInit(&matrix);
+    XftMatrixRotate(&matrix, cos(radians), sin(radians));
+
+    Display *disp = App::instance()->display();
+
+    XftPattern * pattern = XftNameParse(m_name.c_str());
+    XftResult result;
+    pattern = XftFontMatch(disp, 0, pattern, &result);
+    XftPatternAddMatrix(pattern, XFT_MATRIX, &matrix);
+    XftFont * new_font = XftFontOpenPattern(disp, pattern);
+
+    if (new_font == 0)
+        return false;
+
+    m_xftfonts[orient] = new_font;
+
+    return true;
+}
+
+
+
+} // end namespace FbTk
