@@ -1,5 +1,5 @@
 // XmbFontImp.cc for FbTk fluxbox toolkit
-// Copyright (c) 2002 - 2005 Henrik Kinnunen (fluxgen at fluxbox dot org)
+// Copyright (c) 2002 - 2006 Henrik Kinnunen (fluxgen at fluxbox dot org)
 //
 // Permission is hereby granted, free of charge, to any person obtaining a
 // copy of this software and associated documentation files (the "Software"),
@@ -19,13 +19,13 @@
 // FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
 // DEALINGS IN THE SOFTWARE.
 
-// $Id: XmbFontImp.cc 3864 2005-01-24 18:02:34Z mathias $
-
 #include "XmbFontImp.hh"
 
 #include "App.hh"
+#include "TextUtils.hh"
 #include "StringUtil.hh"
-#include "FbDrawable.hh"
+#include "FbPixmap.hh"
+#include "GContext.hh"
 
 #ifdef HAVE_CONFIG_H
 #include "config.h"
@@ -49,14 +49,13 @@
 #else
   #include <stdarg.h>
 #endif
-#include <iostream>
 #ifdef HAVE_CSTRING
   #include <cstring>
 #else
   #include <string.h>
 #endif
 
-using namespace std;
+using std::string;
 
 namespace {
 
@@ -113,16 +112,18 @@ const char *getFontElement(const char *pattern, char *buf, int bufsiz, ...) {
 XFontSet createFontSet(const char *fontname, bool& utf8mode) {
     Display *display = FbTk::App::instance()->display();
     XFontSet fs;
-    const int FONT_ELEMENT_SIZE=50;
-    char **missing, *def = "-";
-    int nmissing, pixel_size = 0, buf_size = 0;
-    char weight[FONT_ELEMENT_SIZE], slant[FONT_ELEMENT_SIZE];
-    std::string orig_locale = "";
+    char **missing;
+    const char *constdef = "-";
+    char *def = const_cast<char *>(constdef);
+    int nmissing;
+    string orig_locale = "";
 
 #ifdef HAVE_SETLOCALE
     if (utf8mode) {
         orig_locale = setlocale(LC_CTYPE, NULL);
-        setlocale(LC_CTYPE, "UTF-8");
+        if (setlocale(LC_CTYPE, "UTF-8") == NULL) {
+            utf8mode = false;
+        }
     }
 #endif // HAVE_SETLOCALE
     fs = XCreateFontSet(display,
@@ -138,62 +139,22 @@ XFontSet createFontSet(const char *fontname, bool& utf8mode) {
 
 #ifdef HAVE_SETLOCALE
     if (! fs) {
-        if (nmissing) XFreeStringList(missing);
+        if (nmissing)
+            XFreeStringList(missing);
 
         setlocale(LC_CTYPE, "C");
         fs = XCreateFontSet(display, fontname,
                             &missing, &nmissing, &def);
         setlocale(LC_CTYPE, orig_locale.c_str());
+        return fs;
     }
-#endif // HAVE_SETLOCALE
-
-    if (fs) {
-        XFontStruct **fontstructs;
-        char **fontnames;
-        XFontsOfFontSet(fs, &fontstructs, &fontnames);
-        fontname = fontnames[0];
-    }
-
-    getFontElement(fontname, weight, FONT_ELEMENT_SIZE,
-                   "-medium-", "-bold-", "-demibold-", "-regular-", NULL);
-    getFontElement(fontname, slant, FONT_ELEMENT_SIZE,
-                   "-r-", "-i-", "-o-", "-ri-", "-ro-", NULL);
-    getFontSize(fontname, &pixel_size);
-
-    if (! strcmp(weight, "*"))
-        strncpy(weight, "medium", FONT_ELEMENT_SIZE);
-    if (! strcmp(slant, "*"))
-        strncpy(slant, "r", FONT_ELEMENT_SIZE);
-    if (pixel_size < 3)
-        pixel_size = 3;
-    else if (pixel_size > 97)
-        pixel_size = 97;
-
-    buf_size = strlen(fontname) + (FONT_ELEMENT_SIZE * 2) + 64;
-    char *pattern2 = new char[buf_size];
-    snprintf(pattern2, buf_size - 1,
-             "%s,"
-             "-*-*-%s-%s-*-*-%d-*-*-*-*-*-*-*,"
-             "-*-*-*-*-*-*-%d-*-*-*-*-*-*-*,*",
-             fontname, weight, slant, pixel_size, pixel_size);
-    fontname = pattern2;
-
-    if (nmissing)
-        XFreeStringList(missing);
-    if (fs)
-        XFreeFontSet(display, fs);
-
-    fs = XCreateFontSet(display, fontname,
-                        &missing, &nmissing, &def);
-    delete [] pattern2;
-
-#ifdef HAVE_SETLOCALE
     if (utf8mode)
         setlocale(LC_CTYPE, orig_locale.c_str());
 #endif // HAVE_SETLOCALE
 
+    // set to false because our strings won't be utf8-happy
     utf8mode = false;
-    
+
     return fs;
 }
 
@@ -210,7 +171,7 @@ XmbFontImp::~XmbFontImp() {
         XFreeFontSet(App::instance()->display(), m_fontset);
 }
 
-bool XmbFontImp::load(const std::string &fontname) {
+bool XmbFontImp::load(const string &fontname) {
     if (fontname.empty())
         return false;
 
@@ -227,41 +188,114 @@ bool XmbFontImp::load(const std::string &fontname) {
     return true;
 }
 
-void XmbFontImp::drawText(const FbDrawable &w, int screen, GC gc, const char *text,
-                          size_t len, int x, int y) const {
+void XmbFontImp::drawText(const FbDrawable &d, int screen, GC main_gc, const FbString &text,
+                          size_t len, int x, int y, FbTk::Orientation orient) {
 
     if (m_fontset == 0)
         return;
 
+    if (orient == ROT0) {
+#ifdef X_HAVE_UTF8_STRING
+        if (m_utf8mode) {
+            Xutf8DrawString(d.display(), d.drawable(), m_fontset,
+                            main_gc, x, y,
+                            text.data(), len);
+        } else
+#endif //X_HAVE_UTF8_STRING
+            {
+                string localestr = text;
+                localestr.erase(len, string::npos);
+                localestr = FbStringUtil::FbStrToLocale(localestr);
+                XmbDrawString(d.display(), d.drawable(), m_fontset,
+                              main_gc, x, y,
+                              localestr.data(), localestr.size());
+            }
+        return;
+    }
+
+    Display *dpy = App::instance()->display();
+
+    int xpos = x, ypos = y;
+    unsigned int w = d.width();
+    unsigned int h = d.height();
+
+    translateSize(orient, w, h);
+    untranslateCoords(orient, xpos, ypos, w, h);
+
+    // not straight forward, we actually draw it elsewhere, then rotate it
+    FbTk::FbPixmap canvas(d.drawable(), w, h, 1);
+
+    // create graphic context for our canvas
+    FbTk::GContext font_gc(canvas);
+    font_gc.setBackground(None);
+    font_gc.setForeground(None);
+
+    XFillRectangle(dpy, canvas.drawable(), font_gc.gc(), 0, 0, canvas.width(), canvas.height());
+    font_gc.setForeground(1);
+
+
 #ifdef X_HAVE_UTF8_STRING
     if (m_utf8mode) {
-        Xutf8DrawString(w.display(), w.drawable(), m_fontset,
-                        gc, x, y,
-                        text, len);
+        Xutf8DrawString(dpy, canvas.drawable(), m_fontset,
+                             font_gc.gc(), xpos, ypos,
+                             text.data(), len);
     } else
 #endif //X_HAVE_UTF8_STRING
     {
-        XmbDrawString(w.display(), w.drawable(), m_fontset,
-                      gc, x, y,
-                      text, len);
+        string localestr = text;
+        localestr.erase(len, string::npos);
+        localestr = FbStringUtil::FbStrToLocale(localestr);
+        XmbDrawString(dpy, canvas.drawable(), m_fontset,
+                           font_gc.gc(), xpos, ypos,
+                           localestr.data(), localestr.size());
     }
+
+    canvas.rotate(orient);
+
+    GC my_gc = XCreateGC(dpy, d.drawable(), 0, 0);
+
+    XCopyGC(dpy, main_gc, GCForeground|GCBackground, my_gc);
+
+    // vertical or upside down
+
+    XSetFillStyle(dpy, my_gc, FillStippled);
+
+    // vertical or upside down
+
+    XSetFillStyle(dpy, my_gc, FillStippled);
+
+    XSetStipple(dpy, my_gc, canvas.drawable());
+
+    XSetTSOrigin(dpy, my_gc, 0, 0);
+
+    XFillRectangle(dpy, d.drawable(), my_gc, 0, 0,
+                   canvas.width(),
+                   canvas.height());
+
+    XFreeGC(dpy, my_gc);
+
 }
 
-unsigned int XmbFontImp::textWidth(const char * const text, unsigned int len) const {
+unsigned int XmbFontImp::textWidth(const FbString &text, unsigned int len) const {
     if (m_fontset == 0)
         return 0;
 
     XRectangle ink, logical;
 #ifdef X_HAVE_UTF8_STRING
     if (m_utf8mode) {
-        Xutf8TextExtents(m_fontset, text, len,
+        Xutf8TextExtents(m_fontset, text.data(), len,
                          &ink, &logical);
         if (logical.width != 0)
             return logical.width;
     }
 #endif // X_HAVE_UTF8_STRING
 
-    XmbTextExtents(m_fontset, text, len,
+    string localestr = text;
+    if (len > localestr.length())
+        len = localestr.length();
+    localestr.erase(len, string::npos);
+    localestr = FbStringUtil::FbStrToLocale(localestr);
+    XmbTextExtents(m_fontset, localestr.data(), localestr.size(),
                    &ink, &logical);
     return logical.width;
 }
@@ -272,5 +306,5 @@ unsigned int XmbFontImp::height() const {
     return m_setextents->max_ink_extent.height;
 }
 
-}; // end namespace FbTk
+} // end namespace FbTk
 
