@@ -1489,7 +1489,8 @@ void FbWinFrame::applyDecorations(bool do_move) {
         client_move = true;
     }
 
-    reconfigure();
+    if (do_move)
+        reconfigure();
     if (client_move)
         frameExtentSig().notify();
 }
@@ -1716,3 +1717,10 @@ void FbWinFrame::displaySize(unsigned int width, unsigned int height) const {
                             width, height - titlebarHeight() - handleHeight());
     m_screen.showGeometry(i, j);
 }
+
+bool FbWinFrame::insideTitlebar(Window win) const {
+    return 
+        gripLeft().window() != win &&
+        gripRight().window() != win &&
+        window().window() != win;
+}
