@@ -19,10 +19,9 @@
 // FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
 // DEALINGS IN THE SOFTWARE.
 
-// $Id$
-
 #include "FocusControl.hh"
 
+#include "ClientPattern.hh"
 #include "Screen.hh"
 #include "Window.hh"
 #include "WinClient.hh"
@@ -46,8 +45,25 @@ using std::string;
 
 WinClient *FocusControl::s_focused_window = 0;
 FluxboxWindow *FocusControl::s_focused_fbwindow = 0;
+WinClient *FocusControl::s_expecting_focus = 0;
 bool FocusControl::s_reverting = false;
 
+namespace {
+
+bool doSkipWindow(const Focusable &win, const ClientPattern *pat) {
+    const FluxboxWindow *fbwin = win.fbwindow();
+    if (!fbwin || fbwin->isFocusHidden())
+        return true; // skip if no fbwindow or if focushidden
+    if (pat && !pat->match(win))
+        return true; // skip if it doesn't match the pattern
+    if (fbwin->workspaceNumber() != win.screen().currentWorkspaceID() &&
+        !fbwin->isStuck())
+        return true; // for now, we only cycle through the current workspace
+    return false; // else don't skip
+}
+
+}; // end anonymous namespace
+    
 FocusControl::FocusControl(BScreen &screen):
     m_screen(screen),
     m_focus_model(screen.resourceManager(), 
@@ -61,32 +77,18 @@ FocusControl::FocusControl(BScreen &screen):
     m_focus_new(screen.resourceManager(), true, 
                 screen.name()+".focusNewWindows", 
                 screen.altName()+".FocusNewWindows"),
+    m_focused_list(screen), m_creation_order_list(screen),
+    m_focused_win_list(screen), m_creation_order_win_list(screen),
     m_cycling_list(0),
     m_was_iconic(false),
     m_cycling_last(0) {
 
-    m_cycling_window = m_focused_list.end();
+    m_cycling_window = m_focused_list.clientList().end();
     
 }
 
-// true if the windows should be skiped else false
-bool doSkipWindow(const WinClient &winclient, int opts) {
-    const FluxboxWindow *win = winclient.fbwindow();
-    return (!win ||
-    // skip if stuck
-    (opts & FocusControl::CYCLESKIPSTUCK) != 0 && win->isStuck() || 
-    // skip if not active client (i.e. only visit each fbwin once)
-    (opts & FocusControl::CYCLEGROUPS) != 0 && win->winClient().window() != winclient.window() ||
-    // skip if shaded
-    (opts & FocusControl::CYCLESKIPSHADED) != 0 && win->isShaded() || 
-    // skip if iconic
-    (opts & FocusControl::CYCLESKIPICONIC) != 0 && win->isIconic() ||
-    // skip if hidden
-    win->isFocusHidden()
-    ); 
-}
-
-void FocusControl::cycleFocus(FocusedWindows &window_list, int opts, bool cycle_reverse) {
+void FocusControl::cycleFocus(const FocusableList &window_list,
+                              const ClientPattern *pat, bool cycle_reverse) {
 
     if (!m_cycling_list) {
         if (&m_screen == FbTk::EventManager::instance()->grabbingKeyboard())
@@ -97,13 +99,15 @@ void FocusControl::cycleFocus(FocusedWindows &window_list, int opts, bool cycle_
     } else if (m_cycling_list != &window_list)
         m_cycling_list = &window_list;
 
-    FocusedWindows::iterator it_begin = window_list.begin();
-    FocusedWindows::iterator it_end = window_list.end();
+    Focusables::const_iterator it_begin = window_list.clientList().begin();
+    Focusables::const_iterator it_end = window_list.clientList().end();
 
     // too many things can go wrong with remembering this
     m_cycling_window = find(it_begin, it_end, s_focused_window);
+    if (m_cycling_window == it_end)
+        m_cycling_window = find(it_begin, it_end, s_focused_fbwindow);
 
-    FocusedWindows::iterator it = m_cycling_window;
+    Focusables::const_iterator it = m_cycling_window;
     FluxboxWindow *fbwin = 0;
     WinClient *last_client = 0;
     WinClient *was_iconic = 0;
@@ -123,11 +127,7 @@ void FocusControl::cycleFocus(FocusedWindows &window_list, int opts, bool cycle_
             continue;
 
         fbwin = (*it)->fbwindow();
-        // make sure the window is on the same workspace,
-        // unless its sticky, which is ok
-        if (!fbwin || 
-            ( fbwin->workspaceNumber() != m_screen.currentWorkspaceID() &&
-              ! fbwin->isStuck()))
+        if (!fbwin)
             continue;
 
         // keep track of the originally selected window in a group
@@ -135,9 +135,10 @@ void FocusControl::cycleFocus(FocusedWindows &window_list, int opts, bool cycle_
         was_iconic = (fbwin->isIconic() ? last_client : 0);
 
         // now we actually try to focus the window
-        if (!doSkipWindow(**it, opts) && (*it)->focus())
+        if (!doSkipWindow(**it, pat) && (*it)->focus())
             break;
     }
+    m_cycling_window = it;
 
     // if we're still in the same fbwin, there's nothing else to do
     if (m_cycling_last && m_cycling_last->fbwindow() == fbwin)
@@ -157,70 +158,87 @@ void FocusControl::cycleFocus(FocusedWindows &window_list, int opts, bool cycle_
         }
     }
 
-    if (isCycling())
-        fbwin->tempRaise();
-    else
+    if (!isCycling())
         fbwin->raise();
 
-    m_cycling_window = it;
     m_cycling_last = last_client;
     m_was_iconic = was_iconic;
 
 }
 
+void FocusControl::goToWindowNumber(const FocusableList &winlist, int num,
+                                    const ClientPattern *pat) {
+    Focusables list = winlist.clientList();
+    if (num < 0) {
+        list.reverse();
+        num = -num;
+    }
+    Focusable *win = 0;
+    Focusables::const_iterator it = list.begin(), it_end = list.end();
+    for (; num && it != it_end; ++it) {
+        if (!doSkipWindow(**it, pat) && (*it)->acceptsFocus()) {
+            --num;
+            win = *it;
+        }
+    }
+    if (win) {
+        win->focus();
+        if (win->fbwindow())
+            win->fbwindow()->raise();
+    }
+}
+
 void FocusControl::addFocusBack(WinClient &client) {
-    m_focused_list.push_back(&client);
-    m_creation_order_list.push_back(&client);
+    m_focused_list.pushBack(client);
+    m_creation_order_list.pushBack(client);
 }
 
 void FocusControl::addFocusFront(WinClient &client) {
-    m_focused_list.push_front(&client);
-    m_creation_order_list.push_back(&client);
+    m_focused_list.pushFront(client);
+    m_creation_order_list.pushBack(client);
+}
+
+void FocusControl::addFocusWinBack(Focusable &win) {
+    m_focused_win_list.pushBack(win);
+    m_creation_order_win_list.pushBack(win);
+}
+
+void FocusControl::addFocusWinFront(Focusable &win) {
+    m_focused_win_list.pushFront(win);
+    m_creation_order_win_list.pushBack(win);
 }
 
 // move all clients in given window to back of focused list
-void FocusControl::setFocusBack(FluxboxWindow *fbwin) {
+void FocusControl::setFocusBack(FluxboxWindow &fbwin) {
     // do nothing if there are no windows open
     // don't change focus order while cycling
     if (m_focused_list.empty() || s_reverting)
         return;
 
-    FocusedWindows::iterator it = m_focused_list.begin();
-    // use back to avoid an infinite loop
-    FocusedWindows::iterator it_back = --m_focused_list.end();
+    m_focused_win_list.moveToBack(fbwin);
 
-    while (it != it_back) {
-        if ((*it)->fbwindow() == fbwin) {
-            m_focused_list.push_back(*it);
-            it = m_focused_list.erase(it);
-        } else
-            ++it;
-    }
-    // move the last one, if necessary, in order to preserve focus order
-    if ((*it)->fbwindow() == fbwin) {
-        m_focused_list.push_back(*it);
-        m_focused_list.erase(it);
+    // we need to move its clients to the back while preserving their order
+    Focusables list = m_focused_list.clientList();
+    Focusables::iterator it = list.begin(), it_end = list.end();
+    for (; it != it_end; ++it) {
+        if ((*it)->fbwindow() == &fbwin)
+            m_focused_list.moveToBack(**it);
     }
 }
-    
+
 void FocusControl::stopCyclingFocus() {
     // nothing to do
     if (m_cycling_list == 0)
         return;
 
-    FocusedWindows::iterator it_end = m_cycling_list->end();
     m_cycling_last = 0;
     m_cycling_list = 0;
 
     // put currently focused window to top
-    // the iterator may be invalid if the window died
-    // in which case we'll do a proper revert focus
-    if (m_cycling_window != it_end && (*m_cycling_window)->fbwindow() &&
-        (*m_cycling_window)->fbwindow()->isVisible()) {
-        WinClient *client = *m_cycling_window;
-        m_focused_list.remove(client);
-        m_focused_list.push_front(client);
-        client->fbwindow()->raise();
+    if (s_focused_window) {
+        setScreenFocusedWindow(*s_focused_window);
+        if (s_focused_fbwindow)
+            s_focused_fbwindow->raise();
     } else
         revertFocus(m_screen);
 }
@@ -230,18 +248,18 @@ void FocusControl::stopCyclingFocus() {
  * If workspace is outside the ID range, then the absolute last focused window
  * is given.
  */
-WinClient *FocusControl::lastFocusedWindow(int workspace) {
-    if (m_focused_list.empty()) return 0;
+Focusable *FocusControl::lastFocusedWindow(int workspace) {
+    if (m_screen.isShuttingdown()) return 0;
     if (workspace < 0 || workspace >= (int) m_screen.numberOfWorkspaces())
-        return m_focused_list.front();
+        return m_focused_list.clientList().front();
 
-    FocusedWindows::iterator it = m_focused_list.begin();    
-    FocusedWindows::iterator it_end = m_focused_list.end();
+    Focusables::iterator it = m_focused_list.clientList().begin();    
+    Focusables::iterator it_end = m_focused_list.clientList().end();
     for (; it != it_end; ++it) {
-        if ((*it)->fbwindow() &&
+        if ((*it)->fbwindow() && (*it)->acceptsFocus() &&
+            (*it)->fbwindow()->winClient().validateClient() &&
             ((((int)(*it)->fbwindow()->workspaceNumber()) == workspace ||
-             (*it)->fbwindow()->isStuck()) && (*it)->acceptsFocus() &&
-             !(*it)->fbwindow()->isIconic()))
+             (*it)->fbwindow()->isStuck()) && !(*it)->fbwindow()->isIconic()))
             return *it;
     }
     return 0;
@@ -253,14 +271,15 @@ WinClient *FocusControl::lastFocusedWindow(int workspace) {
  * Stuck, iconic etc don't matter within a group
  */
 WinClient *FocusControl::lastFocusedWindow(FluxboxWindow &group, WinClient *ignore_client) {
-    if (m_focused_list.empty()) return 0;
+    if (m_focused_list.empty() || m_screen.isShuttingdown())
+        return 0;
 
-    FocusedWindows::iterator it = m_focused_list.begin();    
-    FocusedWindows::iterator it_end = m_focused_list.end();
+    Focusables::iterator it = m_focused_list.clientList().begin();
+    Focusables::iterator it_end = m_focused_list.clientList().end();
     for (; it != it_end; ++it) {
         if (((*it)->fbwindow() == &group) &&
             (*it) != ignore_client)
-            return *it;
+            return dynamic_cast<WinClient *>(*it);
     }
     return 0;
 }
@@ -269,11 +288,10 @@ void FocusControl::setScreenFocusedWindow(WinClient &win_client) {
 
      // raise newly focused window to the top of the focused list
      // don't change the order if we're cycling or shutting down
-     // don't change on startup, as it may add windows that aren't listed yet
-    if (!isCycling() && !m_screen.isShuttingdown() && !s_reverting &&
-            !Fluxbox::instance()->isStartup()) {
-        m_focused_list.remove(&win_client);
-        m_focused_list.push_front(&win_client);
+    if (!isCycling() && !m_screen.isShuttingdown() && !s_reverting) {
+        m_focused_list.moveToFront(win_client);
+        if (win_client.fbwindow())
+            m_focused_win_list.moveToFront(*win_client.fbwindow());
     }
 }
 
@@ -293,7 +311,7 @@ void FocusControl::dirFocus(FluxboxWindow &win, FocusDir dir) {
     
     FluxboxWindow *foundwin = 0;
     int weight = 999999, exposure = 0; // extreme values
-    int borderW = m_screen.winFrameTheme().border().width(),
+    int borderW = win.frame().window().borderWidth(),
         top = win.y() + borderW, 
         bottom = win.y() + win.height() + borderW,
         left = win.x() + borderW,
@@ -305,7 +323,7 @@ void FocusControl::dirFocus(FluxboxWindow &win, FocusDir dir) {
         if ((*it) == &win 
             || (*it)->isIconic() 
             || (*it)->isFocusHidden() 
-            || !(*it)->winClient().acceptsFocus()) 
+            || !(*it)->acceptsFocus()) 
             continue; // skip self
         
         // we check things against an edge, and within the bounds (draw a picture)
@@ -380,7 +398,7 @@ void FocusControl::dirFocus(FluxboxWindow &win, FocusDir dir) {
     }
 
     if (foundwin) 
-        foundwin->setInputFocus();
+        foundwin->focus();
 
 }
 
@@ -388,40 +406,52 @@ void FocusControl::removeClient(WinClient &client) {
     if (client.screen().isShuttingdown())
         return;
 
-    WinClient *cyc = 0;
-    if (m_cycling_list && m_cycling_window != m_cycling_list->end())
-        cyc = *m_cycling_window;
+    if (isCycling() && m_cycling_window != m_cycling_list->clientList().end() &&
+        *m_cycling_window == &client) {
+        m_cycling_window = m_cycling_list->clientList().end();
+        stopCyclingFocus();
+    } else if (m_cycling_last == &client)
+        m_cycling_last = 0;
 
-    m_focused_list.remove(&client);
-    m_creation_order_list.remove(&client);
+    m_focused_list.remove(client);
+    m_creation_order_list.remove(client);
+    client.screen().clientListSig().notify();
+}
 
-    if (cyc == &client) {
-        m_cycling_window = m_cycling_list->end();
+void FocusControl::removeWindow(Focusable &win) {
+    if (win.screen().isShuttingdown())
+        return;
+
+    if (isCycling() && m_cycling_window != m_cycling_list->clientList().end() &&
+        *m_cycling_window == &win) {
+        m_cycling_window = m_cycling_list->clientList().end();
         stopCyclingFocus();
     }
+
+    m_focused_win_list.remove(win);
+    m_creation_order_win_list.remove(win);
+    win.screen().clientListSig().notify();
 }
 
 void FocusControl::shutdown() {
     // restore windows backwards so they get put back correctly on restart
-    FocusedWindows::reverse_iterator it = m_focused_list.rbegin();
-    for (; it != m_focused_list.rend(); ++it) {
-        if (*it && (*it)->fbwindow())
-            (*it)->fbwindow()->restore(*it, true);
+    Focusables::reverse_iterator it = m_focused_list.clientList().rbegin();
+    for (; it != m_focused_list.clientList().rend(); ++it) {
+        WinClient *client = dynamic_cast<WinClient *>(*it);
+        if (client && client->fbwindow())
+            client->fbwindow()->restore(client, true);
     }
 }
 
 /**
  * This function is called whenever we aren't quite sure what
  * focus is meant to be, it'll make things right ;-)
- * last_focused is set to something if we want to make use of the
- * previously focused window (it must NOT be set focused now, it
- *   is probably dying).
  */
 void FocusControl::revertFocus(BScreen &screen) {
-    if (s_reverting)
+    if (s_reverting || screen.isShuttingdown())
         return;
 
-    WinClient *next_focus = 
+    Focusable *next_focus =
         screen.focusControl().lastFocusedWindow(screen.currentWorkspaceID());
 
     if (next_focus && next_focus->fbwindow() &&
@@ -430,6 +460,7 @@ void FocusControl::revertFocus(BScreen &screen) {
 
     // if setting focus fails, or isn't possible, fallback correctly
     if (!(next_focus && next_focus->focus())) {
+
         setFocusedWindow(0); // so we don't get dangling m_focused_window pointer
         // if there's a menu open, focus it
         if (FbTk::Menu::shownMenu())
@@ -458,9 +489,8 @@ void FocusControl::revertFocus(BScreen &screen) {
  * If unfocus_frame is true, we won't focus anything in the same frame
  * as the client.
  *
- * So, we first prefer to choose a transient parent, then the last
- * client in this window, and if no luck (or unfocus_frame), then
- * we just use the normal revertFocus on the screen.
+ * So, we first prefer to choose the last client in this window, and if no luck
+ * (or unfocus_frame), then we just use the normal revertFocus on the screen.
  *
  * assumption: client has focus
  */
@@ -475,13 +505,13 @@ void FocusControl::unfocusWindow(WinClient &client,
 
     BScreen &screen = fbwin->screen();
 
+    if (client.isTransient() && client.transientFor()->focus())
+        return;
+
     if (!unfocus_frame) {
         WinClient *last_focus = screen.focusControl().lastFocusedWindow(*fbwin, &client);
-        if (last_focus != 0 &&
-            fbwin->setCurrentClient(*last_focus, 
-                                    s_focused_window == &client)) {
+        if (last_focus && last_focus->focus())
             return;
-        }
     }
 
     if (full_revert && s_focused_window == &client)
@@ -491,6 +521,10 @@ void FocusControl::unfocusWindow(WinClient &client,
 
 
 void FocusControl::setFocusedWindow(WinClient *client) {
+    if (client == s_focused_window &&
+        (!client || client->fbwindow() == s_focused_fbwindow))
+        return;
+
     BScreen *screen = client ? &client->screen() : 0;
     BScreen *old_screen = 
         FocusControl::focusedWindow() ? 
@@ -506,13 +540,15 @@ void FocusControl::setFocusedWindow(WinClient *client) {
 #endif // DEBUG
 
     // Update the old focused client to non focus
-    if (s_focused_fbwindow)
+    if (s_focused_fbwindow &&
+        (!client || client->fbwindow() != s_focused_fbwindow))
         s_focused_fbwindow->setFocusFlag(false);
 
     if (client && client->fbwindow() && !client->fbwindow()->isIconic()) {
         // screen should be ok
         s_focused_fbwindow = client->fbwindow();        
         s_focused_window = client;     // update focused window
+        s_expecting_focus = 0;
         s_focused_fbwindow->setCurrentClient(*client, 
                               false); // don't set inputfocus
         s_focused_fbwindow->setFocusFlag(true); // set focus flag
@@ -523,7 +559,10 @@ void FocusControl::setFocusedWindow(WinClient *client) {
     }
 
     // update AtomHandlers and/or other stuff...
-    Fluxbox::instance()->updateFocusedWindow(screen, old_screen);
+    if (screen)
+        screen->focusedWindowSig().notify();
+    if (old_screen && screen != old_screen)
+        old_screen->focusedWindowSig().notify();
 }
 
 ////////////////////// FocusControl RESOURCES
