@@ -1,5 +1,5 @@
 // GenericTool.cc for Fluxbox
-// Copyright (c) 2003 - 2005 Henrik Kinnunen (fluxgen at fluxbox dot org)
+// Copyright (c) 2003 - 2006 Henrik Kinnunen (fluxgen at fluxbox dot org)
 //
 // Permission is hereby granted, free of charge, to any person obtaining a
 // copy of this software and associated documentation files (the "Software"),
@@ -19,15 +19,14 @@
 // FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
 // DEALINGS IN THE SOFTWARE.
 
-// $Id: GenericTool.cc 3932 2005-04-10 18:18:14Z simonb $
-
 #include "GenericTool.hh"
 #include "FbTk/FbWindow.hh"
+#include "ToolTheme.hh"
 
 #include <string>
 
 GenericTool::GenericTool(FbTk::FbWindow *new_window, ToolbarItem::Type type,
-                         ToolTheme &theme):
+                         FbTk::ThemeProxy<ToolTheme> &theme):
     ToolbarItem(type),
     m_window(new_window),
     m_theme(theme) {
@@ -84,3 +83,8 @@ void GenericTool::renderTheme(unsigned char alpha) {
 void GenericTool::update(FbTk::Subject *subj) {
     m_window->clear();
 }
+
+void GenericTool::parentMoved() {
+    m_window->parentMoved();
+}
+
