@@ -19,20 +19,17 @@
 // FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
 // DEALINGS IN THE SOFTWARE.
 
-// $Id: Gnome.cc 4984 2007-07-15 16:32:54Z markt $
-
 #include "Gnome.hh"
 
-#include "App.hh"
+#include "FbTk/App.hh"
+#include "FbTk/I18n.hh"
 #include "Window.hh"
 #include "Screen.hh"
 #include "WinClient.hh"
 #include "Workspace.hh"
 #include "Layer.hh"
-#include "FbTk/I18n.hh"
 
 #include <iostream>
-#include <new>
 #ifdef HAVE_CSTRING
   #include <cstring>
 #else
@@ -396,7 +393,7 @@ void Gnome::setState(FluxboxWindow *win, int state) {
         if (win->isIconic())
             win->iconify();
     } else if (win->isIconic())
-        win->deiconify(true, true);
+        win->deiconify(true);
 
     if (state & WIN_STATE_SHADED) {
 #ifdef DEBUG
