@@ -19,8 +19,6 @@
 // FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
 // DEALINGS IN THE SOFTWARE.
 
-// $Id$
-
 #include "LayerMenu.hh"
 
 #include "FbCommands.hh"
@@ -30,12 +28,13 @@
 #include "FbTk/SimpleCommand.hh"
 #include "FbTk/I18n.hh"
 
-LayerMenu::LayerMenu(MenuTheme &tm, FbTk::ImageControl &imgctrl,
+LayerMenu::LayerMenu(FbTk::ThemeProxy<FbTk::MenuTheme> &tm,
+                     FbTk::ImageControl &imgctrl,
                      FbTk::XLayer &layer, LayerObject *object, bool save_rc):
     ToggleMenu(tm, imgctrl, layer) {
     _FB_USES_NLS;
 
-    
+
     struct {
         int set;
         int base;
@@ -50,17 +49,17 @@ LayerMenu::LayerMenu(MenuTheme &tm, FbTk::ImageControl &imgctrl,
         {0, 0, _FB_XTEXT(Layer, Bottom, "Bottom", "Layer bottom"), Layer::BOTTOM},
         {0, 0, _FB_XTEXT(Layer, Desktop, "Desktop", "Layer desktop"), Layer::DESKTOP},
     };
-    
-    FbTk::RefCount<FbTk::Command> saverc_cmd(new FbCommands::SaveResources());
+
+    FbTk::RefCount<FbTk::Command<void> > saverc_cmd(new FbCommands::SaveResources());
 
     for (size_t i=0; i < 6; ++i) {
         // TODO: fetch nls string
-        if (save_rc) {    
-            insert(new LayerMenuItem(layer_menuitems[i].default_str, 
+        if (save_rc) {
+            insert(new LayerMenuItem(layer_menuitems[i].default_str,
                                      object, layer_menuitems[i].layernum, saverc_cmd));
         } else {
-            insert(new LayerMenuItem(layer_menuitems[i].default_str, 
-                                     object, layer_menuitems[i].layernum));               
+            insert(new LayerMenuItem(layer_menuitems[i].default_str,
+                                     object, layer_menuitems[i].layernum));
         }
     }
     updateMenu();
