@@ -20,25 +20,23 @@
 // FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
 // DEALINGS IN THE SOFTWARE.
 
-// $Id: MenuCreator.cc 5060 2007-09-08 18:38:30Z markt $
-
 #include "MenuCreator.hh"
 
 #include "defaults.hh"
 #include "Screen.hh"
-#include "CommandParser.hh"
+#include "FbTk/CommandParser.hh"
 #include "fluxbox.hh"
 #include "Window.hh"
 #include "WindowCmd.hh"
+#include "CurrentWindowCmd.hh"
+#include "WindowMenuAccessor.hh"
 
-#include "FbMenu.hh"
-#include "IconMenu.hh"
+#include "ClientMenu.hh"
 #include "WorkspaceMenu.hh"
 #include "LayerMenu.hh"
 #include "SendToMenu.hh"
 #include "AlphaMenu.hh"
 #include "Layer.hh"
-#include "BoolMenuItem.hh"
 
 #include "FbMenuParser.hh"
 #include "StyleMenuItem.hh"
@@ -46,13 +44,13 @@
 
 #include "FbTk/I18n.hh"
 #include "FbTk/MultiButtonMenuItem.hh"
+#include "FbTk/BoolMenuItem.hh"
 #include "FbTk/RefCount.hh"
 #include "FbTk/MacroCommand.hh"
 #include "FbTk/SimpleCommand.hh"
 #include "FbTk/StringUtil.hh"
 #include "FbTk/FileUtil.hh"
 #include "FbTk/MenuSeparator.hh"
-#include "FbTk/MenuIcon.hh"
 #include "FbTk/Transparent.hh"
 
 #include <iostream>
@@ -63,20 +61,26 @@ using std::string;
 using std::vector;
 using std::list;
 using std::less;
+using FbTk::AutoReloadHelper;
 
 list<string> MenuCreator::encoding_stack;
 list<size_t> MenuCreator::stacksize_stack;
 
 FbTk::StringConvertor MenuCreator::m_stringconvertor(FbTk::StringConvertor::ToFbString);
 
-static void createStyleMenu(FbTk::Menu &parent, const string &label,
-                            const string &directory) {
+namespace {
+
+void createStyleMenu(FbTk::Menu &parent, const string &label,
+                     AutoReloadHelper *reloader, const string &directory) {
     // perform shell style ~ home directory expansion
     string stylesdir(FbTk::StringUtil::expandFilename(directory));
 
     if (!FbTk::FileUtil::isDirectory(stylesdir.c_str()))
         return;
 
+    if (reloader)
+        reloader->addFile(stylesdir);
+
     FbTk::Directory dir(stylesdir.c_str());
 
     // create a vector of all the filenames in the directory
@@ -101,18 +105,21 @@ static void createStyleMenu(FbTk::Menu &parent, const string &label,
     }
     // update menu graphics
     parent.updateMenu();
-    Fluxbox::instance()->saveMenuFilename(stylesdir.c_str());
 
 }
 
-static void createRootCmdMenu(FbTk::Menu &parent, const string &label,
-                                const string &directory, const string &cmd) {
+void createRootCmdMenu(FbTk::Menu &parent, const string &label,
+                       const string &directory, AutoReloadHelper *reloader,
+                       const string &cmd) {
     // perform shell style ~ home directory expansion
     string rootcmddir(FbTk::StringUtil::expandFilename(directory));
 
     if (!FbTk::FileUtil::isDirectory(rootcmddir.c_str()))
         return;
 
+    if (reloader)
+        reloader->addFile(rootcmddir);
+
     FbTk::Directory dir(rootcmddir.c_str());
 
     // create a vector of all the filenames in the directory
@@ -136,7 +143,6 @@ static void createRootCmdMenu(FbTk::Menu &parent, const string &label,
     }
     // update menu graphics
     parent.updateMenu();
-    Fluxbox::instance()->saveMenuFilename(rootcmddir.c_str());
 
 }
 
@@ -145,85 +151,55 @@ class ParseItem {
 public:
     explicit ParseItem(FbTk::Menu *menu):m_menu(menu) {}
 
-    inline void load(Parser &p, FbTk::StringConvertor &m_labelconvertor) {
+    void load(FbTk::Parser &p, FbTk::StringConvertor &m_labelconvertor) {
         p>>m_key>>m_label>>m_cmd>>m_icon;
         m_label.second = m_labelconvertor.recode(m_label.second);
     }
-    inline const string &icon() const { return m_icon.second; }
-    inline const string &command() const { return m_cmd.second; }
-    inline const string &label() const { return m_label.second; }
-    inline const string &key() const { return m_key.second; }
-    inline FbTk::Menu *menu() { return m_menu; }
+    const string &icon() const { return m_icon.second; }
+    const string &command() const { return m_cmd.second; }
+    const string &label() const { return m_label.second; }
+    const string &key() const { return m_key.second; }
+    FbTk::Menu *menu() { return m_menu; }
 private:
-    Parser::Item m_key, m_label, m_cmd, m_icon;
+    FbTk::Parser::Item m_key, m_label, m_cmd, m_icon;
     FbTk::Menu *m_menu;
 };
 
-class MenuContext: public LayerObject, public AlphaObject {
+class MenuContext: public LayerObject {
 public:
     void moveToLayer(int layer_number) {
-        if (WindowCmd<void>::window() == 0)
+        if (FbMenu::window() == 0)
             return;
-        WindowCmd<void>::window()->moveToLayer(layer_number);
+        FbMenu::window()->moveToLayer(layer_number);
     }
     int layerNumber() const {
-        if (WindowCmd<void>::window() == 0)
+        if (FbMenu::window() == 0)
             return -1;
-        return WindowCmd<void>::window()->layerItem().getLayerNum();
-    }
-
-    int getFocusedAlpha() const {
-        if (WindowCmd<void>::window() == 0)
-            return 255;
-        return WindowCmd<void>::window()->getFocusedAlpha();
-    }
-
-    int getUnfocusedAlpha() const {
-        if (WindowCmd<void>::window() == 0)
-            return 255;
-        return WindowCmd<void>::window()->getUnfocusedAlpha();
-    }
-
-    bool getUseDefaultAlpha() const { 
-        if (WindowCmd<void>::window() == 0)
-            return true;
-        return WindowCmd<void>::window()->getUseDefaultAlpha();
-    }
-
-    void setFocusedAlpha(int alpha) {
-        if (WindowCmd<void>::window() == 0)
-            return;
-        WindowCmd<void>::window()->setFocusedAlpha(alpha);
-    }
-
-    void setUnfocusedAlpha(int alpha) {
-        if (WindowCmd<void>::window() == 0)
-            return;
-        WindowCmd<void>::window()->setUnfocusedAlpha(alpha);
-    }
-
-    void setUseDefaultAlpha(bool use_default) {
-        if (WindowCmd<void>::window() == 0)
-            return;
-        WindowCmd<void>::window()->setUseDefaultAlpha(use_default);
+        return FbMenu::window()->layerItem().getLayerNum();
     }
 
 };
 
-static void translateMenuItem(Parser &parse, ParseItem &item, FbTk::StringConvertor &labelconvertor);
+void translateMenuItem(FbTk::Parser &parse, ParseItem &item,
+                       FbTk::StringConvertor &labelconvertor,
+                       AutoReloadHelper *reloader);
 
 
-static void parseMenu(Parser &pars, FbTk::Menu &menu, FbTk::StringConvertor &label_convertor) {
+void parseMenu(FbTk::Parser &pars, FbTk::Menu &menu,
+               FbTk::StringConvertor &label_convertor,
+               AutoReloadHelper *reloader) {
     ParseItem pitem(&menu);
     while (!pars.eof()) {
         pitem.load(pars, label_convertor);
         if (pitem.key() == "end")
             return;
-        translateMenuItem(pars, pitem, label_convertor);
+        translateMenuItem(pars, pitem, label_convertor, reloader);
     }
 }
 
-static void translateMenuItem(Parser &parse, ParseItem &pitem, FbTk::StringConvertor &labelconvertor) {
+void translateMenuItem(FbTk::Parser &parse, ParseItem &pitem,
+                       FbTk::StringConvertor &labelconvertor,
+                       AutoReloadHelper *reloader) {
     if (pitem.menu() == 0)
         throw string("translateMenuItem: We must have a menu in ParseItem!");
 
@@ -249,32 +225,18 @@ static void translateMenuItem(Parser &parse, ParseItem &pitem, FbTk::StringConve
         else
             menu.insert(str_label, submenu);
     } else if (str_key == "exit") { // exit
-        FbTk::RefCount<FbTk::Command> exit_cmd(CommandParser::instance().parseLine("exit"));
+        FbTk::RefCount<FbTk::Command<void> > exit_cmd(FbTk::CommandParser<void>::instance().parse("exit"));
         if (str_label.empty())
             menu.insert(_FB_XTEXT(Menu, Exit, "Exit", "Exit Command"), exit_cmd);
         else
             menu.insert(str_label, exit_cmd);
     } else if (str_key == "exec") {
         // execute and hide menu
-        using namespace FbTk;
-        RefCount<Command> exec_cmd(CommandParser::instance().parseLine("exec " + str_cmd));
-        RefCount<Command> hide_menu(new SimpleCommand<FbTk::Menu>(menu,
-                                                                  &Menu::hide));
-        MacroCommand *exec_and_hide = new FbTk::MacroCommand();
-        exec_and_hide->add(hide_menu);
-        exec_and_hide->add(exec_cmd);
-        RefCount<Command> exec_and_hide_cmd(exec_and_hide);
-        menu.insert(str_label, exec_and_hide_cmd);
+        FbTk::RefCount<FbTk::Command<void> > exec_cmd(FbTk::CommandParser<void>::instance().parse("exec " + str_cmd));
+        menu.insert(str_label, exec_cmd);
     } else if (str_key == "macrocmd") {
-        using namespace FbTk;
-        RefCount<Command> macro_cmd(CommandParser::instance().parseLine("macrocmd " + str_cmd));
-        RefCount<Command> hide_menu(new SimpleCommand<FbTk::Menu>(menu,
-                                                                  &Menu::hide));
-        MacroCommand *exec_and_hide = new FbTk::MacroCommand();
-        exec_and_hide->add(hide_menu);
-        exec_and_hide->add(macro_cmd);
-        RefCount<Command> exec_and_hide_cmd(exec_and_hide);
-        menu.insert(str_label, exec_and_hide_cmd);
+        FbTk::RefCount<FbTk::Command<void> > macro_cmd(FbTk::CommandParser<void>::instance().parse("macrocmd " + str_cmd));
+        menu.insert(str_label, macro_cmd);
     } else if (str_key == "style") {	// style
         menu.insert(new StyleMenuItem(str_label, str_cmd));
     } else if (str_key == "config") {
@@ -307,13 +269,13 @@ static void translateMenuItem(Parser &parse, ParseItem &pitem, FbTk::StringConve
                 if (FbTk::FileUtil::isRegularFile(thisfile.c_str()) &&
                         (filelist[file_index][0] != '.') &&
                         (thisfile[thisfile.length() - 1] != '~')) {
-                    MenuCreator::createFromFile(thisfile, menu, false);
+                    MenuCreator::createFromFile(thisfile, menu, reloader, false);
                 }
             }
 
         } else {
             // inject this file into the current menu
-            MenuCreator::createFromFile(newfile, menu, false);
+            MenuCreator::createFromFile(newfile, menu, reloader, false);
         }
 
         safe_counter--;
@@ -330,26 +292,22 @@ static void translateMenuItem(Parser &parse, ParseItem &pitem, FbTk::StringConve
         else
             submenu->setLabel(str_label);
 
-        parseMenu(parse, *submenu, labelconvertor);
+        parseMenu(parse, *submenu, labelconvertor, reloader);
         submenu->updateMenu();
         menu.insert(str_label, submenu);
-        // save to screen list so we can delete it later
-        BScreen *screen = Fluxbox::instance()->findScreen(screen_number);
-        if (screen != 0)
-            screen->saveMenu(*submenu);
 
     } // end of submenu
     else if (str_key == "stylesdir" || str_key == "stylesmenu") {
-        createStyleMenu(menu, str_label,
+        createStyleMenu(menu, str_label, reloader,
                         str_key == "stylesmenu" ? str_cmd : str_label);
     } // end of stylesdir
     else if (str_key == "themesdir" || str_key == "themesmenu") {
-        createStyleMenu(menu, str_label,
+        createStyleMenu(menu, str_label, reloader,
                         str_key == "themesmenu" ? str_cmd : str_label);
     } // end of themesdir
     else if (str_key == "wallpapers" || str_key == "wallpapermenu" ||
              str_key == "rootcommands") {
-         createRootCmdMenu(menu, str_label, str_label,
+         createRootCmdMenu(menu, str_label, str_label, reloader,
                           str_cmd == "" ? realProgramName("fbsetbg") : str_cmd);
     } // end of wallpapers
     else if (str_key == "workspaces") {
@@ -364,11 +322,11 @@ static void translateMenuItem(Parser &parse, ParseItem &pitem, FbTk::StringConve
         MenuCreator::startEncoding(str_cmd);
     } else if (str_key == "endencoding") {
         MenuCreator::endEncoding();
-    }
-    else { // ok, if we didn't find any special menu item we try with command parser
-        // we need to attach command with arguments so command parser can parse it
+    } else if (!MenuCreator::createWindowMenuItem(str_key, str_label, menu)) {
+        // if we didn't find any special menu item we try with command parser
+        // we need to attach command to arguments so command parser can parse it
         string line = str_key + " " + str_cmd;
-        FbTk::RefCount<FbTk::Command> command(CommandParser::instance().parseLine(line));
+        FbTk::RefCount<FbTk::Command<void> > command(FbTk::CommandParser<void>::instance().parse(line));
         if (*command != 0) {
             // special NLS default labels
             if (str_label.empty()) {
@@ -390,43 +348,6 @@ static void translateMenuItem(Parser &parse, ParseItem &pitem, FbTk::StringConve
     }
 }
 
-
-static void parseWindowMenu(Parser &parse, FbTk::Menu &menu, FbTk::StringConvertor &labelconvertor) {
-
-    ParseItem pitem(&menu);
-    while (!parse.eof()) {
-        pitem.load(parse, labelconvertor);
-        if (MenuCreator::createWindowMenuItem(pitem.key(), pitem.label(), menu))
-            continue;
-
-        if (pitem.key() == "end") {
-            return;
-        } else if (pitem.key() == "submenu") {
-            FbTk::Menu *submenu = MenuCreator::createMenu(pitem.label(), menu.screenNumber());
-            parseWindowMenu(parse, *submenu, labelconvertor);
-            submenu->updateMenu();
-            menu.insert(pitem.label(), submenu);
-
-        } else { // try non window menu specific stuff
-            translateMenuItem(parse, pitem, labelconvertor);
-        }
-    }
-}
-
-FbTk::Menu *MenuCreator::createMenu(const string &label, int screen_number) {
-    BScreen *screen = Fluxbox::instance()->findScreen(screen_number);
-    if (screen == 0)
-        return 0;
-
-    FbTk::Menu *menu = new FbMenu(screen->menuTheme(),
-                                  screen->imageControl(),
-                                  *screen->layerManager().getLayer(Layer::MENU));
-    if (!label.empty())
-        menu->setLabel(label);
-
-    return menu;
-}
-
 bool getStart(FbMenuParser &parser, string &label, FbTk::StringConvertor &labelconvertor) {
     ParseItem pitem(0);
     while (!parser.eof()) {
@@ -443,114 +364,60 @@ bool getStart(FbMenuParser &parser, string &label, FbTk::StringConvertor &labelc
     return true;
 }
 
-FbTk::Menu *MenuCreator::createFromFile(const string &filename, int screen_number, bool require_begin) {
-    string real_filename = FbTk::StringUtil::expandFilename(filename);
-    Fluxbox::instance()->saveMenuFilename(real_filename.c_str());
+}; // end of anonymous namespace
 
-    FbMenuParser parser(real_filename);
-    if (!parser.isLoaded())
+FbMenu *MenuCreator::createMenu(const string &label, int screen_number) {
+    BScreen *screen = Fluxbox::instance()->findScreen(screen_number);
+    if (screen == 0)
         return 0;
 
-    startFile();
-    string label;
-    if (require_begin && !getStart(parser, label, m_stringconvertor)) {
-        endFile();
-        return 0;
-    }
-
-    FbTk::Menu *menu = createMenu(label, screen_number);
-    if (menu != 0)
-        parseMenu(parser, *menu, m_stringconvertor);
-
-    endFile();
+    FbMenu *menu = new FbMenu(screen->menuTheme(),
+                                  screen->imageControl(),
+                                  *screen->layerManager().getLayer(Layer::MENU));
+    if (!label.empty())
+        menu->setLabel(label);
 
     return menu;
 }
 
-
 bool MenuCreator::createFromFile(const string &filename,
-                                 FbTk::Menu &inject_into, bool require_begin) {
-
+                                 FbTk::Menu &inject_into,
+                                 AutoReloadHelper *reloader, bool begin) {
     string real_filename = FbTk::StringUtil::expandFilename(filename);
+
     FbMenuParser parser(real_filename);
     if (!parser.isLoaded())
         return false;
 
     startFile();
-    string label;
-    if (require_begin && !getStart(parser, label, m_stringconvertor)) {
-        endFile();
-        return false;
+    if (begin) {
+        string label;
+        if (!getStart(parser, label, m_stringconvertor)) {
+            endFile();
+            return false;
+        }
+        inject_into.setLabel(label);
     }
 
     // save menu filename, so we can check if it changes
-    Fluxbox::instance()->saveMenuFilename(real_filename.c_str());
+    if (reloader)
+        reloader->addFile(real_filename);
 
-    parseMenu(parser, inject_into, m_stringconvertor);
+    parseMenu(parser, inject_into, m_stringconvertor, reloader);
     endFile();
 
     return true;
 }
 
-
-bool MenuCreator::createWindowMenuFromFile(const string &filename,
-                                           FbTk::Menu &inject_into,
-                                           bool require_begin) {
-    string real_filename = FbTk::StringUtil::expandFilename(filename);
-    FbMenuParser parser(real_filename);
-    if (!parser.isLoaded())
-        return false;
-
-    string label;
-
-    startFile();
-    if (require_begin && !getStart(parser, label, m_stringconvertor)) {
-        endFile();
-        return false;
-    }
-
-    parseWindowMenu(parser, inject_into, m_stringconvertor);
-    endFile();
-
-    return true;
-}
-
-
-FbTk::Menu *MenuCreator::createMenuType(const string &type, int screen_num) {
+FbMenu *MenuCreator::createMenuType(const string &type, int screen_num) {
     BScreen *screen = Fluxbox::instance()->findScreen(screen_num);
     if (screen == 0)
         return 0;
-    if (type == "iconmenu") {
-        return new IconMenu(*screen);
-    } else if (type == "workspacemenu") {
+    if (type == "iconmenu")
+        return new ClientMenu(*screen, screen->iconList(),
+                              &screen->iconListSig());
+    else if (type == "workspacemenu")
         return new WorkspaceMenu(*screen);
-    } else if (type == "windowmenu") {
-        FbTk::Menu *menu = screen->createMenu("");
-
-        menu->disableTitle(); // not titlebar
-        if (screen->windowMenuFilename().empty() ||
-            ! createWindowMenuFromFile(screen->windowMenuFilename(), *menu, true)) {
-            const char *default_menu[] = {
-                "shade",
-                "stick",
-                "maximize",
-                "iconify",
-                "raise",
-                "lower",
-                "sendto",
-                "layer",
-                "alpha",
-                "extramenus",
-                "separator",
-                "close",
-                0
-            };
-            for (unsigned int i=0; default_menu[i]; ++i)
-                createWindowMenuItem(default_menu[i], "", *menu);
-        }
-        menu->reconfigure(); // update graphics
-        return menu;
-    }
 
     return 0;
 }
@@ -558,14 +425,14 @@ FbTk::Menu *MenuCreator::createMenuType(const string &type, int screen_num) {
 bool MenuCreator::createWindowMenuItem(const string &type,
                                        const string &label,
                                        FbTk::Menu &menu) {
-    typedef FbTk::RefCount<FbTk::Command> RefCmd;
+    typedef FbTk::RefCount<FbTk::Command<void> > RefCmd;
     _FB_USES_NLS;
 
     static MenuContext context;
 
     if (type == "shade") {
-        static ObjectResource<FluxboxWindow, bool> res(&WindowCmd<void>::window, &FluxboxWindow::isShaded, &FluxboxWindow::shade, false);
-        menu.insert(new BoolResMenuItem<ObjectResource<FluxboxWindow, bool> >(
+        static WindowMenuAccessor<bool> res(&FluxboxWindow::isShaded, &FluxboxWindow::setShaded, false);
+        menu.insert(new FbTk::BoolMenuItem(
                         label.empty()?_FB_XTEXT(Windowmenu, Shade, "Shade", "Shade the window"):label, 
                         res));
 
@@ -588,8 +455,8 @@ bool MenuCreator::createWindowMenuItem(const string &type,
         maximize_item->setCommand(3, maximize_horiz_cmd);
         menu.insert(maximize_item);
     } else if (type == "iconify") {
-        static ObjectResource<FluxboxWindow, bool> res(&WindowCmd<void>::window, &FluxboxWindow::isIconic, &FluxboxWindow::toggleIconic, false);
-        menu.insert(new BoolResMenuItem<ObjectResource<FluxboxWindow, bool> >(
+        static WindowMenuAccessor<bool> res(&FluxboxWindow::isIconic, &FluxboxWindow::setIconic, false);
+        menu.insert(new FbTk::BoolMenuItem(
                         label.empty() ?
                         _FB_XTEXT(Windowmenu, Iconify,
                                   "Iconify", "Iconify the window") :
@@ -620,12 +487,18 @@ bool MenuCreator::createWindowMenuItem(const string &type,
                     label, raise_cmd);
 
     } else if (type == "stick") {
-        static ObjectResource<FluxboxWindow, bool> res(&WindowCmd<void>::window, &FluxboxWindow::isStuck, &FluxboxWindow::stick, false);
-        menu.insert(new BoolResMenuItem<ObjectResource<FluxboxWindow, bool> >(
+        static WindowMenuAccessor<bool> res(&FluxboxWindow::isStuck, &FluxboxWindow::setStuck, false);
+        menu.insert(new FbTk::BoolMenuItem(
                         label.empty() ?
                         _FB_XTEXT(Windowmenu, Stick,
                                   "Stick", "Stick the window"):
                         label, res));
+    } else if (type == "settitledialog") {
+        RefCmd setname_cmd(new SetTitleDialogCmd());
+        menu.insert(label.empty() ?
+                    _FB_XTEXT(Windowmenu, SetTitle,
+                              "Set Title", "Change the title of the window"):
+                    label, setname_cmd);
 #ifdef HAVE_XRENDER
     } else if (type == "alpha") {
         if (FbTk::Transparent::haveComposite() || 
@@ -634,12 +507,14 @@ bool MenuCreator::createWindowMenuItem(const string &type,
             if (screen == 0)
                 return false;
 
+            FbTk::Menu *submenu =
+                new AlphaMenu(screen->menuTheme(),
+                              screen->imageControl(),
+                              *screen->layerManager().getLayer(Layer::MENU));
+            submenu->disableTitle();
             menu.insert(label.empty() ? _FB_XTEXT(Configmenu, Transparency, "Transparency",
                                                   "Menu containing various transparency options"): label,
-                        new AlphaMenu(screen->menuTheme(),
-                                      screen->imageControl(),
-                                      *screen->layerManager().getLayer(Layer::MENU),
-                                      context));
+                        submenu);
         }
 #endif // HAVE_XRENDER
     } else if (type == "extramenus") {
