@@ -286,7 +286,7 @@ getString() const {
 BScreen::ScreenResource::ScreenResource(FbTk::ResourceManager &rm,
                                         const string &scrname,
                                         const string &altscrname):
-    opaque_move(rm, false, scrname + ".opaqueMove", altscrname+".OpaqueMove"),
+    opaque_move(rm, true, scrname + ".opaqueMove", altscrname+".OpaqueMove"),
     full_max(rm, false, scrname+".fullMaximization", altscrname+".FullMaximization"),
     max_ignore_inc(rm, true, scrname+".maxIgnoreIncrement", altscrname+".MaxIgnoreIncrement"),
     max_disable_move(rm, false, scrname+".maxDisableMove", altscrname+".MaxDisableMove"),
@@ -300,9 +300,9 @@ BScreen::ScreenResource::ScreenResource(FbTk::ResourceManager &rm,
     tab_placement(rm, FbWinFrame::TOPLEFT, scrname+".tab.placement", altscrname+".Tab.Placement"),
     windowmenufile(rm, Fluxbox::instance()->getDefaultDataFilename("windowmenu"), scrname+".windowMenu", altscrname+".WindowMenu"),
     typing_delay(rm, 0, scrname+".noFocusWhileTypingDelay", altscrname+".NoFocusWhileTypingDelay"),
-    follow_model(rm, IGNORE_OTHER_WORKSPACES, scrname+".followModel", altscrname+".followModel"),
-    user_follow_model(rm, FOLLOW_ACTIVE_WINDOW, scrname+".userFollowModel", altscrname+".UserFollowModel"),
-    workspaces(rm, 1, scrname+".workspaces", altscrname+".Workspaces"),
+    follow_model(rm, SEMIFOLLOW_ACTIVE_WINDOW, scrname+".followModel", altscrname+".followModel"),
+    user_follow_model(rm, SEMIFOLLOW_ACTIVE_WINDOW, scrname+".userFollowModel", altscrname+".UserFollowModel"),
+    workspaces(rm, 4, scrname+".workspaces", altscrname+".Workspaces"),
     edge_snap_threshold(rm, 10, scrname+".edgeSnapThreshold", altscrname+".EdgeSnapThreshold"),
     focused_alpha(rm, 255, scrname+".window.focus.alpha", altscrname+".Window.Focus.Alpha"),
     unfocused_alpha(rm, 255, scrname+".window.unfocus.alpha", altscrname+".Window.Unfocus.Alpha"),
@@ -326,8 +326,6 @@ BScreen::ScreenResource::ScreenResource(FbTk::ResourceManager &rm,
                  FbTk::GContext::CAPNOTLAST,
                  scrname+".overlay.capStyle",
                  altscrname+".overlay.CapStyle"),
-    scroll_action(rm, "", scrname+".windowScrollAction", altscrname+".WindowScrollAction"),
-    scroll_reverse(rm, false, scrname+".windowScrollReverse", altscrname+".WindowScrollReverse"),
     allow_remote_actions(rm, false, scrname+".allowRemoteActions", altscrname+".AllowRemoteActions"),
     clientmenu_use_pixmap(rm, true, scrname+".clientMenu.usePixmap", altscrname+".ClientMenu.UsePixmap"),
     tabs_use_pixmap(rm, true, scrname+".tabs.usePixmap", altscrname+".Tabs.UsePixmap"),
@@ -506,6 +504,11 @@ BScreen::BScreen(FbTk::ResourceManager &rm,
     renderPosWindow();
     m_tooltip_window->setDelay(*resource.tooltip_delay);
 
+
+    //!! TODO: we shouldn't do this more than once, but since slit handles their
+    // own resources we must do this.
+    fluxbox->load_rc(*this);
+
     // setup workspaces and workspace menu
     int nr_ws = *resource.workspaces;
     addWorkspace(); // at least one
@@ -515,11 +518,6 @@ BScreen::BScreen(FbTk::ResourceManager &rm,
 
     m_current_workspace = m_workspaces_list.front();
 
-
-    //!! TODO: we shouldn't do this more than once, but since slit handles their
-    // own resources we must do this.
-    fluxbox->load_rc(*this);
-
     m_windowmenu.reset(createMenu(""));
     m_windowmenu->setInternalMenu();
     m_windowmenu->setReloadHelper(new FbTk::AutoReloadHelper());
@@ -1021,7 +1019,7 @@ void BScreen::updateWorkspaceName(unsigned int w) {
     Workspace *space = getWorkspace(w);
     if (space) {
         m_workspace_names[w] = space->name();
-        updateWorkspaceNamesAtom();
+        m_workspacenames_sig.notify();
         Fluxbox::instance()->save_rc();
     }
 }
@@ -1030,10 +1028,6 @@ void BScreen::removeWorkspaceNames() {
     m_workspace_names.clear();
 }
 
-void BScreen::updateWorkspaceNamesAtom() {
-    m_workspacenames_sig.notify();
-}
-
 void BScreen::addIcon(FluxboxWindow *w) {
     if (w == 0)
         return;
@@ -1104,14 +1098,16 @@ void BScreen::removeClient(WinClient &client) {
 
 int BScreen::addWorkspace() {
 
-    bool save_name = getNameOfWorkspace(m_workspaces_list.size()) != "" ? false : true;
+    bool save_name = getNameOfWorkspace(m_workspaces_list.size()) == "";
     Workspace *wkspc = new Workspace(*this,
                                      getNameOfWorkspace(m_workspaces_list.size()),
                                      m_workspaces_list.size());
     m_workspaces_list.push_back(wkspc);
 
-    if (save_name)
+    if (save_name) {
         addWorkspaceName(wkspc->name().c_str()); //update names
+        m_workspacenames_sig.notify();
+    }
 
     saveWorkspaces(m_workspaces_list.size());
     workspaceCountSig().notify();
