@@ -22,8 +22,6 @@
 // FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
 // DEALINGS IN THE SOFTWARE.
 
-// $Id: Slit.cc 4967 2007-07-03 18:50:53Z markt $
-
 #include "Slit.hh"
 
 //use GNU extensions
@@ -36,34 +34,30 @@
 #endif // HAVE_CONFIG_H
 
 #include "Screen.hh"
-#include "ImageControl.hh"
-#include "RefCount.hh"
-#include "BoolMenuItem.hh"
-#include "EventManager.hh"
-#include "SimpleCommand.hh"
-#include "MacroCommand.hh"
+#include "FbTk/ImageControl.hh"
+#include "FbTk/RefCount.hh"
+#include "FbTk/EventManager.hh"
+#include "FbTk/SimpleCommand.hh"
+#include "FbTk/Theme.hh"
+#include "FbTk/Transparent.hh"
+#include "FbTk/MacroCommand.hh"
 #include "FbCommands.hh"
 #include "Layer.hh"
 #include "LayerMenu.hh"
-#include "XLayer.hh"
+#include "FbTk/XLayer.hh"
 #include "RootTheme.hh"
-#include "FbTk/Theme.hh"
 #include "FbMenu.hh"
-#include "Transparent.hh"
-#include "IntResMenuItem.hh"
-
-#ifdef XINERAMA
-#include "Xinerama.hh"
-#endif // XINERAMA
 
 #include "SlitTheme.hh"
 #include "SlitClient.hh"
 #include "Xutil.hh"
-#include "FbAtoms.hh"
 #include "FbTk/App.hh"
 #include "FbTk/MenuSeparator.hh"
 #include "FbTk/StringUtil.hh"
 #include "FbTk/I18n.hh"
+#include "FbTk/BoolMenuItem.hh"
+#include "FbTk/IntMenuItem.hh"
+#include "FbTk/RadioMenuItem.hh"
 
 #ifdef HAVE_SYS_STAT_H
 #include <sys/types.h>
@@ -71,7 +65,6 @@
 #endif // HAVE_SYS_STAT_H
 
 #include <X11/Xatom.h>
-#include <X11/Xlib.h>
 
 #include <iostream>
 #include <algorithm>
@@ -100,8 +93,7 @@ template<>
 void FbTk::Resource<Slit::Placement>::setFromString(const char *strval) {
     if (strcasecmp(strval, "TopLeft")==0)
         m_value = Slit::TOPLEFT;
-    else if (strcasecmp(strval, "LeftCenter")==0
-             || strcasecmp(strval, "CenterLeft")==0)
+    else if (strcasecmp(strval, "LeftCenter")==0)
         m_value = Slit::LEFTCENTER;
     else if (strcasecmp(strval, "BottomLeft")==0)
         m_value = Slit::BOTTOMLEFT;
@@ -111,21 +103,18 @@ void FbTk::Resource<Slit::Placement>::setFromString(const char *strval) {
         m_value = Slit::BOTTOMCENTER;
     else if (strcasecmp(strval, "TopRight")==0)
         m_value = Slit::TOPRIGHT;
-    else if (strcasecmp(strval, "RightCenter")==0
-             || strcasecmp(strval, "CenterRight")==0)
+    else if (strcasecmp(strval, "RightCenter")==0)
         m_value = Slit::RIGHTCENTER;
     else if (strcasecmp(strval, "BottomRight")==0)
         m_value = Slit::BOTTOMRIGHT;
-    else
-        setDefaultValue();
-}
-
-template<>
-void FbTk::Resource<Slit::Direction>::setFromString(const char *strval) {
-    if (strcasecmp(strval, "Vertical") == 0)
-        m_value = Slit::VERTICAL;
-    else if (strcasecmp(strval, "Horizontal") == 0)
-        m_value = Slit::HORIZONTAL;
+    else if (strcasecmp(strval, "LeftTop")==0)
+        m_value = Slit::LEFTTOP;
+    else if (strcasecmp(strval, "LeftBottom")==0)
+        m_value = Slit::LEFTBOTTOM;
+    else if (strcasecmp(strval, "RightTop")==0)
+        m_value = Slit::RIGHTTOP;
+    else if (strcasecmp(strval, "RightBottom")==0)
+        m_value = Slit::RIGHTBOTTOM;
     else
         setDefaultValue();
 }
@@ -157,34 +146,34 @@ string FbTk::Resource<Slit::Placement>::getString() const {
     case Slit::BOTTOMRIGHT:
         return string("BottomRight");
         break;
+    case Slit::LEFTTOP:
+        return string("LeftTop");
+        break;
+    case Slit::RIGHTTOP:
+        return string("RightTop");
+        break;
+    case Slit::LEFTBOTTOM:
+        return string("LeftBottom");
+        break;
+    case Slit::RIGHTBOTTOM:
+        return string("RightBottom");
+        break;
     }
     //default string
-    return string("BottomRight");
+    return string("RightBottom");
 }
 
-template<>
-string FbTk::Resource<Slit::Direction>::getString() const {
-    switch (m_value) {
-    case Slit::VERTICAL:
-        return string("Vertical");
-        break;
-    case Slit::HORIZONTAL:
-        return string("Horizontal");
-        break;
-    }
-    // default string
-    return string("Vertical");
-}
 } // end namespace FbTk
 namespace {
 
 class SlitClientMenuItem: public FbTk::MenuItem{
 public:
-    explicit SlitClientMenuItem(Slit& slit, SlitClient &client, FbTk::RefCount<FbTk::Command> &cmd):
+    explicit SlitClientMenuItem(Slit& slit, SlitClient &client, FbTk::RefCount<FbTk::Command<void> > &cmd):
         FbTk::MenuItem(client.matchName().c_str(), cmd), m_slit(slit), m_client(client) {
         setCommand(cmd);
         FbTk::MenuItem::setSelected(client.visible());
         setToggleItem(true);
+        setCloseOnClick(false);
     }
     const string &label() const {
         return m_client.matchName();
@@ -192,7 +181,7 @@ public:
     bool isSelected() const {
         return m_client.visible();
     }
-    void click(int button, int time) {
+    void click(int button, int time, unsigned int mods) {
         if (button == 4 || button == 2) { // wheel up
             m_slit.clientUp(&m_client);
         } else if (button == 5 || button == 3) { // wheel down
@@ -200,7 +189,7 @@ public:
         } else {
             m_client.setVisible(!m_client.visible());
             FbTk::MenuItem::setSelected(m_client.visible());
-            FbTk::MenuItem::click(button, time);
+            FbTk::MenuItem::click(button, time, mods);
         }
     }
 private:
@@ -208,50 +197,16 @@ private:
     SlitClient &m_client;
 };
 
-class SlitDirMenuItem: public FbTk::MenuItem {
+class PlaceSlitMenuItem: public FbTk::RadioMenuItem {
 public:
-    SlitDirMenuItem(const FbTk::FbString &label, Slit &slit, FbTk::RefCount<FbTk::Command> &cmd)
-        :FbTk::MenuItem(label,cmd),
-         m_slit(slit),
-         m_label(label) {
-        setLabel(m_label); // update label
+    PlaceSlitMenuItem(const FbTk::FbString &label, Slit &slit, Slit::Placement place, FbTk::RefCount<FbTk::Command<void> > &cmd):
+        FbTk::RadioMenuItem(label, cmd), m_slit(slit), m_place(place) {
+        setCloseOnClick(false);
     }
-
-    void click(int button, int time) {
-        // toggle direction
-        if (m_slit.direction() == Slit::HORIZONTAL)
-            m_slit.setDirection(Slit::VERTICAL);
-        else
-            m_slit.setDirection(Slit::HORIZONTAL);
-        setLabel(m_label);
-        FbTk::MenuItem::click(button, time);
-    }
-
-    void setLabel(const FbTk::FbString &label) {
-        _FB_USES_NLS;
-        m_label = (label);
-        string reallabel = m_label + " " +
-            ( m_slit.direction() == Slit::HORIZONTAL ?
-
-              _FB_XTEXT(Align, Horizontal, "Horizontal", "Horizontal"):
-              _FB_XTEXT(Align, Vertical,   "Vertical",   "Vertical"));
-        FbTk::MenuItem::setLabel(reallabel);
-    }
-private:
-    Slit &m_slit;
-    string m_label;
-};
-
-class PlaceSlitMenuItem: public FbTk::MenuItem {
-public:
-    PlaceSlitMenuItem(const FbTk::FbString &label, Slit &slit, Slit::Placement place, FbTk::RefCount<FbTk::Command> &cmd):
-        FbTk::MenuItem(label, cmd), m_slit(slit), m_place(place) {
-
-    }
-    bool isEnabled() const { return m_slit.placement() != m_place; }
-    void click(int button, int time) {
+    bool isSelected() const { return m_slit.placement() == m_place; }
+    void click(int button, int time, unsigned int mods) {
         m_slit.setPlacement(m_place);
-        FbTk::MenuItem::click(button, time);
+        FbTk::RadioMenuItem::click(button, time, mods);
     }
 private:
     Slit &m_slit;
@@ -272,6 +227,9 @@ Slit::Slit(BScreen &scr, FbTk::XLayer &layer, const char *filename)
       m_slitmenu(scr.menuTheme(),
                  scr.imageControl(),
                  *scr.layerManager().getLayer(Layer::MENU)),
+#ifdef XINERAMA
+      m_xineramaheadmenu(0),
+#endif // XINERAMA
       frame(scr.rootWindow()),
        //For KDE dock applets
       m_kwm1_dockwindow(XInternAtom(FbTk::App::instance()->display(),
@@ -285,15 +243,15 @@ Slit::Slit(BScreen &scr, FbTk::XLayer &layer, const char *filename)
       m_strut(0),
       // resources
       // lock in first resource
+      m_rc_kde_dockapp(scr.resourceManager(), true,
+                       scr.name() + ".slit.acceptKdeDockapps", scr.altName() + ".Slit.AcceptKdeDockapps"),
       m_rc_auto_hide(scr.resourceManager().lock(), false,
                      scr.name() + ".slit.autoHide", scr.altName() + ".Slit.AutoHide"),
       // TODO: this resource name must change
       m_rc_maximize_over(scr.resourceManager(), false,
                          scr.name() + ".slit.maxOver", scr.altName() + ".Slit.MaxOver"),
-      m_rc_placement(scr.resourceManager(), BOTTOMRIGHT,
+      m_rc_placement(scr.resourceManager(), RIGHTBOTTOM,
                      scr.name() + ".slit.placement", scr.altName() + ".Slit.Placement"),
-      m_rc_direction(scr.resourceManager(), VERTICAL,
-                     scr.name() + ".slit.direction", scr.altName() + ".Slit.Direction"),
       m_rc_alpha(scr.resourceManager(), 255,
                  scr.name() + ".slit.alpha", scr.altName() + ".Slit.Alpha"),
       m_rc_on_head(scr.resourceManager(), 0,
@@ -304,7 +262,7 @@ Slit::Slit(BScreen &scr, FbTk::XLayer &layer, const char *filename)
     _FB_USES_NLS;
 
     // attach to theme and root window change signal
-    m_slit_theme->reconfigSig().attach(this);
+    theme().reconfigSig().attach(this);
     scr.resizeSig().attach(this);
     scr.bgChangeSig().attach(this);
     scr.reconfigureSig().attach(this); // if alpha changed (we disablethis signal when we get theme change sig)
@@ -317,7 +275,7 @@ Slit::Slit(BScreen &scr, FbTk::XLayer &layer, const char *filename)
     // setup timer
     m_timer.setTimeout(200); // default timeout
     m_timer.fireOnce(true);
-    FbTk::RefCount<FbTk::Command> toggle_hidden(new FbTk::SimpleCommand<Slit>(*this, &Slit::toggleHidden));
+    FbTk::RefCount<FbTk::Command<void> > toggle_hidden(new FbTk::SimpleCommand<Slit>(*this, &Slit::toggleHidden));
     m_timer.setCommand(toggle_hidden);
 
 
@@ -386,46 +344,40 @@ void Slit::updateStrut() {
     int left = 0, right = 0, top = 0, bottom = 0;
     switch (placement()) {
     case TOPLEFT:
-        if (direction() == HORIZONTAL)
-            top = height() + 2 * bw;
-        else
-            left = width() + 2 * bw;
+        top = height() + 2 * bw;
+        break;
+    case LEFTTOP:
+        left = width() + 2 * bw;
         break;
     case TOPCENTER:
-        if (direction() == HORIZONTAL)
-            top = height() + 2 * bw;
+        top = height() + 2 * bw;
         break;
     case TOPRIGHT:
-        if (direction() == HORIZONTAL)
-            top = height() + 2 * bw;
-        else
-            right = width() + 2 * bw;
+        top = height() + 2 * bw;
+        break;
+    case RIGHTTOP:
+        right = width() + 2 * bw;
         break;
     case BOTTOMLEFT:
-        if (direction() == HORIZONTAL)
-            bottom = height() + 2 * bw;
-        else
-            left = width() + 2 * bw;
+        bottom = height() + 2 * bw;
+        break;
+    case LEFTBOTTOM:
+        left = width() + 2 * bw;
         break;
     case BOTTOMCENTER:
-        // would be strange to have it request size on vertical direction
-        // each time we add a client
-        if (direction() == HORIZONTAL)
-            bottom = height() + 2 * bw;
+        bottom = height() + 2 * bw;
         break;
     case BOTTOMRIGHT:
-        if (direction() == HORIZONTAL)
-            bottom = height() + 2 * bw;
-        else
-            right = width() + 2 * bw;
+        bottom = height() + 2 * bw;
+        break;
+    case RIGHTBOTTOM:
+        right = width() + 2 * bw;
         break;
     case LEFTCENTER:
-        if (direction() == VERTICAL)
-            left = width() + 2 * bw;
+        left = width() + 2 * bw;
         break;
     case RIGHTCENTER:
-        if (direction() == VERTICAL)
-            right = width() + 2 * bw;
+        right = width() + 2 * bw;
         break;
     }
 
@@ -441,6 +393,9 @@ void Slit::addClient(Window w) {
     if (w == None)
         return;
 
+    if (!acceptKdeDockapp() && screen().isKdeDockapp(w))
+        return;
+
     // Look for slot in client list by name
     SlitClient *client = 0;
     string match_name;
@@ -479,8 +434,7 @@ void Slit::addClient(Window w) {
     if (wmhints != 0) {
         if ((wmhints->flags & IconWindowHint) &&
             (wmhints->icon_window != None)) {
-            XMoveWindow(disp, client->clientWindow(), screen().width() + 10,
-                        screen().height() + 10);
+            XMoveWindow(disp, client->clientWindow(), -100, -100);
             XMapWindow(disp, client->clientWindow());
             client->setIconWindow(wmhints->icon_window);
             client->setWindow(client->iconWindow());
@@ -497,15 +451,9 @@ void Slit::addClient(Window w) {
 
     Atom *proto = 0;
     int num_return = 0;
-    FbAtoms *fbatoms = FbAtoms::instance();
 
     if (XGetWMProtocols(disp, w, &proto, &num_return)) {
 
-        for (int i = 0; i < num_return; ++i) {
-            if (proto[i] == fbatoms->getFluxboxStructureMessagesAtom())
-                screen().addNetizen(w);
-        }
-
         XFree((void *) proto);
 #ifdef DEBUG
     } else {
@@ -515,19 +463,12 @@ void Slit::addClient(Window w) {
 
     XWindowAttributes attrib;
 
-#ifdef KDE
     if (screen().isKdeDockapp(w))
         client->resize(24, 24);
-    else
-#endif // KDE
-
-        {
-            if (XGetWindowAttributes(disp, client->window(), &attrib)) {
-                client->resize(attrib.width, attrib.height);
-            } else { // set default size if we failed to get window attributes
-                client->resize(64, 64);
-            }
-        }
+    else if (XGetWindowAttributes(disp, client->window(), &attrib))
+        client->resize(attrib.width, attrib.height);
+    else // set default size if we failed to get window attributes
+        client->resize(64, 64);
 
     // disable border for client window
     XSetWindowBorderWidth(disp, client->window(), 0);
@@ -562,11 +503,6 @@ void Slit::addClient(Window w) {
 
 }
 
-void Slit::setDirection(Direction dir) {
-    *m_rc_direction = dir;
-    reconfigure();
-}
-
 void Slit::setPlacement(Placement place) {
     *m_rc_placement = place;
     reconfigure();
@@ -588,8 +524,6 @@ void Slit::removeClient(SlitClient *client, bool remap, bool destroy) {
     else // Clear the window info, but keep around to help future sorting?
         client->initialize();
 
-    screen().removeNetizen(client->window());
-
     if (remap && client->window() != 0) {
         Display *disp = FbTk::App::instance()->display();
 
@@ -646,14 +580,18 @@ void Slit::reconfigure() {
     // Need to count windows because not all client list entries
     // actually correspond to mapped windows.
     int num_windows = 0;
-    const int bevel_width = theme().bevelWidth();
+    const int bevel_width = theme()->bevelWidth();
     // determine width or height increase
     bool height_inc = false;
-    switch (direction()) {
-    case VERTICAL:
+    switch (placement()) {
+    case LEFTTOP:
+    case RIGHTTOP:
+    case LEFTCENTER:
+    case RIGHTCENTER:
+    case LEFTBOTTOM:
+    case RIGHTBOTTOM:
         height_inc = true;
-        break;
-    case HORIZONTAL: // already false
+    default:
         break;
     }
 
@@ -698,8 +636,8 @@ void Slit::reconfigure() {
 
     Display *disp = FbTk::App::instance()->display();
 
-    frame.window.setBorderWidth(theme().borderWidth());
-    frame.window.setBorderColor(theme().borderColor());
+    frame.window.setBorderWidth(theme()->borderWidth());
+    frame.window.setBorderColor(theme()->borderColor());
 
     Pixmap tmp = frame.pixmap;
     FbTk::ImageControl &image_ctrl = screen().imageControl();
@@ -737,19 +675,10 @@ void Slit::reconfigure() {
         show();
 
     int x = 0, y = 0;
-    height_inc = false;
-    switch (direction()) {
-    case VERTICAL:
-        x = 0;
+    if (height_inc)
         y = bevel_width;
-        height_inc = true;
-        break;
-
-    case HORIZONTAL:
+    else
         x = bevel_width;
-        y = 0;
-        break;
-    }
 
     client_it = m_client_list.begin();
     for (; client_it != client_it_end; ++client_it) {
@@ -829,8 +758,8 @@ void Slit::reposition() {
         head_h = screen().height();
     }
 
-    int border_width = theme().borderWidth();
-    int bevel_width = theme().bevelWidth();
+    int border_width = theme()->borderWidth();
+    int bevel_width = theme()->bevelWidth();
     // make sure at leaste one pixel is visible
     if (border_width >= bevel_width)
         bevel_width = border_width + 1;
@@ -839,90 +768,86 @@ void Slit::reposition() {
     case TOPLEFT:
         frame.x = head_x;
         frame.y = head_y;
-        if (direction() == VERTICAL) {
-            frame.x_hidden = bevel_width -
-                border_width - frame.width;
-            frame.y_hidden = head_y;
-        } else {
-            frame.x_hidden = head_x;
-            frame.y_hidden = bevel_width -
-                border_width - frame.height;
-        }
+        frame.x_hidden = head_x;
+        frame.y_hidden = bevel_width - border_width - frame.height;
+        break;
+
+    case LEFTTOP:
+        frame.x = head_x;
+        frame.y = head_y;
+        frame.x_hidden = bevel_width - border_width - frame.width;
+        frame.y_hidden = head_y;
         break;
 
     case LEFTCENTER:
         frame.x = head_x;
         frame.y = head_y + (head_h - frame.height) / 2;
-        frame.x_hidden = head_x + bevel_width -
-            border_width - frame.width;
+        frame.x_hidden = head_x + bevel_width - border_width - frame.width;
         frame.y_hidden = frame.y;
         break;
 
     case BOTTOMLEFT:
         frame.x = head_x;
         frame.y = head_y + head_h - frame.height - border_width*2;
-        if (direction() == VERTICAL) {
-            frame.x_hidden = head_x + bevel_width -
-                border_width - frame.width;
-            frame.y_hidden = frame.y;
-        } else {
-            frame.x_hidden = head_x;
-            frame.y_hidden = head_y + head_h -
-                bevel_width - border_width;
-        }
+        frame.x_hidden = head_x;
+        frame.y_hidden = head_y + head_h - bevel_width - border_width;
+        break;
+
+    case LEFTBOTTOM:
+        frame.x = head_x;
+        frame.y = head_y + head_h - frame.height - border_width*2;
+        frame.x_hidden = head_x + bevel_width - border_width - frame.width;
+        frame.y_hidden = frame.y;
         break;
 
     case TOPCENTER:
         frame.x = head_x + ((head_w - frame.width) / 2);
         frame.y = head_y;
         frame.x_hidden = frame.x;
-        frame.y_hidden = head_y + bevel_width -
-            border_width - frame.height;
+        frame.y_hidden = head_y + bevel_width - border_width - frame.height;
         break;
 
     case BOTTOMCENTER:
         frame.x = head_x + ((head_w - frame.width) / 2);
         frame.y = head_y + head_h - frame.height - border_width*2;
         frame.x_hidden = frame.x;
-        frame.y_hidden = head_y + head_h -
-            bevel_width - border_width;
+        frame.y_hidden = head_y + head_h - bevel_width - border_width;
         break;
 
     case TOPRIGHT:
         frame.x = head_x + head_w - frame.width - border_width*2;
         frame.y = head_y;
-        if (direction() == VERTICAL) {
-            frame.x_hidden = head_x + head_w -
-                bevel_width - border_width;
-            frame.y_hidden = head_y;
-        } else {
-            frame.x_hidden = frame.x;
-            frame.y_hidden = head_y + bevel_width -
-                border_width - frame.height;
-        }
+        frame.x_hidden = frame.x;
+        frame.y_hidden = head_y + bevel_width - border_width - frame.height;
+        break;
+
+    case RIGHTTOP:
+        frame.x = head_x + head_w - frame.width - border_width*2;
+        frame.y = head_y;
+        frame.x_hidden = head_x + head_w - bevel_width - border_width;
+        frame.y_hidden = head_y;
         break;
 
     case RIGHTCENTER:
-    default:
         frame.x = head_x + head_w - frame.width - border_width*2;
         frame.y = head_y + ((head_h - frame.height) / 2);
-        frame.x_hidden = head_x + head_w -
-            bevel_width - border_width;
+        frame.x_hidden = head_x + head_w - bevel_width - border_width;
         frame.y_hidden = frame.y;
         break;
 
     case BOTTOMRIGHT:
         frame.x = head_x + head_w - frame.width - border_width*2;
         frame.y = head_y + head_h - frame.height - border_width*2;
-        if (direction() == VERTICAL) {
-            frame.x_hidden = head_x + head_w -
-                bevel_width - border_width;
-            frame.y_hidden = frame.y;
-        } else {
-            frame.x_hidden = frame.x;
-            frame.y_hidden = head_y + head_h -
-                bevel_width - border_width;
-        }
+        frame.x_hidden = frame.x;
+        frame.y_hidden = head_y + head_h - bevel_width - border_width;
+        break;
+
+    case RIGHTBOTTOM:
+    default:
+        frame.x = head_x + head_w - frame.width - border_width*2;
+        frame.y = head_y + head_h - frame.height - border_width*2;
+        frame.x_hidden = head_x + head_w - bevel_width - border_width;
+        frame.y_hidden = frame.y;
         break;
     }
 
@@ -1104,7 +1029,7 @@ void Slit::configureRequestEvent(XConfigureRequestEvent &event) {
         if ((*it)->window() == event.window) {
             if ((*it)->width() != ((unsigned) event.width) ||
                 (*it)->height() != ((unsigned) event.height)) {
-                (*it)->resize(event.width, event.height);;
+                (*it)->resize(event.width, event.height);
 
                 reconf = true; //requires reconfiguration
 
@@ -1125,6 +1050,10 @@ void Slit::exposeEvent(XExposeEvent &ev) {
 
 void Slit::update(FbTk::Subject *subj) {
     reconfigure();
+#ifdef XINERAMA
+    if (subj == &m_screen.resizeSig() && m_xineramaheadmenu)
+        m_xineramaheadmenu->reloadHeads();
+#endif // XINERAMA
 }
 
 void Slit::clearWindow() {
@@ -1190,14 +1119,14 @@ void Slit::updateClientmenu() {
     m_clientlist_menu.removeAll();
     m_clientlist_menu.setLabel(_FB_XTEXT(Slit, ClientsMenu, "Clients", "Slit client menu"));
 
-    FbTk::RefCount<FbTk::Command> cycle_up(new FbTk::SimpleCommand<Slit>(*this, &Slit::cycleClientsUp));
-    FbTk::RefCount<FbTk::Command> cycle_down(new FbTk::SimpleCommand<Slit>(*this, &Slit::cycleClientsDown));
+    FbTk::RefCount<FbTk::Command<void> > cycle_up(new FbTk::SimpleCommand<Slit>(*this, &Slit::cycleClientsUp));
+    FbTk::RefCount<FbTk::Command<void> > cycle_down(new FbTk::SimpleCommand<Slit>(*this, &Slit::cycleClientsDown));
     m_clientlist_menu.insert(_FB_XTEXT(Slit, CycleUp, "Cycle Up", "Cycle clients upwards"), cycle_up);
     m_clientlist_menu.insert(_FB_XTEXT(Slit, CycleDown, "Cycle Down", "Cycle clients downwards"), cycle_down);
 
     m_clientlist_menu.insert(new FbTk::MenuSeparator());
 
-    FbTk::RefCount<FbTk::Command> reconfig(new FbTk::SimpleCommand<Slit>(*this, &Slit::reconfigure));
+    FbTk::RefCount<FbTk::Command<void> > reconfig(new FbTk::SimpleCommand<Slit>(*this, &Slit::reconfigure));
     SlitClients::iterator it = m_client_list.begin();
     for (; it != m_client_list.end(); ++it) {
         if ((*it) != 0 && (*it)->window() != 0)
@@ -1205,7 +1134,7 @@ void Slit::updateClientmenu() {
     }
 
     m_clientlist_menu.insert(new FbTk::MenuSeparator());
-    FbTk::RefCount<FbTk::Command> savecmd(new FbTk::SimpleCommand<Slit>(*this, &Slit::saveClientList));
+    FbTk::RefCount<FbTk::Command<void> > savecmd(new FbTk::SimpleCommand<Slit>(*this, &Slit::saveClientList));
     m_clientlist_menu.insert(_FB_XTEXT(Slit,
                                      SaveSlitList,
                                      "Save SlitList", "Saves the current order in the slit"),
@@ -1236,9 +1165,9 @@ void Slit::setupMenu() {
 
     FbTk::MacroCommand *s_a_reconf_macro = new FbTk::MacroCommand();
     FbTk::MacroCommand *s_a_reconf_slit_macro = new FbTk::MacroCommand();
-    FbTk::RefCount<FbTk::Command> saverc_cmd(new FbCommands::SaveResources());
-    FbTk::RefCount<FbTk::Command> reconf_cmd(new FbCommands::ReconfigureFluxboxCmd());
-    FbTk::RefCount<FbTk::Command> reconf_slit_cmd(new FbTk::SimpleCommand<Slit>(*this, &Slit::reconfigure));
+    FbTk::RefCount<FbTk::Command<void> > saverc_cmd(new FbCommands::SaveResources());
+    FbTk::RefCount<FbTk::Command<void> > reconf_cmd(new FbCommands::ReconfigureFluxboxCmd());
+    FbTk::RefCount<FbTk::Command<void> > reconf_slit_cmd(new FbTk::SimpleCommand<Slit>(*this, &Slit::reconfigure));
 
     s_a_reconf_macro->add(saverc_cmd);
     s_a_reconf_macro->add(reconf_cmd);
@@ -1246,8 +1175,8 @@ void Slit::setupMenu() {
     s_a_reconf_slit_macro->add(saverc_cmd);
     s_a_reconf_slit_macro->add(reconf_slit_cmd);
 
-    FbTk::RefCount<FbTk::Command> save_and_reconfigure(s_a_reconf_macro);
-    FbTk::RefCount<FbTk::Command> save_and_reconfigure_slit(s_a_reconf_slit_macro);
+    FbTk::RefCount<FbTk::Command<void> > save_and_reconfigure(s_a_reconf_macro);
+    FbTk::RefCount<FbTk::Command<void> > save_and_reconfigure_slit(s_a_reconf_slit_macro);
 
 
     // it'll be freed by the slitmenu (since not marked internal)
@@ -1266,6 +1195,7 @@ void Slit::setupMenu() {
 #ifdef XINERAMA
     if (screen().hasXinerama()) {
         m_slitmenu.insert(_FB_XTEXT(Menu, OnHead, "On Head...", "Title of On Head menu"),
+                          m_xineramaheadmenu =
                           new XineramaHeadMenu<Slit>(
                               screen().menuTheme(),
                               screen(),
@@ -1277,32 +1207,29 @@ void Slit::setupMenu() {
     }
 #endif //XINERAMA
 
-    m_slitmenu.insert(new BoolMenuItem(_FB_XTEXT(Common, AutoHide, "Auto hide", "This thing automatically hides when not close by"),
-                                       *m_rc_auto_hide,
+    m_slitmenu.insert(new FbTk::BoolMenuItem(_FB_XTEXT(Common, AutoHide, "Auto hide", "This thing automatically hides when not close by"),
+                                       m_rc_auto_hide,
                                        save_and_reconfigure_slit));
 
-    m_slitmenu.insert(new BoolMenuItem(_FB_XTEXT(Common, MaximizeOver,"Maximize Over", "Maximize over this thing when maximizing"),
-                                       *m_rc_maximize_over,
+    m_slitmenu.insert(new FbTk::BoolMenuItem(_FB_XTEXT(Common, MaximizeOver,"Maximize Over", "Maximize over this thing when maximizing"),
+                                       m_rc_maximize_over,
                                        save_and_reconfigure_slit));
 
     // this saves resources and clears the slit window to update alpha value
     FbTk::MenuItem *alpha_menuitem =
-        new IntResMenuItem< FbTk::Resource<int> >(_FB_XTEXT(Common, Alpha, "Alpha", "Transparency level"),
+        new FbTk::IntMenuItem(_FB_XTEXT(Common, Alpha, "Alpha", "Transparency level"),
                            m_rc_alpha,
                            0, 255, m_slitmenu);
     // setup command for alpha value
     MacroCommand *alpha_macrocmd = new MacroCommand();
-    RefCount<Command> alpha_cmd(new SimpleCommand<Slit>(*this, &Slit::updateAlpha));
+    RefCount<Command<void> > alpha_cmd(new SimpleCommand<Slit>(*this, &Slit::updateAlpha));
     alpha_macrocmd->add(saverc_cmd);
     alpha_macrocmd->add(alpha_cmd);
-    RefCount<Command> set_alpha_cmd(alpha_macrocmd);
+    RefCount<Command<void> > set_alpha_cmd(alpha_macrocmd);
     alpha_menuitem->setCommand(set_alpha_cmd);
 
     m_slitmenu.insert(alpha_menuitem);
 
-    m_slitmenu.insert(new SlitDirMenuItem(_FB_XTEXT(Slit, Direction, "Slit Direction", "Orientation of slit"),
-                                          *this,
-                                          save_and_reconfigure));
     m_slitmenu.insert(_FB_XTEXT(Slit, ClientsMenu, "Clients", "Slit client menu"), &m_clientlist_menu);
     m_slitmenu.updateMenu();
 
@@ -1319,18 +1246,24 @@ void Slit::setupMenu() {
 
     // menu is 3 wide, 5 down
     place_menu.push_back(PlacementP(_FB_XTEXT(Align, TopLeft, "Top Left", "Top Left"), Slit::TOPLEFT));
+    place_menu.push_back(PlacementP(_FB_XTEXT(Align, LeftTop, "Left Top", "Left Top"), Slit::LEFTTOP));
     place_menu.push_back(PlacementP(_FB_XTEXT(Align, LeftCenter, "Left Center", "Left Center"), Slit::LEFTCENTER));
+    place_menu.push_back(PlacementP(_FB_XTEXT(Align, LeftBottom, "Left Bottom", "Left Bottom"), Slit::LEFTBOTTOM));
     place_menu.push_back(PlacementP(_FB_XTEXT(Align, BottomLeft, "Bottom Left", "Bottom Left"), Slit::BOTTOMLEFT));
     place_menu.push_back(PlacementP(_FB_XTEXT(Align, TopCenter, "Top Center", "Top Center"), Slit::TOPCENTER));
     place_menu.push_back(PlacementP("", Slit::TOPLEFT));
+    place_menu.push_back(PlacementP("", Slit::TOPLEFT));
+    place_menu.push_back(PlacementP("", Slit::TOPLEFT));
     place_menu.push_back(PlacementP(_FB_XTEXT(Align, BottomCenter, "Bottom Center", "Bottom Center"), Slit::BOTTOMCENTER));
     place_menu.push_back(PlacementP(_FB_XTEXT(Align, TopRight, "Top Right", "Top Right"), Slit::TOPRIGHT));
+    place_menu.push_back(PlacementP(_FB_XTEXT(Align, RightTop, "Right Top", "Right Top"), Slit::RIGHTTOP));
     place_menu.push_back(PlacementP(_FB_XTEXT(Align, RightCenter, "Right Center", "Right Center"), Slit::RIGHTCENTER));
+    place_menu.push_back(PlacementP(_FB_XTEXT(Align, RightBottom, "Right Bottom", "Right Bottom"), Slit::RIGHTBOTTOM));
     place_menu.push_back(PlacementP(_FB_XTEXT(Align, BottomRight, "Bottom Right", "Bottom Right"), Slit::BOTTOMRIGHT));
 
 
     // create items in sub menu
-    for (size_t i=0; i<9; ++i) {
+    for (size_t i=0; i<15; ++i) {
         const FbTk::FbString &str = place_menu.front().first;
         Slit::Placement placement = place_menu.front().second;
 
