@@ -1,5 +1,5 @@
 // SlitClient.cc for fluxbox
-// Copyright (c) 2003 Henrik Kinnunen (fluxgen at users.sourceforge.net)
+// Copyright (c) 2003 - 2006 Henrik Kinnunen (fluxgen at fluxbox dot org)
 //
 // Permission is hereby granted, free of charge, to any person obtaining a
 // copy of this software and associated documentation files (the "Software"),
@@ -19,12 +19,10 @@
 // FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
 // DEALINGS IN THE SOFTWARE.
 
-// $Id: SlitClient.cc,v 1.3 2004/01/31 11:39:32 rathnor Exp $
-
 #include "SlitClient.hh"
 
 #include "Screen.hh"
-#include "App.hh"
+#include "FbTk/App.hh"
 #include "Xutil.hh"
 
 #include <X11/Xutil.h>
