@@ -1,5 +1,5 @@
 // ToolFactory.cc for Fluxbox
-// Copyright (c) 2003 - 2005 Henrik Kinnunen (fluxgen at fluxbox dot org)
+// Copyright (c) 2003 - 2006 Henrik Kinnunen (fluxgen at fluxbox dot org)
 //
 // Permission is hereby granted, free of charge, to any person obtaining a
 // copy of this software and associated documentation files (the "Software"),
@@ -19,8 +19,6 @@
 // FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
 // DEALINGS IN THE SOFTWARE.
 
-// $Id: ToolFactory.cc 3961 2005-04-27 15:15:36Z simonb $
-
 #include "ToolFactory.hh"
 
 // Tools
@@ -32,25 +30,21 @@
 #include "ArrowButton.hh"
 
 // Themes
-#include "IconbarTheme.hh"
 #include "WorkspaceNameTheme.hh"
 #include "ButtonTheme.hh"
 
-#include "CommandParser.hh"
+#include "FbTk/CommandParser.hh"
 #include "Screen.hh"
 #include "Toolbar.hh"
 #include "fluxbox.hh"
 
-#include "FbTk/FbWindow.hh"
-
 #include <utility>
 
 namespace {
-class ShowMenuAboveToolbar: public FbTk::Command {
+class ShowMenuAboveToolbar: public FbTk::Command<void> {
 public:
     explicit ShowMenuAboveToolbar(Toolbar &tbar):m_tbar(tbar) { }
     void execute() {
-        m_tbar.screen().hideMenus();
         // get last button pos
         const XEvent &event = Fluxbox::instance()->lastEvent();
         int head = m_tbar.screen().getHead(event.xbutton.x_root, event.xbutton.y_root);
@@ -65,7 +59,8 @@ public:
                                 m_tbar.screen().getHeadWidth(head),
                                 m_tbar.screen().getHeadHeight(head));
         m_tbar.menu().move(m.first, m.second);
-        m_tbar.menu().show();        
+        m_tbar.menu().show();
+        m_tbar.menu().grabInputFocus();
     }
 private:
     Toolbar &m_tbar;
@@ -77,10 +72,12 @@ ToolFactory::ToolFactory(BScreen &screen):m_screen(screen),
     m_clock_theme(screen.screenNumber(), "toolbar.clock", "Toolbar.Clock"),
     m_button_theme(new ButtonTheme(screen.screenNumber(), "toolbar.button", "Toolbar.Button", 
                                    "toolbar.clock", "Toolbar.Clock")),
+    m_workspace_theme(new WorkspaceNameTheme(screen.screenNumber(), "toolbar.workspace", "Toolbar.Workspace")),
     m_systray_theme(new ButtonTheme(screen.screenNumber(), "toolbar.systray", "Toolbar.Systray",
                                     "toolbar.clock", "Toolbar.Systray")),
-    m_workspace_theme(new WorkspaceNameTheme(screen.screenNumber(), "toolbar.workspace", "Toolbar.Workspace")),
-    m_iconbar_theme(screen.screenNumber(), "toolbar.iconbar", "Toolbar.Iconbar") {
+    m_iconbar_theme(screen.screenNumber(), "toolbar.iconbar", "Toolbar.Iconbar"),
+    m_focused_iconbar_theme(screen.screenNumber(), "toolbar.iconbar.focused", "Toolbar.Iconbar.Focused"),
+    m_unfocused_iconbar_theme(screen.screenNumber(), "toolbar.iconbar.unfocused", "Toolbar.Iconbar.Unfocused") {
 
 }
 
@@ -88,18 +85,18 @@ ToolbarItem *ToolFactory::create(const std::string &name, const FbTk::FbWindow &
     ToolbarItem * item = 0;
 
     unsigned int button_size = 24;
-    if (tbar.theme().buttonSize() > 0)
-        button_size = tbar.theme().buttonSize();
+    if (tbar.theme()->buttonSize() > 0)
+        button_size = tbar.theme()->buttonSize();
 
     if (name == "workspacename") {
         WorkspaceNameTool *witem = new WorkspaceNameTool(parent,
                                                         *m_workspace_theme, screen());
         using namespace FbTk;
-        RefCount<Command> showmenu(new ShowMenuAboveToolbar(tbar));
+        RefCount<Command<void> > showmenu(new ShowMenuAboveToolbar(tbar));
         witem->button().setOnClick(showmenu);
         item = witem;
     } else if (name == "iconbar") {
-        item = new IconbarTool(parent, m_iconbar_theme, screen(), tbar.menu());
+        item = new IconbarTool(parent, m_iconbar_theme, m_focused_iconbar_theme, m_unfocused_iconbar_theme, screen(), tbar.menu());
     } else if (name == "systemtray") {
         item = new SystemTray(parent, dynamic_cast<ButtonTheme &>(*m_systray_theme), screen());
     } else if (name == "clock") {
@@ -107,14 +104,14 @@ ToolbarItem *ToolFactory::create(const std::string &name, const FbTk::FbWindow &
     } else if (name == "nextworkspace" || 
                name == "prevworkspace") {
 
-        FbTk::RefCount<FbTk::Command> cmd(CommandParser::instance().parseLine(name));
+        FbTk::RefCount<FbTk::Command<void> > cmd(FbTk::CommandParser<void>::instance().parse(name));
         if (*cmd == 0) // we need a command
             return 0;
 
 		// TODO maybe direction of arrows should depend on toolbar layout ?
-        ArrowButton::Type arrow_type = ArrowButton::LEFT;
+        FbTk::FbDrawable::TriangleType arrow_type = FbTk::FbDrawable::LEFT;
         if (name == "nextworkspace")
-            arrow_type = ArrowButton::RIGHT;
+            arrow_type = FbTk::FbDrawable::RIGHT;
 
         ArrowButton *win = new ArrowButton(arrow_type, parent,
                                            0, 0,
@@ -124,16 +121,15 @@ ToolbarItem *ToolFactory::create(const std::string &name, const FbTk::FbWindow &
                               dynamic_cast<ButtonTheme &>(*m_button_theme),
                               screen().imageControl());
 
-    } else if (name == "nextwindow" || 
-               name == "prevwindow") {
+    } else {
 
-        FbTk::RefCount<FbTk::Command> cmd(CommandParser::instance().parseLine(name));
+        FbTk::RefCount<FbTk::Command<void> > cmd(FbTk::CommandParser<void>::instance().parse(name));
         if (*cmd == 0) // we need a command
             return 0;
 
-        ArrowButton::Type arrow_type = ArrowButton::LEFT;
+        FbTk::FbDrawable::TriangleType arrow_type = FbTk::FbDrawable::LEFT;
         if (name == "nextwindow")
-            arrow_type = ArrowButton::RIGHT;
+            arrow_type = FbTk::FbDrawable::RIGHT;
                     
         ArrowButton *win = new ArrowButton(arrow_type, parent,
                                            0, 0,
@@ -152,27 +148,24 @@ ToolbarItem *ToolFactory::create(const std::string &name, const FbTk::FbWindow &
 }
 
 void ToolFactory::updateThemes() {
-    m_clock_theme.setAntialias(screen().antialias());
     m_clock_theme.reconfigTheme();
-    m_iconbar_theme.setAntialias(screen().antialias());
-    m_iconbar_theme.reconfigTheme();
-    m_button_theme->setAntialias(screen().antialias());
+    m_focused_iconbar_theme.reconfigTheme();
+    m_unfocused_iconbar_theme.reconfigTheme();
     m_button_theme->reconfigTheme();
-    m_workspace_theme->setAntialias(screen().antialias());
     m_workspace_theme->reconfigTheme();
 }
 
 
-int ToolFactory::maxFontHeight() const {
+int ToolFactory::maxFontHeight() {
     unsigned int max_height = 0;
     if (max_height < m_clock_theme.font().height())
         max_height = m_clock_theme.font().height();
 
-    if (max_height < m_iconbar_theme.focusedText().font().height())
-        max_height = m_iconbar_theme.focusedText().font().height();
+    if (max_height < m_focused_iconbar_theme.text().font().height())
+        max_height = m_focused_iconbar_theme.text().font().height();
 
-    if (max_height < m_iconbar_theme.unfocusedText().font().height())
-        max_height = m_iconbar_theme.unfocusedText().font().height();
+    if (max_height < m_unfocused_iconbar_theme.text().font().height())
+        max_height = m_unfocused_iconbar_theme.text().font().height();
 
     if (max_height < m_workspace_theme->font().height())
         max_height = m_workspace_theme->font().height();
