@@ -19,8 +19,6 @@
 // FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
 // DEALINGS IN THE SOFTWARE.
 
-// $Id: FbRun.cc 4199 2006-02-16 06:53:05Z mathias $
-
 #include "FbRun.hh"
 
 #include "App.hh"
@@ -48,13 +46,15 @@
 #include <iterator>
 #include <fstream>
 #include <algorithm>
-#ifdef HAVE_CASSERT
-  #include <cassert>
-#else
-  #include <assert.h>
-#endif
 
-using namespace std;
+using std::cerr;
+using std::endl;
+using std::string;
+using std::fstream;
+using std::ifstream;
+using std::ofstream;
+using std::ios;
+
 FbRun::FbRun(int x, int y, size_t width):
     FbTk::TextBox(DefaultScreen(FbTk::App::instance()->display()),
                   m_font, ""),
@@ -64,9 +64,10 @@ FbRun::FbRun(int x, int y, size_t width):
     m_gc(*this),
     m_end(false),
     m_current_history_item(0),
+    m_last_completion_prefix(""),
     m_current_apps_item(0),
     m_cursor(XCreateFontCursor(FbTk::App::instance()->display(), XC_xterm)) {
-    
+
     setGC(m_gc.gc());
     setCursor(m_cursor);
     // setting nomaximize in local resize
@@ -76,17 +77,17 @@ FbRun::FbRun(int x, int y, size_t width):
     XClassHint *class_hint = XAllocClassHint();
     if (class_hint == 0)
         throw string("Out of memory");
-    class_hint->res_name = "fbrun";
-    class_hint->res_class = "FbRun";    
+    class_hint->res_name = const_cast<char *>("fbrun");
+    class_hint->res_class = const_cast<char *>("FbRun");
     XSetClassHint(m_display, window(), class_hint);
-    
+
     XFree(class_hint);
 #ifdef HAVE_XPM
     Pixmap mask = 0;
     Pixmap pm;
     XpmCreatePixmapFromData(m_display,
                             window(),
-                            fbrun_xpm,
+                            const_cast<char **>(fbrun_xpm),
                             &pm,
                             &mask,
                             0); // attribs
@@ -116,13 +117,18 @@ void FbRun::run(const std::string &command) {
 
     // fork and execute program
     if (!fork()) {
+
+        const char *shell = getenv("SHELL");
+        if (!shell)
+            shell = "/bin/sh";
+
         setsid();
-        execl("/bin/sh", "/bin/sh", "-c", command.c_str(), static_cast<void*>(NULL));
+        execl(shell, shell, "-c", command.c_str(), static_cast<void*>(NULL));
         exit(0); //exit child
     }
 
     hide(); // hide gui
-    
+
     // save command history to file
     if (text().size() != 0) { // no need to save empty command
 
@@ -133,7 +139,7 @@ void FbRun::run(const std::string &command) {
             // m_current_history_item is the duplicate
         } else {
             m_current_history_item = 0;
-            for (; m_current_history_item < m_history.size(); 
+            for (; m_current_history_item < m_history.size();
                  ++m_current_history_item) {
                 if (m_history[m_current_history_item] == text())
                     break;
@@ -145,7 +151,7 @@ void FbRun::run(const std::string &command) {
             // now m_current_history_item points at the duplicate, or
             // at m_history.size() if no duplicate
             if (m_current_history_item != m_history.size()) {
-                int i = 0;
+                unsigned int i = 0;
                 // read past history items before current
                 for (; inoutfile.good() && i < m_current_history_item; i++)
                     inoutfile.ignore(1, '\n');
@@ -153,7 +159,7 @@ void FbRun::run(const std::string &command) {
                 // write the history items that come after current
                 for (i++; i < m_history.size(); i++)
                     inoutfile<<m_history[i]<<endl;
-                
+
             } else {
                 // set put-pointer at end of file
                 inoutfile.seekp(0, ios::end);
@@ -214,8 +220,7 @@ void FbRun::setTitle(const string &title) {
 }
 
 void FbRun::resize(unsigned int width, unsigned int height) {
-    FbTk::TextBox::resize(width, height);    
-    setNoMaximize();
+    FbTk::TextBox::resize(width, height);
 }
 
 void FbRun::redrawLabel() {
@@ -223,38 +228,43 @@ void FbRun::redrawLabel() {
 }
 
 void FbRun::keyPressEvent(XKeyEvent &ke) {
-    
+    // reset last completion prefix if we don't do a tab completion thing
+    bool did_tab_complete = false;
+
     ke.state = FbTk::KeyUtil::instance().cleanMods(ke.state);
 
-    int cp= cursorPosition();
     FbTk::TextBox::keyPressEvent(ke);
     KeySym ks;
     char keychar[1];
     XLookupString(&ke, keychar, 1, &ks, 0);
     // a modifier key by itself doesn't do anything
-    if (IsModifierKey(ks)) 
+    if (IsModifierKey(ks))
         return;
 
     if (FbTk::KeyUtil::instance().isolateModifierMask(ke.state)) { // a modifier key is down
         if ((ke.state & ControlMask) == ControlMask) {
             switch (ks) {
             case XK_p:
+                did_tab_complete = true;
                 prevHistoryItem();
                 break;
             case XK_n:
+                did_tab_complete = true;
                 nextHistoryItem();
                 break;
             case XK_Tab:
+                did_tab_complete = true;
                 tabCompleteHistory();
-                setCursorPosition(cp);
                 break;
             }
         } else if ((ke.state & (Mod1Mask|ShiftMask)) == (Mod1Mask | ShiftMask)) {
             switch (ks) {
             case XK_less:
+                did_tab_complete = true;
                 firstHistoryItem();
                 break;
             case XK_greater:
+                did_tab_complete = true;
                 lastHistoryItem();
                 break;
             }
@@ -277,15 +287,17 @@ void FbRun::keyPressEvent(XKeyEvent &ke) {
             nextHistoryItem();
             break;
         case XK_Tab:
+            did_tab_complete = true;
             tabCompleteApps();
-            setCursorPosition(cp);
             break;
         }
     }
     clear();
+    if (!did_tab_complete)
+        m_last_completion_prefix = "";
 }
 
-void FbRun::setNoMaximize() {
+void FbRun::lockPosition(bool size_too) {
     // we don't need to maximize this window
     XSizeHints sh;
     sh.flags = PMaxSize | PMinSize;
@@ -293,6 +305,11 @@ void FbRun::setNoMaximize() {
     sh.max_height = height();
     sh.min_width = width();
     sh.min_height = height();
+    if (size_too) {
+        sh.flags |= USPosition;
+        sh.x = x();
+        sh.y = y();
+    }
     XSetWMNormalHints(m_display, window(), &sh);
 }
 
@@ -342,16 +359,19 @@ void FbRun::tabCompleteHistory() {
         XBell(m_display, 0);
     } else {
         unsigned int nr= 0;
-        int history_item = m_current_history_item - 1;
-        string prefix = text().substr(0, textStartPos() + cursorPosition());
+        unsigned int history_item = m_current_history_item - 1;
+        if (m_last_completion_prefix.empty())
+            m_last_completion_prefix = text().substr(0, textStartPos() + cursorPosition());
+
         while (history_item != m_current_history_item && nr++ < m_history.size()) {
-            if (history_item <= -1 )
-                history_item= m_history.size() - 1;
-            if (m_history[history_item].find(prefix) == 0) {
+            if (m_history[history_item].find(m_last_completion_prefix) == 0) {
                 m_current_history_item = history_item;
                 setText(m_history[m_current_history_item]);
+                cursorEnd();
                 break;
             }
+            if (history_item == 0) // loop
+                history_item = m_history.size();
             history_item--;
         }
         if (history_item == m_current_history_item) XBell(m_display, 0);
@@ -359,31 +379,34 @@ void FbRun::tabCompleteHistory() {
 }
 
 void FbRun::tabCompleteApps() {
-  
+
     static bool first_run= true;
-    static string saved_prefix= "";
-    string prefix= text().substr(0, textStartPos() + cursorPosition());
+    if (m_last_completion_prefix.empty())
+        m_last_completion_prefix = text().substr(0, textStartPos() + cursorPosition());
+    string prefix = m_last_completion_prefix;
     FbTk::Directory dir;
 
     bool add_dirs= false;
     bool changed_prefix= false;
 
     // (re)build m_apps-container
-    if (first_run || saved_prefix != prefix) {
+    if (first_run || m_last_completion_prefix != prefix) {
         first_run= false;
-        
+
         string path;
-        
-        if(!prefix.empty() && 
+
+        if(!prefix.empty() &&
             string("/.~").find_first_of(prefix[0]) != string::npos) {
             size_t rseparator= prefix.find_last_of("/");
             path= prefix.substr(0, rseparator + 1) +  ":";
             add_dirs= true;
-        } else
-            path= getenv("PATH");
-
+        } else {
+            char* tmp_path = getenv("PATH");
+            if (tmp_path)
+                path = tmp_path;
+        }
         m_apps.clear();
-        
+
         unsigned int l;
         unsigned int r;
 
@@ -396,27 +419,27 @@ void FbRun::tabCompleteApps() {
                 if (n >= 0) {
                     while(n--) {
                         filename= dir.readFilename();
-                        fncomplete= dir.name() + 
-                                    (*dir.name().rbegin() != '/' ? "/" : "") + 
+                        fncomplete= dir.name() +
+                                    (*dir.name().rbegin() != '/' ? "/" : "") +
                                     filename;
 
                         // directories in dirmode ?
                         if (add_dirs && FbTk::FileUtil::isDirectory(fncomplete.c_str()) &&
                             filename != ".." && filename != ".") {
-                            m_apps.push_back(fncomplete); 
+                            m_apps.push_back(fncomplete);
                         // executables in dirmode ?
-                        } else if (add_dirs && FbTk::FileUtil::isRegularFile(fncomplete.c_str()) && 
-                                   FbTk::FileUtil::isExecutable(fncomplete.c_str()) && 
-                                   (prefix == "" || 
+                        } else if (add_dirs && FbTk::FileUtil::isRegularFile(fncomplete.c_str()) &&
+                                   FbTk::FileUtil::isExecutable(fncomplete.c_str()) &&
+                                   (prefix == "" ||
                                     fncomplete.substr(0, prefix.size()) == prefix)) {
                             m_apps.push_back(fncomplete);
                         // executables in $PATH ?
-                        } else if (FbTk::FileUtil::isRegularFile(fncomplete.c_str()) && 
-                                   FbTk::FileUtil::isExecutable(fncomplete.c_str()) && 
-                                   (prefix == "" || 
+                        } else if (FbTk::FileUtil::isRegularFile(fncomplete.c_str()) &&
+                                   FbTk::FileUtil::isExecutable(fncomplete.c_str()) &&
+                                   (prefix == "" ||
                                     filename.substr(0, prefix.size()) == prefix)) {
                             m_apps.push_back(filename);
-                        } 
+                        }
                     }
                 }
                 l= r + 1;
@@ -426,7 +449,7 @@ void FbRun::tabCompleteApps() {
         sort(m_apps.begin(), m_apps.end());
         unique(m_apps.begin(), m_apps.end());
 
-        saved_prefix= prefix;
+        m_last_completion_prefix = prefix;
         changed_prefix= true;
         m_current_apps_item= 0;
     }
@@ -452,11 +475,12 @@ void FbRun::tabCompleteApps() {
                     setText(m_apps[m_current_apps_item] +  "/");
                 else
                     setText(m_apps[m_current_apps_item]);
+                cursorEnd();
                 break;
             }
             apps_item++;
         }
-        if (!changed_prefix && apps_item == m_current_apps_item) 
+        if (!changed_prefix && apps_item == m_current_apps_item)
             XBell(m_display, 0);
     }
 }
