@@ -19,8 +19,6 @@
 // FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
 // DEALINGS IN THE SOFTWARE.
 
-// $Id: main.cc 4199 2006-02-16 06:53:05Z mathias $
-
 #include "FbRun.hh"
 #include "App.hh"
 #include "StringUtil.hh"
@@ -34,8 +32,20 @@ extern  "C" {
 
 #include <string>
 #include <iostream>
+#ifdef HAVE_CSTRING
+  #include <cstring>
+#else
+  #include <string.h>
+#endif
+#ifdef HAVE_CSTDLIB
+  #include <cstdlib>
+#else
+  #include <stdlib.h>
+#endif
 
-using namespace std;
+using std::cerr;
+using std::endl;
+using std::string;
 
 void showUsage(const char *progname) {
     cerr<<"fbrun 1.5 : (c) 2002-2004 Henrik Kinnunen"<<endl;
@@ -81,7 +91,7 @@ int main(int argc, char **argv) {
         } else if (strcmp(argv[i], "-text") == 0 && i+1 < argc) {
             text = argv[++i];
         } else if (strcmp(argv[i], "-w") == 0 && i+1 < argc) {
-            width = atoi(argv[++i]);			
+            width = atoi(argv[++i]);
             set_width = true;
         } else if (strcmp(argv[i], "-h") == 0 && i+1 < argc) {
             height = atoi(argv[++i]);
@@ -115,7 +125,7 @@ int main(int argc, char **argv) {
     }
 
     try {
-		
+
         FbTk::App application(display_name.c_str());
         FbRun fbrun;
 
@@ -131,7 +141,7 @@ int main(int argc, char **argv) {
         // get color
         FbTk::Color fg_color(foreground.c_str(), 0);
         FbTk::Color bg_color(background.c_str(), 0);
-		
+
         fbrun.setForegroundColor(fg_color);
         fbrun.setBackgroundColor(bg_color);
 
@@ -148,17 +158,17 @@ int main(int argc, char **argv) {
 
         fbrun.setTitle(title);
         fbrun.setText(text);
-	
+
         if (near_mouse) {
-            
+
             int wx, wy;
             unsigned int mask;
             Window ret_win;
             Window child_win;
-            
+
             Display* dpy = FbTk::App::instance()->display();
-            
-            if (XQueryPointer(dpy, DefaultRootWindow(dpy), 
+
+            if (XQueryPointer(dpy, DefaultRootWindow(dpy),
                               &ret_win, &child_win,
                               &x, &y, &wx, &wy, &mask)) {
 
@@ -202,10 +212,12 @@ int main(int argc, char **argv) {
                     y = root_y + root_h - fbrun.height();
             }
         }
-        
+
         if (set_pos)
             fbrun.move(x, y);
-		
+
+        fbrun.lockPosition(set_pos);
+
         fbrun.show();
 
         application.eventLoop();
